/*
 * This software program is licensed subject to the GNU General Public License
 * (GPL).Version 2,June 1991, available at http://www.fsf.org/copyleft/gpl.html

 * (C) Copyright 2011 Bosch Sensortec GmbH
 * All Rights Reserved
 */

/**********************************************************************
* Modified by
* (C) NEC CASIO Mobile Communications, Ltd. 2011
**********************************************************************/












#include <linux/i2c/smb380.h>
#include <linux/i2c/BMA150calib.h>
#include <linux/kernel.h>



int bma150_calibrate(smb380acc_t orientation, smb380acc_t *offset)
{
	return bma150_calibration(orientation,offset,BMA150_CALIBRATION_MAX_TRIES);
}


int bma150_readCalib(smb380acc_t *offset)
{
	int comres;
	comres = smb380_set_ee_w(1);
	comres |= smb380_get_offset(0, &(offset->x));
    comres |= smb380_get_offset(1, &(offset->y));
    comres |= smb380_get_offset(2, &(offset->z));
	comres |= smb380_set_ee_w(0);
	return comres;
}


int bma150_writeCalib(smb380acc_t *offset,int EEPROM)
{
	int comres;

	comres = smb380_set_ee_w(1);
	if (EEPROM)
	{
		comres |= smb380_set_offset_eeprom(0, offset->x);
		comres |= smb380_set_offset_eeprom(1, offset->y);
		comres |= smb380_set_offset_eeprom(2, offset->z);
	}
	else
	{
		comres = smb380_set_offset(0, offset->x);
		comres |= smb380_set_offset(1, offset->y);
		comres |= smb380_set_offset(2, offset->z);
	}
	comres |= smb380_set_ee_w(0);
	return comres;
}


int bma150_calibration(smb380acc_t orientation, smb380acc_t *offset, int tries)
{

	short offset_x, offset_y, offset_z;
	int need_calibration=0, min_max_ok=0;
	int error_flag=0;
	int iteration_counter =0;
	int dummy;

	
	unsigned char dummy2;
	short min_accel_adapt_x = 0xFE00, min_accel_adapt_y = 0xFE00, min_accel_adapt_z = 0xFE00;
	short min_offset_x = 0, min_offset_y = 0, min_offset_z = 0; 
	short min_reached_x	= 0, min_reached_y	= 0, min_reached_z	= 0;
	short prev_steps_x = 0x01FF, prev_steps_y = 0x01FF, prev_steps_z = 0x01FF;


	smb380acc_t min,max,avg,gain,step;

    smb380_set_range(SMB380_RANGE_2G);
    smb380_set_bandwidth(SMB380_BW_25HZ); 
	smb380_set_ee_w(1);
	smb380_get_offset(0, &offset_x);
    smb380_get_offset(1, &offset_y);
    smb380_get_offset(2, &offset_z);
	

	smb380_read_reg(0x16, &dummy2, 1);
	gain.x = dummy2 & 0x3F;
	smb380_read_reg(0x17, &dummy2, 1);
	gain.y = dummy2 & 0x3F;
	smb380_read_reg(0x18, &dummy2, 1);
	gain.z = dummy2 & 0x3F;
	smb380_set_ee_w(0);

	step.x = gain.x * 15/64 + 7;
	step.y = gain.y * 15/64 + 7;
	step.z = gain.z * 15/64 + 7;

	smb380_pause(50);

	do {

		bma150_read_accel_avg(10, &min, &max, &avg);

		min_max_ok = bma150_verify_min_max(min, max, avg);

		if (!min_max_ok)
			return CALIB_ERR_MOV;

		need_calibration = 0;

		
		dummy = bma150_calc_new_offset(orientation.x, avg.x, &offset_x, &step.x, &min_accel_adapt_x, &min_offset_x, &min_reached_x, &prev_steps_x, iteration_counter);
		smb380_set_ee_w(1);
		if (dummy & 0x01)
		{					
			smb380_set_offset(0, offset_x);
			need_calibration = CALIB_X_AXIS;
		}
		if (dummy & 0x10)
			error_flag |= (CALIB_X_AXIS << 4);

		
		
		dummy = bma150_calc_new_offset(orientation.y, avg.y, &offset_y, &step.y, &min_accel_adapt_y, &min_offset_y, &min_reached_y, &prev_steps_y, iteration_counter);
		if (dummy & 0x01)
		{
			smb380_set_offset(1, offset_y);
			need_calibration |= CALIB_Y_AXIS;
		}
		if (dummy & 0x10)
			error_flag |= (CALIB_Y_AXIS << 4);

		
		
		dummy = bma150_calc_new_offset(orientation.z, avg.z, &offset_z, &step.z, &min_accel_adapt_z, &min_offset_z, &min_reached_z, &prev_steps_z, iteration_counter);
		if (dummy & 0x01)
		{
			smb380_set_offset(2, offset_z);
			need_calibration |= CALIB_Z_AXIS;
		}
		if (dummy & 0x10)
			error_flag |= (CALIB_Z_AXIS << 4);
		

		smb380_set_ee_w(0);
		iteration_counter++;

		if (need_calibration)
		{
			smb380_pause(50);
   		}
    } while (need_calibration && (iteration_counter != tries));

	offset->x = offset_x;
	offset->y = offset_y;
	offset->z = offset_z;
	
	if(iteration_counter == 1)
	{
		error_flag |= NO_CALIB;
		return error_flag;
	}
	else if (iteration_counter == tries)
	{
		error_flag |= need_calibration;
		return error_flag;
	}
	
	return NO_ERR;
}


int bma150_calc_new_offset(short orientation, short accel, short *offset, short *stepsize, short *min_accel_adapt, short *min_offset, short *min_reached, short *prev_steps, int iteration_counter)
{
	short old_offset;
	short new_offset;
	short accel_adapt;



	short steps = 0;


   	unsigned char  calibrated = 0;

   	old_offset = *offset;
   
   	accel_adapt = accel - (orientation * 256);
	if(accel_adapt < -512)
	{
	 	accel_adapt = -512;
	}
	else if(accel_adapt > 511)
	{
	 	accel_adapt = 511;
	}
		                                
   	if (((accel_adapt > 7) || (accel_adapt < -7)) && !(*min_reached))
   	{
		if (abs(accel_adapt) <= abs(*min_accel_adapt))
		{
			if(((3*(*prev_steps) * (*stepsize)) <= 2*(abs((*min_accel_adapt) - (accel_adapt)))) && (iteration_counter >= 1))
			{
				(*stepsize) = (abs((*min_accel_adapt) - (accel_adapt))) / (*prev_steps);
			}
						
			if ((accel_adapt < (*stepsize)) && (accel_adapt > 0))
				new_offset = old_offset -1;          
		 	else if ((accel_adapt > -(*stepsize)) && (accel_adapt < 0))    
		   		new_offset = old_offset +1;
	     	else
			{
				steps = (accel_adapt/(*stepsize));
				if((2*(abs(steps * (*stepsize) - accel_adapt))) > (*stepsize))
				{
					if(accel_adapt < 0)
						steps-=1;
					else
						steps+=1;	
				} 
	       		new_offset = old_offset - steps;
			}
	     	
			if (new_offset < 0)
			{
				new_offset = 0;
				calibrated = 0x10;
			}
	     	else if (new_offset > 1023)
		 	{
				new_offset = 1023;
				calibrated = 0x10;
		 	}

			*prev_steps = abs(steps);
			if(*prev_steps==0)
				*prev_steps = 1;
						
			*min_accel_adapt = accel_adapt;
			*min_offset = old_offset;

			*offset = new_offset;
		}
		else
		{
			*offset = *min_offset;
			if((2*(*prev_steps) * (*stepsize)) <= (abs((*min_accel_adapt) - (accel_adapt))))
			{
				(*stepsize) = (abs((*min_accel_adapt) - (accel_adapt))) / (*prev_steps);
			}
			else
			{
				*min_reached = 0x01;
			}	
		}
					    
	 	calibrated |= 0x01;
   	}
  	return calibrated;
}


int bma150_read_accel_avg(int num_avg, smb380acc_t *min, smb380acc_t *max, smb380acc_t *avg )
{
   	long x_avg=0, y_avg=0, z_avg=0;   
   	int comres=0;
   	int i;
   	smb380acc_t accel;

   	x_avg = 0; y_avg=0; z_avg=0;                  
   	max->x = -512; max->y =-512; max->z = -512;
   	min->x = 512;  min->y = 512; min->z = 512;  

	for (i=0; i<num_avg; i++) {
		comres += smb380_read_accel_xyz(&accel);
		if (accel.x > max->x)
			max->x = accel.x;
		if (accel.x < min->x) 
			min->x = accel.x;

		if (accel.y > max->y)
			max->y = accel.y;
		if (accel.y < min->y) 
			min->y = accel.y;

		if (accel.z > max->z)
			max->z = accel.z;
		if (accel.z < min->z) 
			min->z = accel.z;
		
		x_avg += accel.x;
		y_avg += accel.y;
		z_avg += accel.z;

		smb380_pause(10);
	}
	avg->x = (x_avg / num_avg);
	avg->y = (y_avg / num_avg);
	avg->z = (z_avg / num_avg);
	return comres;
}
	 


int bma150_verify_min_max(smb380acc_t min, smb380acc_t max, smb380acc_t avg) 
{
	short dx, dy, dz;
	int ver_ok=1;

	dx =  ((max.x) - (min.x));
	dy =  ((max.y) - (min.y));
	dz =  ((max.z) - (min.z));


	if ((dx> 10) || (dx<-10)) 
	  ver_ok = 0;
    if ((dy> 10) || (dy<-10)) 
	  ver_ok = 0;
	if ((dz> 10) || (dz<-10)) 
	  ver_ok = 0;

	return ver_ok;
}

	

