/* Copyright (c) 2009-2010, Code Aurora Forum. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 and
 * only version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 *
 */

/**********************************************************************
* Modified by
* (C) NEC CASIO Mobile Communications, Ltd. 2011
**********************************************************************/






















#include <linux/kernel.h>
#include <mach/camera.h>
#include "camcmp_cmdmaker_mt9e311.h"





#define CAMCMP_POLL_TIME      10            
#define CAMCMP_POLL_RETRY     500           


#define CAMCMP_WRITE_BIN_MAX	0x0800



















extern camcmp_makecmd_type g_camcmp_mt9e311_cmd_list[];




extern uint16_t* g_p_camcmp_mt9e311_ver;

























static int32_t camcmp_mt9e311_write_single(unsigned short addr, unsigned int val, unsigned short size)
{
	int32_t rc = 0;
	int32_t i = 0;
	unsigned char buf[6];

	memset(buf, 0x00, sizeof(buf));

	
	buf[0] = (addr & 0xFF00) >> 8;
	buf[1] = (addr & 0x00FF);

	
	for (i = size; i > 0; i--) {
		buf[i+1] = ((val >> ((size-i) * 8)) & 0x000000FF);
	}
	CAMCMP_LOG_INF("i2c write addr,val,size: W,0x%02X%02X,0x%08X,%d\n", buf[0], buf[1], val, size);


	
	rc = camcmp_i2c_transfer(buf, (size+2), NULL, 0);
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_i2c_transfer failed! rc=%d\n", rc);
		return rc;
	}

	return 0;
}




















static int32_t camcmp_mt9e311_write_sequential(unsigned short addr, unsigned short* p_data, unsigned short size)
{
	int32_t i, w_size, total, cnt;
	int32_t rc = 0;
	unsigned char buf[254];

	
	if (p_data == NULL) {
		CAMCMP_LOG_ERR("input NULL!\n");
		return -EINVAL;
	}
	CAMCMP_LOG_INF("sequential write all: 0x%04X,%d\n", addr, (size*2));
	total = size;

	
	do {
		
		buf[0] = (addr & 0xFF00) >> 8;
		buf[1] = (addr & 0x00FF);

		
		w_size = 126;
		if (total < 126) {
			w_size = total;
		}

		CAMCMP_LOG_INF("sequential write: 0x%04X,%d\n", ((buf[0] << 8) + buf[1]), (w_size*2));

		for (i = 0; i < w_size; i++) {
			cnt = 2*i+2;
			buf[cnt]   = (*p_data & 0xFF00) >> 8;
			buf[cnt+1] = (*p_data & 0x00FF);

			CAMCMP_LOG_INF("i2c write addr,val,size: W,0x%04X,0x0000%02X%02X,%d\n", addr, buf[cnt], buf[cnt+1], 2);


			
			addr+=2;
			p_data++;
		}

		
		rc = camcmp_i2c_transfer(buf, (w_size*2+2), NULL, 0);
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_i2c_transfer failed! addr=0x%04X rc=%d\n", ((buf[0] << 8) + buf[1]), rc);
			return rc;
		}

		
		total -= 126;
	} while (total > 0);

	return 0;
}


















static int32_t camcmp_mt9e311_write_single_nolog(unsigned short addr, unsigned int val, unsigned short size, camcmp_cmd_kind_enum cmd_kind)
{
	int32_t rc = 0;
	int32_t i = 0;
	unsigned char buf[6];

	memset(buf, 0x00, sizeof(buf));

	
	buf[0] = (addr & 0xFF00) >> 8;
	buf[1] = (addr & 0x00FF);

	
	if ( ( cmd_kind == CAMCMP_REG_WRITE_FW_NOINV ) )  {
        
        memcpy(&(buf[2]), &val, size);
    } else {
        
        for (i = size; i > 0; i--) {
        	buf[i+1] = ((val >> ((size-i) * 8)) & 0x000000FF);
        }
    }
	
	



	
	rc = camcmp_i2c_transfer(buf, (size+2), NULL, 0);
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_i2c_transfer failed! rc=%d\n", rc);
		return rc;
	}

	return 0;
}




















static int32_t camcmp_mt9e311_write_sequential_nolog(unsigned short addr, unsigned short* p_data, unsigned short size, camcmp_cmd_kind_enum cmd_kind)
{
	int32_t i, w_size, total, cnt;
	int32_t rc = 0;
	unsigned char buf[254];

	
	if (p_data == NULL) {
		CAMCMP_LOG_ERR("input NULL!\n");
		return -EINVAL;
	}



	total = size;

	
	do {
		
		buf[0] = (addr & 0xFF00) >> 8;
		buf[1] = (addr & 0x00FF);

		
		w_size = 126;
		if (total < 126) {
			w_size = total;
		}





        if ((cmd_kind == CAMCMP_REG_WRITE_FW_NOINV) ||
            (cmd_kind == CAMCMP_REG_BINARY_FW_NOINV) ) {
            
    		for (i = 0; i < w_size; i++) {
    			cnt = 2*i+2;
    			memcpy(&(buf[cnt]), p_data, sizeof(*p_data));
    			



    			
    			addr+=2;
    			p_data++;
    		}
        } else {
    		
    		for (i = 0; i < w_size; i++) {
    			cnt = 2*i+2;
    			buf[cnt]   = (*p_data & 0xFF00) >> 8;
    			buf[cnt+1] = (*p_data & 0x00FF);
 		


    			
    			addr+=2;
    			p_data++;
    	    }
    	}

		
		rc = camcmp_i2c_transfer(buf, (w_size*2+2), NULL, 0);
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_i2c_transfer failed! addr=0x%04X rc=%d\n", ((buf[0] << 8) + buf[1]), rc);
			return rc;
		}

		
		total -= 126;
	} while (total > 0);

	return 0;
}




















static int32_t camcmp_mt9e311_read(unsigned short addr, unsigned short size, uint32_t* p_rev)
{
	int32_t rc = 0;
	int32_t i = 0;
	unsigned char w_buf[2];
	unsigned char r_buf[4];

	
	if (p_rev == NULL) {
		CAMCMP_LOG_ERR("input NULL!\n");
		return -EINVAL;
	}

	
	if ( (size > CAMCMP_D_LEN_LWORD) || (size == 0) ) {
		CAMCMP_LOG_ERR("input size failed! size=%d", size);
		return -EINVAL;
	}

	
	w_buf[0] = (addr & 0xFF00) >> 8;
	w_buf[1] = (addr & 0x00FF);

	
	memset(r_buf, 0x00, sizeof(r_buf));

	
	rc = camcmp_i2c_transfer(w_buf, sizeof(w_buf), r_buf, size);
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_i2c_transfer failed! rc=%d\n", rc);
		return rc;
	}

	
	*p_rev = 0;
	for (i = size; i > 0; i--) {
		*p_rev = *p_rev | (r_buf[i-1] << ((size-i)*8));
	}

	CAMCMP_LOG_INF("i2c read  addr,val,size: R,0x%04X,0x%08X,%d\n", addr, *p_rev, size);

	return 0;
}


















static int32_t camcmp_mt9e311_load_binary(camcmp_mt9e311_reg_val_type* p_reg)
{
	int32_t cnt = 0;
	int32_t reg_addr, w_size;
	unsigned short *p_reg_val;
	int32_t rc = 0;
	camcmp_mt9e311_reg_val_type *p_info;

	
	if ( p_reg == NULL ) {
		CAMCMP_LOG_ERR("input NULL! reg=%p \n", p_reg);
		return -EINVAL;
	}

	
	reg_addr = p_reg->lwdata;

	
	do {
		
		p_info = p_reg;

		
		rc = camcmp_mt9e311_write_single(p_info->waddr, reg_addr, p_info->size);
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_mt9e311_write_single failed! rc=%d\n", rc);
			return rc;
		}

		
		p_info++;
		if (p_info->p_data == NULL) {
			CAMCMP_LOG_ERR("not Binary!\n");
			return -EINVAL;
		}
		p_reg_val = p_info->p_data + (CAMCMP_WRITE_BIN_MAX * cnt);

		
		w_size = CAMCMP_WRITE_BIN_MAX;
		if (p_info->size < (CAMCMP_WRITE_BIN_MAX * (cnt+1))) {
			w_size = p_info->size - (CAMCMP_WRITE_BIN_MAX * cnt);
		}

		
		rc = camcmp_mt9e311_write_sequential(p_info->waddr, p_reg_val, w_size);
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_mt9e311_write_single failed! rc=%d\n", rc);
			return rc;
		}
		reg_addr += (w_size * 2);
		cnt++;
	} while ( p_info->size > (CAMCMP_WRITE_BIN_MAX * cnt) );

	return 0;
}


















static int32_t camcmp_mt9e311_load_binary_nolog(camcmp_mt9e311_reg_val_type* p_reg)
{
	int32_t cnt = 0;
	int32_t reg_addr, w_size;
	unsigned short *p_reg_val;
	int32_t rc = 0;
	camcmp_mt9e311_reg_val_type *p_info;

	
	if ( p_reg == NULL ) {
		CAMCMP_LOG_ERR("input NULL! reg=%p \n", p_reg);
		return -EINVAL;
	}

	
	reg_addr = p_reg->lwdata;

	
	do {
		
		p_info = p_reg;

		
		rc = camcmp_mt9e311_write_single(p_info->waddr, reg_addr, p_info->size);
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_mt9e311_write_single failed! rc=%d\n", rc);
			return rc;
		}

		
		p_info++;
		if (p_info->p_data == NULL) {
			CAMCMP_LOG_ERR("not Binary!\n");
			return -EINVAL;
		}
		p_reg_val = p_info->p_data + (CAMCMP_WRITE_BIN_MAX * cnt);

		
		w_size = CAMCMP_WRITE_BIN_MAX;
		if (p_info->size < (CAMCMP_WRITE_BIN_MAX * (cnt+1))) {
			w_size = p_info->size - (CAMCMP_WRITE_BIN_MAX * cnt);
		}

		
		rc = camcmp_mt9e311_write_sequential_nolog(p_info->waddr, p_reg_val, w_size, p_info->cmd_kind);
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_mt9e311_write_sequential_nolog failed! rc=%d\n", rc);
			return rc;
		}
		reg_addr += (w_size * 2);
		cnt++;
	} while ( p_info->size > (CAMCMP_WRITE_BIN_MAX * cnt) );

	return 0;
}





















static int32_t camcmp_mt9e311_write_or(unsigned short addr, unsigned int val, unsigned short size)
{
	int32_t rc = 0;
	uint32_t cur_val;

	
	rc = camcmp_mt9e311_read(addr, size, &cur_val);
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_mt9e311_read failed! addr=0x%04X rc=%d\n", addr, rc);
		return rc;
	}

	
	cur_val = cur_val | val;

	
	rc = camcmp_mt9e311_write_single(addr, cur_val, size);
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_mt9e311_write_single failed! rc=%d\n", rc);
		return rc;
	}

	return 0;
}





















static int32_t camcmp_mt9e311_write_and(unsigned short addr, unsigned int val, unsigned short size)
{
	int32_t rc = 0;
	uint32_t cur_val;

	
	rc = camcmp_mt9e311_read(addr, size, &cur_val);
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_mt9e311_read failed! addr=0x%04X rc=%d\n", addr, rc);
		return rc;
	}

	
	cur_val = cur_val & val;

	
	rc = camcmp_mt9e311_write_single(addr, cur_val, size);
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_mt9e311_write_single failed! rc=%d\n", rc);
		return rc;
	}

	return 0;
}





















static int32_t camcmp_mt9e311_write_plus(unsigned short addr, unsigned int val, unsigned short size)
{
	int32_t rc = 0;
	uint32_t cur_val;

	
	rc = camcmp_mt9e311_read(addr, size, &cur_val);
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_mt9e311_read failed! addr=0x%04X rc=%d\n", addr, rc);
		return rc;
	}

	
	cur_val = cur_val + val;

	
	rc = camcmp_mt9e311_write_single(addr, cur_val, size);
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_mt9e311_write_single failed! rc=%d\n", rc);
		return rc;
	}

	return 0;
}




















static int32_t camcmp_mt9e311_polling(camcmp_mt9e311_reg_val_type* p_reg, int32_t num)
{
	int32_t i, j;
	int32_t rc = 0;
	uint32_t val;
	camcmp_mt9e311_reg_val_type reg_val;

	
	if ( (p_reg == NULL) || (num == 0) ) {
		CAMCMP_LOG_ERR("input NULL! reg=%p, num=%d \n", p_reg, num);
		return -EINVAL;
	}

	
	for (i = 0; i < CAMCMP_POLL_RETRY; i++) {
		
		for (j = 0; j < num; j++) {
			reg_val = *(p_reg + j);

			
			rc = camcmp_mt9e311_read(reg_val.waddr, reg_val.size, &val);
			if (rc < 0) {
				CAMCMP_LOG_ERR("camcmp_mt9e311_read failed! addr=0x%04X rc=%d\n", reg_val.waddr, rc);
				return rc;
			}

			
			
			if ( (reg_val.cmd_kind == CAMCMP_REG_POLL1) &&
			     ((val & reg_val.lwdata) == reg_val.lwdata) ) {
				return j;
			
			} else if ( (reg_val.cmd_kind == CAMCMP_REG_POLL2) &&
			     ((~val & reg_val.lwdata) == reg_val.lwdata) ) {
				return j;
			
			} else if ((reg_val.cmd_kind == CAMCMP_REG_POLL3) && (val == reg_val.lwdata)) {
				return j;
			
			} else if ((reg_val.cmd_kind == CAMCMP_REG_POLL4) && (val != reg_val.lwdata)) {
				return j;
			
			} else {
				
			}
		}

		CAMCMP_LOG_INF("i2c polling wait time=%d\n", CAMCMP_POLL_TIME);


		
		msleep(CAMCMP_POLL_TIME);
	}

	
	CAMCMP_LOG_ERR("polling time out! \n");
	return -ETIMEDOUT;
}




















static int32_t camcmp_mt9e311_set_blank(camcmp_data_type blank_info, void* p_in_data)
{
	int32_t i;
	camcmp_mt9e311_blank_val_type *p_blank;  
	unsigned short *p_w  = NULL;             
	unsigned int   *p_lw = NULL;             
	uint32_t       val = 0;                  

	
	if ( (blank_info.p_data == NULL) || (blank_info.size == 0) ) {
		CAMCMP_LOG_ERR("input NULL! pinf=0x%08X, num=%d\n", (int32_t)blank_info.p_data, blank_info.size);
		return -EINVAL;
	}

	
	p_blank = (camcmp_mt9e311_blank_val_type*)blank_info.p_data;

	
	if (p_in_data != NULL) {
		
		if (blank_info.size > 1) {
			CAMCMP_LOG_ERR("input size failed! size=%d", blank_info.size);
			return -EINVAL;
		}

		
		val = *((uint32_t*)(p_in_data));
	}

	
	for (i = 0; i < blank_info.size; i++) {
		
		switch (p_blank->size) {
		
		case CAMCMP_D_LEN_WORD:
			p_w = (unsigned short*)p_blank->p_copy;
			if (p_in_data == NULL) {
				*p_w = (unsigned short)p_blank->org_val;
			} else {
				*p_w = (unsigned short)val;
			}
			break;

		
		case CAMCMP_D_LEN_LWORD:
			p_lw = (unsigned int*)p_blank->p_copy;
			if (p_in_data == NULL) {
				*p_lw = p_blank->org_val;
			} else {
				*p_lw = val;
			}
			break;

		
		default:
			CAMCMP_LOG_ERR("blank table size failed! size=%d", p_blank->size);
			return -EINVAL;
		}

		
		p_blank++;
	}

	return 0;
}





















static int32_t camcmp_mt9e311_send_cmd(camcmp_data_type send_cmd, void* p_rev, uint32_t* p_rev_num, uint32_t max_num)
{
	int32_t rc = 0;
	int32_t i;
	int32_t wait_msec, wait_usec;
	uint32_t                    *p_rev_data;    
	camcmp_mt9e311_reg_val_type *p_info;        


	
	if ( (send_cmd.p_data == NULL) || (send_cmd.size == 0) ) {
		CAMCMP_LOG_ERR("input NULL! pinf=0x%08X, num=%d\n", (int32_t)send_cmd.p_data, send_cmd.size);
		return -EINVAL;
	}

	
	p_info = (camcmp_mt9e311_reg_val_type*)send_cmd.p_data;

	
	for (i = 0; i < send_cmd.size; i++) {
		
		switch (p_info->cmd_kind) {
		
		case CAMCMP_REG_WRITE:
			
			if (p_info->size > CAMCMP_D_LEN_LWORD) {
				rc = camcmp_mt9e311_write_sequential(p_info->waddr, p_info->p_data, p_info->size);
			
			} else {
				rc = camcmp_mt9e311_write_single(p_info->waddr, p_info->lwdata, p_info->size);
			}
			if (rc < 0) {
				CAMCMP_LOG_ERR("camcmp_mt9e311_write failed! size=%d rc=%d\n", p_info->size, rc);
				return rc;
			}
			break;

		
		case CAMCMP_REG_READ:
			
			if ( (p_rev == NULL) || (p_rev_num == NULL) ) {
				CAMCMP_LOG_ERR("input NULL! rev=%p, rev_num=%p\n", p_rev, p_rev_num);
				return -EINVAL;
			}

			
			if (*p_rev_num > max_num) {
				CAMCMP_LOG_ERR("recieve size failed! num=%d\n", *p_rev_num);
		 		return -EINVAL;
			}

			
			p_rev_data = ((uint32_t*)p_rev) + *p_rev_num;
			rc = camcmp_mt9e311_read(p_info->waddr, p_info->size, p_rev_data);
			if (rc < 0) {
				CAMCMP_LOG_ERR("camcmp_mt9e311_read failed! rc=%d\n", rc);
				return rc;
			}
			(*p_rev_num)++;
			break;

		
		case CAMCMP_REG_BINARY:
			rc = camcmp_mt9e311_load_binary(p_info);
			if (rc < 0) {
				CAMCMP_LOG_ERR("camcmp_mt9e311_write failed! size=%d rc=%d\n", p_info->size, rc);
			}
			return rc;

		
		case CAMCMP_REG_OR:
			rc = camcmp_mt9e311_write_or(p_info->waddr, p_info->lwdata, p_info->size);
			if (rc < 0) {
				CAMCMP_LOG_ERR("camcmp_mt9e311_write_or failed! rc=%d\n", rc);
			}
			break;

		
		case CAMCMP_REG_AND:
			rc = camcmp_mt9e311_write_and(p_info->waddr, p_info->lwdata, p_info->size);
			if (rc < 0) {
				CAMCMP_LOG_ERR("camcmp_mt9e311_write_and failed! rc=%d\n", rc);
			}
			break;

		
		case CAMCMP_REG_PLUS:
			rc = camcmp_mt9e311_write_plus(p_info->waddr, p_info->lwdata, p_info->size);
			if (rc < 0) {
				CAMCMP_LOG_ERR("camcmp_mt9e311_write_plus failed! rc=%d\n", rc);
			}
			break;

		
		case CAMCMP_REG_WAIT:
			
			wait_msec = p_info->lwdata / 1000;
			if (wait_msec) {
				CAMCMP_LOG_INF("i2c wait %dms\n", wait_msec);

				msleep(wait_msec);
			}

			
			wait_usec = p_info->lwdata % 1000;
			if (wait_usec) {
				CAMCMP_LOG_INF("i2c wait %dus\n", wait_usec);
				udelay(wait_usec);
			}
			break;

		
		case CAMCMP_REG_POLL1:
		
		case CAMCMP_REG_POLL2:
		
		case CAMCMP_REG_POLL3:
		
		case CAMCMP_REG_POLL4:
			rc = camcmp_mt9e311_polling(p_info, 1);
			if (rc < 0) {
				CAMCMP_LOG_ERR("camcmp_mt9e311_polling failed! rc=%d\n", rc);
				return rc;
			}
			break;


















		
		case CAMCMP_REG_WRITE_FW_NOINV:
		case CAMCMP_REG_WRITE_FW_INV:
			
			if (p_info->size > CAMCMP_D_LEN_LWORD) {
				rc = camcmp_mt9e311_write_sequential_nolog(p_info->waddr, p_info->p_data, p_info->size, p_info->cmd_kind);
			
			} else {
				rc = camcmp_mt9e311_write_single_nolog(p_info->waddr, p_info->lwdata, p_info->size, p_info->cmd_kind);
			}
			if (rc < 0) {
				CAMCMP_LOG_ERR("camcmp_mt9e311_write_nolog failed! size=%d rc=%d\n", p_info->size, rc);
				return rc;
			}
			break;

		
		case CAMCMP_REG_BINARY_FW_NOINV:
		case CAMCMP_REG_BINARY_FW_INV:
			rc = camcmp_mt9e311_load_binary_nolog(p_info);
			if (rc < 0) {
				CAMCMP_LOG_ERR("camcmp_mt9e311_load_binary_nolog failed! size=%d rc=%d\n", p_info->size, rc);
			}
			return rc;

		default:
			CAMCMP_LOG_ERR("cmd kind failed! kind=%d\n", p_info->cmd_kind);
			
			return -EFAULT;
		}

		
		p_info++;
	}

	return 0;
}





















static camcmp_makecmd_type* camcmp_mt9e311_make_cmd(void)
{
	CAMCMP_LOG_DBG("camcmp_mt9e311_make_cmd\n");
	return g_camcmp_mt9e311_cmd_list;
}

















static int32_t camcmp_mt9e311_correct_cmd(void* p_val, uint32_t num)
{
	return 0;
}















static void camcmp_mt9e311_get_version(uint16_t* p_ver, uint8_t* p_flg)
{
	
	if ( (p_ver == NULL) || (p_flg == NULL) ) {
		CAMCMP_LOG_ERR("input NULL! ver=%p, flg=%p\n", p_ver, p_flg);
		return;
	}

	*p_ver = *g_p_camcmp_mt9e311_ver;
	*p_flg = 0;
}

















void camcmp_mt9e311_get_info(camcmp_cmdmaker_func_type* p_func)
{
	CAMCMP_LOG_DBG("camcmp_mt9e311_get_info\n");
	p_func->make_cmd    = camcmp_mt9e311_make_cmd;          
	p_func->correct_cmd = camcmp_mt9e311_correct_cmd;       
	p_func->get_version = camcmp_mt9e311_get_version;       
	p_func->set_blank   = camcmp_mt9e311_set_blank;         
	p_func->send_cmd    = camcmp_mt9e311_send_cmd;          
}



