/* Copyright (c) 2009-2010, Code Aurora Forum. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 and
 * only version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 *
 */

/**********************************************************************
* Modified by
* (C) NEC CASIO Mobile Communications, Ltd. 2011
**********************************************************************/


















































#include <linux/delay.h>
#include <linux/types.h>
#include <linux/i2c.h>
#include <linux/uaccess.h>
#include <linux/miscdevice.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <media/msm_camera.h>
#include <mach/gpio.h>
#include <mach/vreg.h>
#include <mach/camera.h>
#include <mach/msm_battery.h>
#include "mt9e311.h"
#include "camcmp_cmdsender.h"
#include "camcmp_read_regfile.h"	
#include "camcmp_mt9e311_setting.h"
#include "camcmp_cmdmaker_mt9e311.h"




#define RAWSNAPSHOT_ENABLE			1			




#define MT9E311_CHECK_STATE_WAITMAX		800			
#define MT9E311_CHECK_STATE_WAIT		1			


#define MT9E311_WAIT_1					1			
#define MT9E311_WAIT_5					5			
#define MT9E311_WAIT_10					10			
#define MT9E311_WAIT_20					20			
#define MT9E311_WAIT_50					50			


#define MT9E311_GPIO_CAM_CLK_EN			19			
#define MT9E311_GPIO_CAM_STBY			25			
#define MT9E311_GPIO_VDD_CAM_EN4		31			 
#define MT9E311_GPIO_VDD_CAM_EN2		34			
#define MT9E311_GPIO_VDD_CAM_EN3		35			


#define MT9E311_SYS_REVISION_01			( 0x0003 )
#define MT9E311_SYS_REVISION_02			( 0x0203 )


















































#define MT9E311_DIAG_SUCCESS            0
#define MT9E311_DIAG_ACT_ERR            1
#define MT9E311_DIAG_PARAM_ERR          2
#define MT9E311_DIAG_STATE_ERR          3

#define MOVIE_CONTINUOUS_AF
static int mt9e311_set_af_mode(struct msm_sync *sync,struct af_mode_cfg *af_mode_chg);

static int mt9e311_set_default_focus(int prev_mode);



static int mt9e311_af_start(struct msm_sync *sync);
static int mt9e311_get_af_pos(struct get_af_pos_cfg *af_pos);
static int mt9e311_get_moni_gain(struct get_monigain_cfg *moni_gain);
static int mt9e311_get_reg_ver(struct get_reg_ver_cfg *get_reg_ver);
static int mt9e311_set_wb(int8_t wb);
static int mt9e311_set_jpeg_quality(int8_t jpeg_quality);


static int mt9e311_afc_start(void);
static int mt9e311_afc_stop(void);


static struct timer_list timer;						
static uint16_t	 sys_rev;							


static int mt9e311_poweroff(void);
static void mt9e311_probe_init_done(const struct msm_camera_sensor_info *data);





struct mt9e311_work {
	struct work_struct work;
};
static struct mt9e311_work *mt9e311_sensorw;		
static struct i2c_client *mt9e311_client;			




struct mt9e311_ctrl {
	const struct msm_camera_sensor_info *sensordata;	

	int sensormode;										
	int af_mode;										
	int prev_res;										
	uint8_t af_cancel_onoff;							
	uint8_t af_exec_flag;								
	uint16_t init_curr_lens_pos;						

	uint8_t ae_mode;									
	uint8_t ae_mode_next;								
	struct exp_gain_cfg prev_exp_gain;					
	struct exp_gain_cfg cap_exp_gain;					


	int8_t afc_mode;									
	int8_t inf_over_search;								

};
static struct mt9e311_ctrl *mt9e311_ctrl;				

static DECLARE_WAIT_QUEUE_HEAD(mt9e311_wait_queue);
DEFINE_MUTEX(mt9e311_mut);

static uint32_t af_start_after_cnt = 0;					




 


static int mt9e311_af_stop(void);
static int i2c_read_write_modify( int r_cid, int w_cid, uint32_t mask, uint32_t data );
static int i2c_write_modify( uint32_t read_data, int w_cid, uint32_t mask, uint32_t data );
static uint16_t scene_mode_rd_to_set( uint16_t scene_mode );
static int32_t mt9e311_set_scene_tbl(int8_t scene);






















static int mt9e311_set_exposure_gain( int mode )
{
	int rc = 0;
	camcmp_cmd_info_type cmd_info[]  = {{0, NULL, 0, NULL, 0}};
	camcmp_blank_info_type blk_info[] = {{0, NULL}, {0, NULL}};
	uint32_t exp_time, gain;

	CAMCMP_LOG_DBG("ae_mode=%x \n", mt9e311_ctrl->ae_mode_next);

	
	if ( (mt9e311_ctrl->ae_mode_next == CAMERA_AE_MODE_AUTO) &&
		 (mt9e311_ctrl->ae_mode == mt9e311_ctrl->ae_mode_next) ) {

		CAMCMP_LOG_DBG("ae_mode not change\n");
		return 0;
	}

	
	if (mt9e311_ctrl->ae_mode_next == CAMERA_AE_MODE_AUTO) {
		
		cmd_info[0].cmd_id = CAMCMP_CID_SET_AE_AUTO;

	
	} else {
		
		switch (mode)
		{
			
			case SENSOR_PREVIEW_MODE:
			case SENSOR_MOVIE_MODE:
				gain = mt9e311_ctrl->prev_exp_gain.gain;
				exp_time = mt9e311_ctrl->prev_exp_gain.line;
				break;

			
			case SENSOR_SNAPSHOT_MODE:
			case SENSOR_RAW_SNAPSHOT_MODE:
				gain = mt9e311_ctrl->cap_exp_gain.gain;
				exp_time = mt9e311_ctrl->cap_exp_gain.line;
				break;

			default:
				CAMCMP_LOG_ERR("input failed! mode=%d\n", mode);
				
				return -EINVAL;
		}

		CAMCMP_LOG_DBG("agc_gain=%x \n", gain);
		CAMCMP_LOG_DBG("exposure_time=%x \n", exp_time);

		
		blk_info[0].blank_id     = CAMCMP_BID_AE_GAIN;
		blk_info[0].p_blank_data = &gain;
		blk_info[1].blank_id     = CAMCMP_BID_AE_EXPOSURE;
		blk_info[1].p_blank_data = &exp_time;

		
		cmd_info[0].cmd_id = CAMCMP_CID_SET_AE_MANUAL;
		cmd_info[0].p_blank_info = blk_info;
		cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(blk_info);
	}

	
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
		return -EFAULT;
	}

	
	mt9e311_ctrl->ae_mode = mt9e311_ctrl->ae_mode_next;

	return rc;
}

















static int mt9e311_set_sensor_reg_read( struct regread_param *param )
{
    int rc = 0;
	camcmp_cmd_info_type cmd_info_src;
	camcmp_blank_info_type blank_info_src;

	camcmp_cmd_info_type cmd_info_dst;

	camcmp_cmd_info_type cmd_info_size;
	camcmp_blank_info_type blank_info_size;

	camcmp_cmd_info_type cmd_info_ctrl;
	
	camcmp_cmd_info_type cmd_info_value;

	uint32_t src_addr;
	uint16_t reg_value;
	
	uint32_t i2cID = 0x6c;
	int aw;
	int dw;
	uint32_t addr;
	
	








		aw = 1;
		addr = ( 0x00000000 | (param->read_addr_3 << 8 ) | param->read_addr_4 );
    







	
	if ( param->read_byte == 1 ) {
		dw = 0;
	} else {
		dw = 1;
	}
	
	CAMCMP_LOG_DBG("i2cID=%d \n", i2cID);
	CAMCMP_LOG_DBG("Address With=%d \n", aw);
	CAMCMP_LOG_DBG("Data With=%d \n", dw);
	CAMCMP_LOG_DBG("Address=0x%08X \n", addr);

	
	memset(&cmd_info_src,   0, sizeof(cmd_info_src));
	memset(&blank_info_src, 0, sizeof(blank_info_src));
	memset(&cmd_info_dst,   0, sizeof(cmd_info_dst));
	memset(&cmd_info_size,  0, sizeof(cmd_info_size));
	memset(&blank_info_size,0, sizeof(blank_info_size));
	memset(&cmd_info_ctrl,  0, sizeof(cmd_info_ctrl));
	memset(&cmd_info_value, 0, sizeof(cmd_info_value));
	
	
	src_addr = ((((((dw << 1) | aw) << 8) | i2cID) << 16) | addr );
	
	cmd_info_src.cmd_id             = CAMCMP_CID_SET_REGREAD_SRC;
		blank_info_src.p_blank_data = &src_addr;
	cmd_info_src.p_blank_info       = &blank_info_src;
	cmd_info_src.blank_num          = CAMCMP_SIZE_BLKINF(blank_info_src);
	rc = camcmp_isp_trans( &cmd_info_src,  CAMCMP_SIZE_CMD(cmd_info_src));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}
	
	
	cmd_info_dst.cmd_id = CAMCMP_CID_SET_REGREAD_DST;
	
	rc = camcmp_isp_trans(&cmd_info_dst, CAMCMP_SIZE_CMD(cmd_info_dst));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}

	
	if ( dw == 0 ) {
		blank_info_size.blank_id = CAMCMP_BID_REGREAD_SIZE_8;
	} else {
		blank_info_size.blank_id = CAMCMP_BID_REGREAD_SIZE_16;
	}
	cmd_info_size.cmd_id = CAMCMP_CID_SET_REGREAD_SIZE;
	cmd_info_size.p_blank_info = &blank_info_size;
	cmd_info_size.blank_num = CAMCMP_SIZE_BLKINF(blank_info_size);
	
	rc = camcmp_isp_trans(&cmd_info_size, CAMCMP_SIZE_CMD(cmd_info_size));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}
	
	
	cmd_info_ctrl.cmd_id = CAMCMP_CID_SET_REGREAD_CTRL;
	
	rc = camcmp_isp_trans(&cmd_info_ctrl, CAMCMP_SIZE_CMD(cmd_info_ctrl));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}
	

	
	cmd_info_value.cmd_id = CAMCMP_CID_GET_REGREAD_VALUE;
	cmd_info_value.p_recieve_data = &reg_value;
	cmd_info_value.recieve_num = 1;

	
	rc = camcmp_isp_trans(&cmd_info_value, CAMCMP_SIZE_CMD(cmd_info_value));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}
	
	param->read_byte_add = param->read_byte;
	param->read_data_1 = 0x00;
	param->read_data_2 = 0x00;
	if ( dw == 0 ) {
		
		param->read_data_3 = 0x00;
		param->read_data_4 = 0x00 | ((reg_value & 0xFF00) >> 8 );
	} else {
		
		param->read_data_3 = 0x00 | ((reg_value & 0xFF00) >> 8 );
		param->read_data_4 = 0x00 | ((reg_value & 0x00FF));
	}
	
	CAMCMP_LOG_DBG("reg_value=0x%04X \n"  , reg_value);
	CAMCMP_LOG_DBG("read_byte_add=0x%02X \n", param->read_byte_add);
	CAMCMP_LOG_DBG("read_data_1=0x%02X \n", param->read_data_1);
	CAMCMP_LOG_DBG("read_data_2=0x%02X \n", param->read_data_2);
	CAMCMP_LOG_DBG("read_data_3=0x%02X \n", param->read_data_3);
	CAMCMP_LOG_DBG("read_data_4=0x%02X \n", param->read_data_4);

	return 0;
}


















static int mt9e311_set_sensor_reg_write( struct regwrite_param param )
{
    int rc = 0;
	camcmp_cmd_info_type cmd_info_src;
	camcmp_blank_info_type blank_info_src;

	camcmp_cmd_info_type cmd_info_dst;
	camcmp_blank_info_type blank_info_dst;

	camcmp_cmd_info_type cmd_info_size;
	camcmp_blank_info_type blank_info_size;

	camcmp_cmd_info_type cmd_info_ctrl;
	
	uint32_t src_value;
	uint32_t dst_addr;
	
	uint32_t i2cID = 0x6c;
	int aw;
	int dw;
	uint32_t addr;
	
	








		aw = 1;
		addr = ( 0x00000000 | (param.write_addr_3 << 8 ) | param.write_addr_4 );

	








	if ( param.write_byte == 1 ) {
		dw = 0;
		src_value = ( 0x00000000 | param.write_data_1 );
	} else {
		dw = 1;
		src_value = ( 0x00000000 | (param.write_data_1 << 8 ) | param.write_data_2 );
	}


	CAMCMP_LOG_DBG("i2cID=%d \n", i2cID);
	CAMCMP_LOG_DBG("Address With=%d \n", aw);
	CAMCMP_LOG_DBG("Data With=%d \n", dw);
	CAMCMP_LOG_DBG("Address=0x%08X \n", addr);
	CAMCMP_LOG_DBG("src_value=0x%08X \n", src_value);

	
	memset(&cmd_info_src,   0, sizeof(cmd_info_src));
	memset(&blank_info_src, 0, sizeof(blank_info_src));
	memset(&cmd_info_dst,   0, sizeof(cmd_info_dst));
	memset(&blank_info_dst, 0, sizeof(blank_info_dst));
	memset(&cmd_info_size,  0, sizeof(cmd_info_size));
	memset(&blank_info_size,0, sizeof(blank_info_size));
	memset(&cmd_info_ctrl,  0, sizeof(cmd_info_ctrl));
	

	
	
	cmd_info_src.cmd_id             = CAMCMP_CID_SET_REGWRITE_SRC;
		blank_info_src.p_blank_data = &src_value;
	cmd_info_src.p_blank_info       = &blank_info_src;
	cmd_info_src.blank_num          = CAMCMP_SIZE_BLKINF(blank_info_src);
	rc = camcmp_isp_trans( &cmd_info_src,  CAMCMP_SIZE_CMD(cmd_info_src));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}
	
	
	dst_addr = ((((((dw << 1) | aw) << 8) | i2cID) << 16) | addr );
	
	cmd_info_dst.cmd_id = CAMCMP_CID_SET_REGWRITE_DST;
		blank_info_dst.p_blank_data = &dst_addr;
	cmd_info_dst.p_blank_info       = &blank_info_dst;
	cmd_info_dst.blank_num          = CAMCMP_SIZE_BLKINF(blank_info_dst);
	rc = camcmp_isp_trans( &cmd_info_dst,  CAMCMP_SIZE_CMD(cmd_info_dst));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}

	
	if ( dw == 0 ) {
		blank_info_size.blank_id = CAMCMP_BID_REGWRITE_SIZE_8;
	} else {
		blank_info_size.blank_id = CAMCMP_BID_REGWRITE_SIZE_16;
	}
	cmd_info_size.cmd_id = CAMCMP_CID_SET_REGWRITE_SIZE;
	cmd_info_size.p_blank_info = &blank_info_size;
	cmd_info_size.blank_num = CAMCMP_SIZE_BLKINF(blank_info_size);
	
	rc = camcmp_isp_trans(&cmd_info_size, CAMCMP_SIZE_CMD(cmd_info_size));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}
	
	
	cmd_info_ctrl.cmd_id = CAMCMP_CID_SET_REGWRITE_CTRL;
	
	rc = camcmp_isp_trans(&cmd_info_ctrl, CAMCMP_SIZE_CMD(cmd_info_ctrl));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}
	

	return 0;
}






















static int32_t mt9e311_change_mode(int mode)
{
	int32_t rc = 0;
	struct msm_camera_csi_params csi_params;

	
	camcmp_cmd_info_type cmd_info[] = {{0, NULL, 0, NULL, 0}};

	
	
	switch (mode)
	{
		
		case SENSOR_PREVIEW_MODE:
			CAMCMP_LOG_DBG("sensor previwe mode -PREVIEW-\n");
			cmd_info[0].cmd_id = CAMCMP_CID_PREVIEW_MODE;
			break;

		
		case SENSOR_MOVIE_MODE:
			CAMCMP_LOG_DBG("sensor previwe mode -MOVIE-\n");
			cmd_info[0].cmd_id = CAMCMP_CID_MOVIE_MODE;
			break;

		
		case SENSOR_SNAPSHOT_MODE:
		case SENSOR_RAW_SNAPSHOT_MODE:
			CAMCMP_LOG_DBG("sensor capture mode\n");
			cmd_info[0].cmd_id = CAMCMP_CID_CAPTURE_MODE;
			break;

		default:
			CAMCMP_LOG_ERR("input failed! mode=%d\n", mode);
			
			return -EINVAL;
	}
	CAMCMP_LOG_DBG("sensor mode %d->%d\n", mt9e311_ctrl->sensormode, mode);

	
	if ( mt9e311_ctrl->sensormode != mode ) {
		
		rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
			return -EFAULT;
		}
		

		
		if ( mode == SENSOR_MOVIE_MODE ) {
			if (sys_rev == MT9E311_SYS_REVISION_02) {
				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE,
											CAMCMP_CID_AF_MODE,
											g_mt9e311_af_mode[CAMCMP_BID_MODE_MF_R2].mask,
											g_mt9e311_af_mode[CAMCMP_BID_MODE_MF_R2].data );
				if (rc != 0) {
					CAMCMP_LOG_ERR("i2c_read_write_modify failed. rc = %d\n", rc);
					return -EFAULT;
				}
				msleep(132);  
			}
		}
		rc = mt9e311_set_default_focus(mode);



		if (rc < 0) {
			CAMCMP_LOG_ERR("mt9e311_set_default_focus failed. rc = %d\n", rc);
			return -EFAULT;
		}
		
		if( mode == SENSOR_PREVIEW_MODE )
		{
            camcmp_cmd_info_type cmd_get_scene_mode[] = {{0, NULL, 0, NULL, 0}};
	        uint16_t scene_mode = 0;
		    uint16_t read_scene_mode = 0;
		
            
    	    cmd_get_scene_mode[0].cmd_id = CAMCMP_CID_GET_SCENE_MODE;
    	    cmd_get_scene_mode[0].p_recieve_data = &scene_mode;
    	    cmd_get_scene_mode[0].recieve_num = 1;

            
            rc = camcmp_isp_trans(cmd_get_scene_mode, CAMCMP_SIZE_CMD(cmd_get_scene_mode));            
            if (rc < 0) {
                CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
                return -EFAULT;
            }
            
	
        	read_scene_mode = scene_mode_rd_to_set( scene_mode );            

            if ( read_scene_mode == CAMERA_SCENE_MODE_NIGHT )
            {
                msleep(600);			
            }

        }

		else if ( mode == SENSOR_MOVIE_MODE )
		{
			if (sys_rev == MT9E311_SYS_REVISION_02) {
				
				if (mt9e311_ctrl->afc_mode == SENSOR_AF_STATUS_MOVE) {
					
					rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE,
												CAMCMP_CID_AF_MODE,
												g_mt9e311_af_mode[CAMCMP_BID_MODE_CAF_R2].mask,
												g_mt9e311_af_mode[CAMCMP_BID_MODE_CAF_R2].data );
					if (rc != 0) {
						CAMCMP_LOG_ERR("i2c_read_write_modify failed. rc = %d\n", rc);
						return -EFAULT;
					}
				}
				
				mt9e311_ctrl->af_mode = CAMERA_AF_MODE_NO_USE;
			}
		}

	}
	mt9e311_ctrl->sensormode = mode;


	rc = mt9e311_set_exposure_gain(mode);
	if (rc < 0) {
		CAMCMP_LOG_ERR("mt9e311_set_exposure_gain failed. rc = %d\n", rc);
		return -EFAULT;
	}


	
	CAMCMP_LOG_DBG("mt9e311_change_mode: config csi controller \n");
	csi_params.data_format = CSI_8BIT;
	csi_params.lane_cnt = 2;
	csi_params.lane_assign = 0xe4;
	csi_params.dpcm_scheme = 0;
	csi_params.settle_cnt = 7;
	rc = msm_camio_csi_config(&csi_params);
	if (rc < 0) {
		CAMCMP_LOG_ERR("config csi controller failed. rc = %d\n", rc);
		return -EFAULT;
	}
	msleep(MT9E311_WAIT_20);

	return 0;
}




















static int32_t mt9e311_video_config(int prev_mode, int res)
{
	int32_t rc;
	camcmp_cmd_info_type cmd_info_blk[] = {{0, NULL, 0, NULL, 0}};
	camcmp_blank_info_type moni_size = {0, NULL};
	camcmp_cmd_info_type cmd_info_blk2[] = {{0, NULL, 0, NULL, 0}};
	camcmp_blank_info_type movie_fps = {0, NULL};

	CAMCMP_LOG_DBG("mt9e311_video_config start\n");

	
	movie_fps.blank_id = CAMCMP_BID_MOVIE_FPS_NML;

	
	switch (res)
	{
		case SENSOR_CAMERA_HD_SIZE:				
			moni_size.blank_id = CAMCMP_BID_MONI_SIZE_HD;
			break;

		case SENSOR_CAMERA_WVGA_SIZE:			
			moni_size.blank_id = CAMCMP_BID_MONI_SIZE_WVGA;
			break;

		case SENSOR_CAMERA_VGA_SIZE:			
			moni_size.blank_id = CAMCMP_BID_MONI_SIZE_VGA;
			break;

		case SENSOR_CAMERA_QVGA_SIZE:			
			moni_size.blank_id = CAMCMP_BID_MONI_SIZE_QVGA;
			break;

		case SENSOR_CAMERA_QCIF_SIZE:			
			moni_size.blank_id = CAMCMP_BID_MONI_SIZE_QCIF;
			break;

		default:
			CAMCMP_LOG_ERR("sensor previwe size failed res = %d\n", res);
			return -EINVAL;
	}

	
	cmd_info_blk[0].cmd_id = CAMCMP_CID_MONI_SIZE;
	cmd_info_blk[0].p_blank_info = &moni_size;
	cmd_info_blk[0].blank_num = CAMCMP_SIZE_BLKINF(moni_size);

	CAMCMP_LOG_DBG("moni size %d->%d\n", mt9e311_ctrl->prev_res, res);

	
	if ( mt9e311_ctrl->prev_res != res ) {
		
		rc = camcmp_isp_trans(cmd_info_blk, CAMCMP_SIZE_CMD(cmd_info_blk));
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
			return -EFAULT;
		}
		mt9e311_ctrl->prev_res = res;
	}


	
	rc = mt9e311_change_mode(prev_mode);
	if (rc < 0) {
		CAMCMP_LOG_ERR("mt9e311_change_mode failed. rc = %d\n", rc);
		return -EFAULT;
	}


	
	if(prev_mode == SENSOR_MOVIE_MODE) {
		
		if (res == SENSOR_CAMERA_HD_SIZE) {
			CAMCMP_LOG_DBG("HD_movie_24fps\n");
			movie_fps.blank_id = CAMCMP_BID_MOVIE_FPS_HD;
		}
		cmd_info_blk2[0].cmd_id = CAMCMP_CID_MOVIE_FPS;
		cmd_info_blk2[0].p_blank_info = &movie_fps;
		cmd_info_blk2[0].blank_num = CAMCMP_SIZE_BLKINF(movie_fps);
		rc = camcmp_isp_trans(cmd_info_blk2, CAMCMP_SIZE_CMD(cmd_info_blk2));
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
			return -EFAULT;
		}
	}

	return rc;
}
























static int32_t mt9e311_snapshot_config(int32_t res)
{
	int32_t ret = 0;
	camcmp_cmd_info_type cmd_info[]  = {{0, NULL, 0, NULL, 0}};
	camcmp_blank_info_type cap_size_yuv  = {0, NULL};
	CAMCMP_LOG_DBG("start here.res =%d  !!!\n", res);

	
	switch (res)
	{
		case SENSOR_CAMERA_8MP_SIZE:		
			cap_size_yuv.blank_id = CAMCMP_BID_CAP_SIZE_8M_YUV;
			break;
		case SENSOR_CAMERA_6MW_SIZE:		
			cap_size_yuv.blank_id = CAMCMP_BID_CAP_SIZE_6MW_YUV;
			break;
		case SENSOR_CAMERA_5MP_SIZE:		
			cap_size_yuv.blank_id = CAMCMP_BID_CAP_SIZE_5M_YUV;
			break;
		case SENSOR_CAMERA_3MP_SIZE:		
			cap_size_yuv.blank_id = CAMCMP_BID_CAP_SIZE_3M_YUV;
			break;
		case SENSOR_CAMERA_2MP_SIZE:		
			cap_size_yuv.blank_id = CAMCMP_BID_CAP_SIZE_2M_YUV;
			break;
		case SENSOR_CAMERA_1MP_SIZE:		
			cap_size_yuv.blank_id = CAMCMP_BID_CAP_SIZE_1M_YUV;
			break;
		case SENSOR_CAMERA_WVGA_SIZE:		
			cap_size_yuv.blank_id = CAMCMP_BID_CAP_SIZE_WVGA_YUV;
			break;
		case SENSOR_CAMERA_VGA_SIZE:		
			cap_size_yuv.blank_id = CAMCMP_BID_CAP_SIZE_VGA_YUV;
			break;
		case SENSOR_CAMERA_QVGA_SIZE:		
			cap_size_yuv.blank_id = CAMCMP_BID_CAP_SIZE_QVGA_YUV;
			break;
		default:
			return -EINVAL;
	}

	cmd_info[0].cmd_id = CAMCMP_CID_CAP_SIZE_YUV;
	cmd_info[0].p_blank_info = &cap_size_yuv;
	cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(cap_size_yuv);

	
	ret = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (ret < 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans failed [RC:%d]\n",ret);
		return ret;
	}


	
	ret = i2c_read_write_modify( CAMCMP_CID_GET_AF_S1_CTRL_ENABLE,
								 CAMCMP_CID_AF_S1_ENABLE,
								 g_mt9e311_s1_ctrl_en[CAMCMP_BID_AF_S1_CTRL_DISABLE].mask,
								 g_mt9e311_s1_ctrl_en[CAMCMP_BID_AF_S1_CTRL_DISABLE].data );
	if (ret < 0) {
		CAMCMP_LOG_ERR("i2c_read_write_modify failed. ret = %d\n", ret);
		return -EFAULT;
	}


	
	ret = mt9e311_change_mode(SENSOR_SNAPSHOT_MODE);
	if (ret < 0) {
		CAMCMP_LOG_ERR("mt9e311_change_mode failed. ret = %d\n", ret);
		return ret;
	}

	CAMCMP_LOG_DBG("succsess\n");

	return ret;
}
























static int32_t mt9e311_raw_snapshot_config(int res)
{
	int32_t ret = 0;
	camcmp_cmd_info_type cmd_info[]  = {{0, NULL, 0, NULL, 0}};
	camcmp_blank_info_type cap_size  = {0, NULL};

	CAMCMP_LOG_DBG("start here !!!\n");

	
	switch (res)
	{
		case SENSOR_CAMERA_8MP_SIZE:		
			cap_size.blank_id = CAMCMP_BID_CAP_SIZE_8M;
			break;
		case SENSOR_CAMERA_6MW_SIZE:		
			cap_size.blank_id = CAMCMP_BID_CAP_SIZE_6MW;
			break;
		case SENSOR_CAMERA_5MP_SIZE:		
			cap_size.blank_id = CAMCMP_BID_CAP_SIZE_5M;
			break;
		case SENSOR_CAMERA_3MP_SIZE:		
			cap_size.blank_id = CAMCMP_BID_CAP_SIZE_3M;
			break;
		case SENSOR_CAMERA_2MP_SIZE:		
			cap_size.blank_id = CAMCMP_BID_CAP_SIZE_2M;
			break;
		case SENSOR_CAMERA_1MP_SIZE:		
			cap_size.blank_id = CAMCMP_BID_CAP_SIZE_1M;
			break;
		case SENSOR_CAMERA_WVGA_SIZE:		
			cap_size.blank_id = CAMCMP_BID_CAP_SIZE_WVGA;
			break;
		case SENSOR_CAMERA_VGA_SIZE:		
			cap_size.blank_id = CAMCMP_BID_CAP_SIZE_VGA;
			break;
		case SENSOR_CAMERA_QVGA_SIZE:		
			cap_size.blank_id = CAMCMP_BID_CAP_SIZE_QVGA;
			break;
		default:
			return -EINVAL;
	}

	cmd_info[0].cmd_id = CAMCMP_CID_CAP_SIZE;
	cmd_info[0].p_blank_info = &cap_size;
	cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(cap_size);

	
	ret = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (ret < 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans failed [RC:%d]\n",ret);
		return ret;
	}


	
	ret = i2c_read_write_modify( CAMCMP_CID_GET_AF_S1_CTRL_ENABLE,
								 CAMCMP_CID_AF_S1_ENABLE,
								 g_mt9e311_s1_ctrl_en[CAMCMP_BID_AF_S1_CTRL_DISABLE].mask,
								 g_mt9e311_s1_ctrl_en[CAMCMP_BID_AF_S1_CTRL_DISABLE].data );
	if (ret < 0) {
		CAMCMP_LOG_ERR("i2c_read_write_modify failed. ret = %d\n", ret);
		return -EFAULT;
	}



	
	ret = mt9e311_change_mode(SENSOR_RAW_SNAPSHOT_MODE);
	if (ret < 0) {
		CAMCMP_LOG_ERR("mt9e311_change_mode failed. ret = %d\n", ret);
		return ret;
	}


	CAMCMP_LOG_DBG("succsess\n");

	return ret;
}




















static int mt9e311_set_mode(int mode, int res)
{
	int rc = 0;

	CAMCMP_LOG_DBG("start here !!!\n");

	switch (mode) {
	case SENSOR_PREVIEW_MODE:
	case SENSOR_MOVIE_MODE:
		CAMCMP_LOG_DBG("SENSOR_MODE mode = %d\n", mode);

		rc = mt9e311_video_config(mode, res);
		break;


	case SENSOR_RAW_SNAPSHOT_MODE:
		CAMCMP_LOG_DBG("SENSOR_RAW_SNAPSHOT_MODE\n");
		rc = mt9e311_raw_snapshot_config(res);
		break;


	case SENSOR_SNAPSHOT_MODE:
		rc = mt9e311_snapshot_config(res);
		break;

	default:
		CAMCMP_LOG_ERR("param error. default\n");
		rc = -EINVAL;
		break;
	}

	return rc;
}





















static int mt9e311_set_af_window(void)
{
    int rc = 0;

	camcmp_cmd_info_type cmd_info;
	camcmp_blank_info_type blank_info;
    uint32_t af_start_val = 0;
    uint32_t af_stop_val = 0;

	
	memset(&cmd_info, 0, sizeof(cmd_info));
	memset(&blank_info, 0, sizeof(blank_info));

    af_start_val = g_mt9e311_sd_read[CAMCMP_AF_START].val;

	blank_info.blank_id = CAMCMP_BID_AFS_INF_VAL;
    blank_info.p_blank_data = &af_start_val;
	cmd_info.cmd_id = CAMCMP_CID_AF_SEARCH_INF;
	cmd_info.p_blank_info = &blank_info;
	cmd_info.blank_num = CAMCMP_SIZE_BLKINF(blank_info);

	
	rc = camcmp_isp_trans(&cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
		return -EFAULT;
	}

	memset(&cmd_info, 0, sizeof(cmd_info));
	memset(&blank_info, 0, sizeof(blank_info));

    af_stop_val = g_mt9e311_sd_read[CAMCMP_AF_STOP].val;

	blank_info.blank_id = CAMCMP_BID_AFS_MAC_VAL;
	cmd_info.cmd_id = CAMCMP_CID_AF_SEARCH_MAC;
    blank_info.p_blank_data = &af_stop_val;
	cmd_info.p_blank_info = &blank_info;
	cmd_info.blank_num = CAMCMP_SIZE_BLKINF(blank_info);

	
	rc = camcmp_isp_trans(&cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
		return -EFAULT;
	}

	
	if (sys_rev == MT9E311_SYS_REVISION_02) {
		memset(&cmd_info, 0, sizeof(cmd_info));
		
		cmd_info.cmd_id = CAMCMP_CID_AF_WINDOW_SET;
		
		rc = camcmp_isp_trans(&cmd_info, CAMCMP_SIZE_CMD(cmd_info));
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
			return -EFAULT;
		}
	}

    return rc;
}

















static int mt9e311_set_af_pos(int af_mode)
{
    int rc = 0;
	camcmp_cmd_info_type cmd_info;
	camcmp_blank_info_type blank_info;
	uint32_t pos_value = 0;

	if ( af_mode != mt9e311_ctrl->af_mode ) {
		
		memset(&cmd_info, 0, sizeof(cmd_info));
		memset(&blank_info, 0, sizeof(blank_info));

		if (sys_rev == MT9E311_SYS_REVISION_01) {
			switch (af_mode)
			{
				case CAMERA_AF_MODE_NORMAL:     
					blank_info.blank_id = CAMCMP_BID_MF_CUSTOM;
					pos_value = g_mt9e311_sd_read[CAMCMP_PAN_NORMAL_REG].val;
					break;
				case CAMERA_AF_MODE_MACRO:      
					blank_info.blank_id = CAMCMP_BID_MF_MACRO;
					break;
				case CAMERA_AF_MODE_INFINITY:   
					blank_info.blank_id = CAMCMP_BID_MF_INF;
					break;
				default:
					CAMCMP_LOG_ERR("invalid af mode. af_mode = %d\n", af_mode);
					blank_info.blank_id = CAMCMP_BID_MF_INF;
					break;
			}
		} else if (sys_rev == MT9E311_SYS_REVISION_02) {
			switch (af_mode)
			{
				case CAMERA_AF_MODE_NORMAL:     
					blank_info.blank_id = CAMCMP_BID_MF_CUSTOM;
					pos_value = g_mt9e311_sd_read[CAMCMP_PAN_NORMAL_REG].val;
					break;
				case CAMERA_AF_MODE_MACRO:      
					blank_info.blank_id = CAMCMP_BID_MF_MACRO_R2;
					break;
				case CAMERA_AF_MODE_INFINITY:   
					blank_info.blank_id = CAMCMP_BID_MF_INF_R2;
					break;
				default:
					CAMCMP_LOG_ERR("invalid af mode. af_mode = %d\n", af_mode);
					blank_info.blank_id = CAMCMP_BID_MF_INF_R2;
					break;
			}
		} else {
			CAMCMP_LOG_ERR("invalid system revision. sys_rev = 0x%04x\n", sys_rev);
			blank_info.blank_id = CAMCMP_BID_MF_INF_R2;
		}
		cmd_info.cmd_id = CAMCMP_CID_MF_START;
		cmd_info.p_blank_info = &blank_info;
		if(blank_info.blank_id == CAMCMP_BID_MF_CUSTOM){
			blank_info.p_blank_data = &pos_value;
		}
		cmd_info.blank_num = CAMCMP_SIZE_BLKINF(blank_info);

		
		rc = camcmp_isp_trans(&cmd_info, CAMCMP_SIZE_CMD(cmd_info));
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
			return -EFAULT;
		}
		msleep(132);  

		
		mt9e311_ctrl->af_mode = af_mode;
	} else {
		CAMCMP_LOG_DBG("request mode is equal to existing mode. af_mode = %d", af_mode );
	}

	return rc;
}




















static int mt9e311_set_af_mode_off(int af_mode)
{
	int rc = 0;

	uint32_t	blank_id;

	
	if( (af_mode == CAMERA_AF_MODE_NORMAL) ||
		(af_mode == CAMERA_AF_MODE_MACRO)  ||
		(af_mode == CAMERA_AF_MODE_INFINITY)) {
		
		switch (sys_rev) {
			case MT9E311_SYS_REVISION_01:
				blank_id = CAMCMP_BID_MODE_MF;
				break;
			case MT9E311_SYS_REVISION_02:
				blank_id = CAMCMP_BID_MODE_MF_R2;
				break;
			default:
				CAMCMP_LOG_ERR("invalid system revision. sys_rev = 0x%04x\n", sys_rev);
				blank_id = CAMCMP_BID_MODE_MF_R2;
				break;
		}
		rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE,
									CAMCMP_CID_AF_MODE,
									( g_mt9e311_af_mode[blank_id].mask |
									  g_mt9e311_af_ctrl[blank_id].mask ),
									( g_mt9e311_af_mode[blank_id].data |
									  g_mt9e311_af_ctrl[blank_id].data ) );
		if (rc < 0) {
			CAMCMP_LOG_ERR("i2c_read_write_modify failed. rc = %d\n", rc);
			return -EFAULT;
		}

		
		rc = mt9e311_set_af_window();
		if (rc < 0) {
			CAMCMP_LOG_ERR("mt9e311_set_af_window failed. rc = %d\n", rc);
			return -EFAULT;
		}

		
		rc = mt9e311_set_af_pos(af_mode);
		if (rc < 0) {
			CAMCMP_LOG_ERR("mt9e311_set_af_pos failed. rc = %d\n", rc);
			return -EFAULT;
		}
	} else {
		
	}


	return rc;
}


















static int mt9e311_set_af_mode_on(struct msm_sync *sync)
{
    int rc = 0;
	camcmp_cmd_info_type cmd_info;
	uint16_t prev_ctrl = 0;
	uint32_t blank_id = 0;

	CAMCMP_LOG_DBG("mt9e311_set_af_mode_on start\n");

	
	memset( &cmd_info, 0, sizeof(cmd_info) );

	cmd_info.cmd_id = CAMCMP_CID_GET_PREVCTL;
	cmd_info.p_recieve_data = &prev_ctrl;
	cmd_info.recieve_num = 1;

	
	rc = camcmp_isp_trans(&cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
		return -EFAULT;
	}

	
	switch (sys_rev) {
		case MT9E311_SYS_REVISION_01:
			if ((prev_ctrl & 0x0010) == 0) {
				
				blank_id = CAMCMP_BID_MODE_SAF;
			} else {
				
				blank_id = CAMCMP_BID_MODE_FACE;
			}
			break;
		case MT9E311_SYS_REVISION_02:
			if ((prev_ctrl & 0x0010) == 0) {
				
				blank_id = CAMCMP_BID_MODE_SAF_R2;
			} else {
				
				blank_id = CAMCMP_BID_MODE_FACE_R2;
			}
			break;
		default:
			CAMCMP_LOG_ERR("invalid system revision. sys_rev = 0x%04x\n", sys_rev);
			if ((prev_ctrl & 0x0010) == 0) {
				
				blank_id = CAMCMP_BID_MODE_SAF_R2;
			} else {
				
				blank_id = CAMCMP_BID_MODE_FACE_R2;
			}
			break;
	}

	rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE,
								CAMCMP_CID_AF_MODE,
								g_mt9e311_af_ctrl[blank_id].mask,
								g_mt9e311_af_ctrl[blank_id].data );
	if (rc < 0) {
		CAMCMP_LOG_ERR("i2c_read_write_modify failed. rc = %d\n", rc);
		return -EFAULT;
	}

	
	rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_S1_CTRL_ENABLE,
								 CAMCMP_CID_AF_S1_ENABLE,
								 g_mt9e311_s1_ctrl_en[CAMCMP_BID_AF_S1_CTRL_ENABLE].mask,
								 g_mt9e311_s1_ctrl_en[CAMCMP_BID_AF_S1_CTRL_ENABLE].data );
	if (rc < 0) {
		CAMCMP_LOG_ERR("i2c_read_write_modify failed. rc = %d\n", rc);
		return -EFAULT;
	}

	
	memset( &cmd_info, 0, sizeof(cmd_info) );

	cmd_info.cmd_id = CAMCMP_CID_AF_S1_S2_SET;
	rc = camcmp_isp_trans(&cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
		return -EFAULT;
	}

























	
	mt9e311_ctrl->af_mode = CAMERA_AF_MODE_NO_USE;
	
	
	mt9e311_ctrl->af_cancel_onoff = SENSOR_AF_OFF;

	
	mt9e311_ctrl->af_exec_flag = SENSOR_ON;

	
	init_timer(&timer);
	timer.expires = jiffies + MT9E311_CHECK_STATE_WAIT;
	timer.data=(unsigned long)sync;
	timer.function = msm_af_start_poll;
	add_timer(&timer);



    return rc;
}




















static int mt9e311_set_conti_af_mode(uint8_t mode)
{
    int rc = 0;
    uint32_t	blank_id;

	CAMCMP_LOG_DBG("mt9e311_set_conti_af_mode\n");
	
	if (sys_rev == MT9E311_SYS_REVISION_01) {
		if(mode == SENSOR_AF_CONTI_ON) {
			blank_id = CAMCMP_BID_MODE_CAF;
		} else {
			blank_id = CAMCMP_BID_MODE_MF;
		}
	} else if (sys_rev == MT9E311_SYS_REVISION_02) {
		if(mode == SENSOR_AF_CONTI_ON) {
			blank_id = CAMCMP_BID_MODE_CAF_R2;
		} else {
			blank_id = CAMCMP_BID_MODE_MF_R2;
		}
	} else {
		CAMCMP_LOG_ERR("invalid system revision. sys_rev = 0x%04x\n", sys_rev);
		if(mode == SENSOR_AF_CONTI_ON) {
			blank_id = CAMCMP_BID_MODE_CAF_R2;
		} else {
			blank_id = CAMCMP_BID_MODE_MF_R2;
		}
	}
	rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE,
								CAMCMP_CID_AF_MODE,
								( g_mt9e311_af_mode[blank_id].mask |
								  g_mt9e311_af_ctrl[blank_id].mask ),
								( g_mt9e311_af_mode[blank_id].data |
								  g_mt9e311_af_ctrl[blank_id].data ) );
	if (rc < 0) {
		CAMCMP_LOG_ERR("i2c_read_write_modify failed. rc = %d\n", rc);
		return -EFAULT;
	}

    
    mt9e311_ctrl->af_mode = CAMERA_AF_MODE_NO_USE;

    return rc;
}


















static int mt9e311_set_face_mode(uint8_t face)
{
    int rc = 0;
    uint32_t	blank_id;

	
	if (face == FACE_SMILE_NO_USE) {
		return rc;
	}
	

	
	switch (face) {
		case FACE_DETECTION_OFF:
			blank_id = CAMCMP_BID_FACE_MODE_OFF;
			break;
		case FACE_DETECTION_ON:
			blank_id = CAMCMP_BID_FACE_MODE_FACE;
			break;
		case FACE_SMILE_DETECTION_ON:
			blank_id = CAMCMP_BID_FACE_MODE_FACE_SMILE;
			break;
		default:
			CAMCMP_LOG_ERR("invalid face mode. face_mode = %d\n", face);
			return -EINVAL;
			break;
	}

	rc = i2c_read_write_modify( CAMCMP_CID_GET_PREVCTL,
								CAMCMP_CID_FACE_MODE,
								g_mt9e311_face_mode[blank_id].mask,
								g_mt9e311_face_mode[blank_id].data );
	if (rc < 0) {
		CAMCMP_LOG_ERR("i2c_read_write_modify failed. rc = %d\n", rc);
		return -EFAULT;
	}
	
    return rc;
}

















static int mt9e311_get_face_pos(msm_camera_face_info_t *face_pos)
{
    int rc = 0;
    int i;

	camcmp_cmd_info_type cmd_info;
	uint32_t face_value[41];

	
	if (sys_rev == MT9E311_SYS_REVISION_02) {
		
		rc = i2c_read_write_modify( CAMCMP_CID_GET_FACE_CTRL,
									CAMCMP_CID_SET_FACE_CTRL,
									g_mt9e311_faced_ctrl[CAMCMP_BID_FACE_CTRL_MANUAL].mask,
									g_mt9e311_faced_ctrl[CAMCMP_BID_FACE_CTRL_MANUAL].data );
		if (rc < 0) {
			CAMCMP_LOG_ERR("i2c_read_write_modify failed. rc = %d\n", rc);
			return -EFAULT;
		}

		
		memset( &cmd_info, 0, sizeof(cmd_info) );

		cmd_info.cmd_id = CAMCMP_CID_POLL_FACE_CTRL;
		rc = camcmp_isp_trans(&cmd_info, CAMCMP_SIZE_CMD(cmd_info));
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
			return -EFAULT;
		}

		
		memset(&cmd_info, 0, sizeof(cmd_info));
		memset(face_value, 0, sizeof(face_value));

		cmd_info.cmd_id = CAMCMP_CID_GET_FACE_POS;
		cmd_info.p_recieve_data = &face_value;
		cmd_info.recieve_num = sizeof(face_value) / sizeof(uint32_t);
		rc = camcmp_isp_trans(&cmd_info, CAMCMP_SIZE_CMD(cmd_info));
		if (rc < 0) {
			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
			return -EFAULT;
		}
		
		
		face_pos->face_cnt = (uint16_t)face_value[0];
		for (i=0; i<10; i++) {
			face_pos->face_pos[i].pos_x0 = (uint16_t)face_value[i+1];
			face_pos->face_pos[i].pos_y0 = (uint16_t)face_value[10+i+1];
			face_pos->face_pos[i].pos_x1 = (uint16_t)face_value[20+i+1];
			face_pos->face_pos[i].pos_y1 = (uint16_t)face_value[30+i+1];
		}
	} else {
		CAMCMP_LOG_DBG("This Revision not support manual face detect.\n");
	}

    return rc;
}
























static int mt9e311_set_af_mode(struct msm_sync *sync, struct af_mode_cfg *af_mode_chg)
{
	int rc = 0;
	camcmp_cmd_info_type cmd_info;


	CAMCMP_LOG_DBG("mt9e311_set_af_mode :af_ends=%d\n", af_mode_chg->af_ends);
	
	rc = mt9e311_set_face_mode(af_mode_chg->face);

	
	msleep(MT9E311_WAIT_50 + MT9E311_WAIT_10 + MT9E311_WAIT_5 + MT9E311_WAIT_1);

	switch(af_mode_chg->af_ends)
	{
		case SENSOR_AF_CONTI_OFF:	
		case SENSOR_AF_CONTI_ON:	
			CAMCMP_LOG_DBG("mt9e311_set_af_mode CONTINUOUS AF_SETTING\n");
			rc = mt9e311_set_conti_af_mode(af_mode_chg->af_ends);
			break;
		
		case SENSOR_AF_OFF:
			CAMCMP_LOG_DBG("mt9e311_set_af_mode AF_OFF\n");
			rc = mt9e311_set_af_mode_off(af_mode_chg->af_mode);
			break;
		
		case SENSOR_AF_ON:
			CAMCMP_LOG_DBG("mt9e311_set_af_mode AF_ON\n");
			rc = mt9e311_set_af_mode_off((int)af_mode_chg->af_mode);

			
			memset(&cmd_info, 0, sizeof(cmd_info));
			
			switch (sys_rev) {
				case MT9E311_SYS_REVISION_01:
					cmd_info.cmd_id = CAMCMP_CID_AF_SET_EVAL;
					break;
				case MT9E311_SYS_REVISION_02:
					cmd_info.cmd_id = CAMCMP_CID_AF_SET_EVAL_R2;
					break;
				default:
					CAMCMP_LOG_ERR("invalid system revision. sys_rev = 0x%04x\n", sys_rev);
					cmd_info.cmd_id = CAMCMP_CID_AF_SET_EVAL_R2;
					break;
			}
			
			rc = camcmp_isp_trans(&cmd_info, CAMCMP_SIZE_CMD(cmd_info));
			if (rc < 0) {
				CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
				return -EFAULT;
			}

			rc = mt9e311_set_af_mode_on(sync);
			break;
		
		default:
			rc = -EINVAL;
			break;
	}
	

	return rc;
}


















static int mt9e311_get_otp(struct get_otp_cfg *get_otp)
{
	int  rc  = 0;
	uint32_t get_otp_data = 0;

	
	get_otp->data_0 = (get_otp_data & 0xff000000) >> 24;
	get_otp->data_1 = (get_otp_data & 0x00ff0000) >> 16;
	get_otp->data_2 = (get_otp_data & 0x0000ff00) >> 8;
	get_otp->data_3 = (get_otp_data & 0x000000ff);

	return rc;
}



















static int mt9e311_set_sensor_pat_chg(int8_t test_pat_mode)
{
	int rc = 0;

	camcmp_cmd_info_type cmd_info[]  = {{0, NULL, 0, NULL, 0}};
	camcmp_blank_info_type set_test_pat = {0, NULL};
	camcmp_blank_info_type pre_sensor_mode = {0, NULL};
	camcmp_blank_info_type set_ctrl_gain = {0, NULL};
	camcmp_blank_sensor_test_pat_enum mode = (camcmp_blank_sensor_test_pat_enum)test_pat_mode;

	CAMCMP_LOG_DBG("start\n");

	CAMCMP_LOG_DBG("sensor_test_pat_mode=%d\n", mode);

	
	if ((mode < CAMCMP_BID_SENSOR_TEST_PAT_OFF) ||
			(mode > CAMCMP_BID_SENSOR_TEST_PAT_WALKING_1S_8)) {
		CAMCMP_LOG_ERR("parameter error root\n");
		return -EINVAL;
	}

	
	if (mode == CAMCMP_BID_SENSOR_TEST_PAT_OFF) {
		pre_sensor_mode.blank_id = CAMCMP_BID_PREVIEW_SENSOR_MODE_OFF;
		set_ctrl_gain.blank_id = CAMCMP_BID_CTRL_GAIN_OFF;
	} else {
    pre_sensor_mode.blank_id = CAMCMP_BID_PREVIEW_SENSOR_MODE_FULL;
		set_ctrl_gain.blank_id = CAMCMP_BID_CTRL_GAIN_ON;
  }

	
	cmd_info[0].cmd_id = CAMCMP_CID_PREVIEW_SENSOR_MODE;
	cmd_info[0].p_blank_info = &pre_sensor_mode;
	cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(pre_sensor_mode);
	
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("preview_senso_mode set error root\n");
		return -EFAULT;
	}

	
	cmd_info[0].cmd_id = CAMCMP_CID_CTRL_GAIN;
	cmd_info[0].p_blank_info = &set_ctrl_gain;
	cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(set_ctrl_gain);
	
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("set_ctrl_gain set error root\n");
		return -EFAULT;
	}

	
	
	set_test_pat.blank_id = mode;

	cmd_info[0].cmd_id = CAMCMP_CID_SENSOR_TEST_PATTERN;
	cmd_info[0].p_blank_info = &set_test_pat;
	cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(set_test_pat);

	
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("sensor_test_pat set error root\n");
		return -EFAULT;
	}

	CAMCMP_LOG_DBG("end\n");

	return rc;
}



















static int mt9e311_get_af_pos(struct get_af_pos_cfg *af_pos)
{

	int rc                          = 0;
	uint32_t af_lens_pos            = 0;
	uint32_t af_value[2]; 
	
	camcmp_cmd_info_type cmd_info[] = {{0, NULL, 0, NULL, 0}}; 
	camcmp_cmd_info_type cmd_info2[] = {{0, NULL, 0, NULL, 0}}; 
	
	
	cmd_info[0].cmd_id = CAMCMP_CID_GET_LENSPOS;
	cmd_info[0].p_recieve_data = &af_lens_pos;
	cmd_info[0].recieve_num = 1;
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}
	CAMCMP_LOG_DBG("af_lens_pos=0x%04x, \n", af_lens_pos);

	
	
	switch (sys_rev)
	{
		case MT9E311_SYS_REVISION_01:
			cmd_info2[0].cmd_id = CAMCMP_CID_GET_EVALUATE;
			break;
		case MT9E311_SYS_REVISION_02:
			cmd_info2[0].cmd_id = CAMCMP_CID_GET_EVALUATE_R2;
			break;
		default:
			CAMCMP_LOG_ERR("invalid system revision. sys_rev = 0x%04x\n", sys_rev);
			cmd_info2[0].cmd_id = CAMCMP_CID_GET_EVALUATE_R2;
			break;
	}
	cmd_info2[0].p_recieve_data = &af_value;
	cmd_info2[0].recieve_num = sizeof(af_value) / sizeof(uint32_t);
	rc = camcmp_isp_trans(cmd_info2, CAMCMP_SIZE_CMD(cmd_info2));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}
	CAMCMP_LOG_DBG("af_value[0]=0x%04x, \n", af_value[0]);
	CAMCMP_LOG_DBG("af_value[1]=0x%04x, \n", af_value[1]);
	

	
	af_pos->af_lens_pos_0    = 0x00;
	af_pos->af_lens_pos_1    = (af_lens_pos  & 0x00ff);
	
	af_pos->af_evaluate_0    = (uint8_t)(af_value[0] >> 8);
	af_pos->af_evaluate_1    = (uint8_t)(af_value[0] & 0x00FF);
	af_pos->af_evaluate_2    = (uint8_t)(af_value[1] >> 8);
	af_pos->af_evaluate_3    = (uint8_t)(af_value[1] & 0x00FF);
	

	if (sys_rev == MT9E311_SYS_REVISION_02){
		if((af_lens_pos & 0xFF00) == 0x0F00){
			
			af_pos->af_result = AF_FAILED;    
		}else{
			
			af_pos->af_result = AF_SUCCESS;
		}
	} else {
		if((af_lens_pos & 0xFF00) == 0xFF00){
			
			af_pos->af_result = AF_FAILED;    
		}else{
			
			af_pos->af_result = AF_SUCCESS;
		}
	}


	return rc;
}




















static int mt9e311_get_moni_gain(struct get_monigain_cfg *moni_gain)
{
	int rc = 0;




	uint32_t contrast[2];
	uint16_t ae_gain = 0;
	uint32_t ae_exp_time = 0;
	camcmp_cmd_info_type cmd_info[] = {{0, NULL, 0, NULL, 0}};
	camcmp_cmd_info_type cmd_info2[] = {{0, NULL, 0, NULL, 0}, {0, NULL, 0, NULL, 0}};
	camcmp_cmd_info_type cmd_info_af_set_init[] = {{0, NULL, 0, NULL, 0}};










	

	if (sys_rev == MT9E311_SYS_REVISION_01){

		cmd_info_af_set_init[0].cmd_id = CAMCMP_CID_AF_SET_EVAL;

		rc = camcmp_isp_trans(cmd_info_af_set_init, CAMCMP_SIZE_CMD(cmd_info_af_set_init));
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
			return -EFAULT;
		}
	}

	switch (sys_rev) {
		case MT9E311_SYS_REVISION_01:
			cmd_info[0].cmd_id = CAMCMP_CID_GET_EVALUATE;
			break;
		case MT9E311_SYS_REVISION_02:
			cmd_info[0].cmd_id = CAMCMP_CID_GET_EVALUATE_R2;
			break;
		default:
			CAMCMP_LOG_ERR("invalid system revision. sys_rev = 0x%04x\n", sys_rev);
			cmd_info[0].cmd_id = CAMCMP_CID_GET_EVALUATE_R2;
			break;
	}

	CAMCMP_LOG_ERR("invalid system revision. sys_rev = 0x%04x\n", sys_rev);
	cmd_info[0].p_recieve_data = &contrast;
	cmd_info[0].recieve_num = 2;

	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}

	
	moni_gain->contrast_0 = (contrast[0] & 0x0000ff00) >> 8;
	moni_gain->contrast_1 = (contrast[0] & 0x000000ff);
	moni_gain->contrast_2 = (contrast[1] & 0x0000ff00) >> 8;
	moni_gain->contrast_3 = (contrast[1] & 0x000000ff);
	CAMCMP_LOG_DBG("contrast=%x\n", contrast[0]);
	CAMCMP_LOG_DBG("contrast=%x\n", contrast[1]);

    
	cmd_info2[0].cmd_id = CAMCMP_CID_GET_AE_GAIN;
	cmd_info2[0].p_recieve_data = &ae_gain;
	cmd_info2[0].recieve_num = 1;

    
	cmd_info2[1].cmd_id = CAMCMP_CID_GET_AE_EXP_TIME;
	cmd_info2[1].p_recieve_data = &ae_exp_time;
	cmd_info2[1].recieve_num = 1; 

	rc = camcmp_isp_trans(cmd_info2, CAMCMP_SIZE_CMD(cmd_info2));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}

	
	moni_gain->ae_gain_0 = (ae_gain & 0x0000ff00) >> 8;
	moni_gain->ae_gain_1 = (ae_gain & 0x000000ff);
	CAMCMP_LOG_DBG("ae_gain=%x \n", ae_gain);

	
	moni_gain->ae_exp_time_0 = (ae_exp_time & 0xff000000) >> 24;
	moni_gain->ae_exp_time_1 = (ae_exp_time & 0x00ff0000) >> 16;
	moni_gain->ae_exp_time_2 = (ae_exp_time & 0x0000ff00) >> 8;
	moni_gain->ae_exp_time_3 = (ae_exp_time & 0x000000ff);
	CAMCMP_LOG_DBG("ae_exp_time=%x\n", ae_exp_time);
	return rc;
}




















static int mt9e311_get_reg_ver(struct get_reg_ver_cfg *get_reg_ver)
{
	int			rc = 0;
	uint16_t	chip_ver=0;
	uint16_t	xml_ver=0;
	camcmp_cmd_info_type cmd_info[] = {{0, NULL, 0, NULL, 0}};
	camcmp_cmd_info_type cmd_info2[] = {{0, NULL, 0, NULL, 0}};

	CAMCMP_LOG_DBG("mt9e311_get_reg_ver start\n");

	
	cmd_info[0].cmd_id = CAMCMP_CID_GET_CHIP_VERSION;
	cmd_info[0].p_recieve_data = &chip_ver;
	cmd_info[0].recieve_num = 1;
	
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}

	
	get_reg_ver->chip_ver0 = (chip_ver & 0x0000ff00) >> 8;
	get_reg_ver->chip_ver1 = (chip_ver & 0x000000ff);

	
	cmd_info2[0].cmd_id = CAMCMP_CID_GET_XML_VERSION;
	cmd_info2[0].p_recieve_data = &xml_ver;
	cmd_info2[0].recieve_num = 1;

	
	rc = camcmp_isp_trans(cmd_info2, CAMCMP_SIZE_CMD(cmd_info2));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}

	
	get_reg_ver->xml_ver0 = 0x00;
	get_reg_ver->xml_ver1 = 0x00;
	get_reg_ver->xml_ver2 = (xml_ver & 0x0000ff00) >> 8;
	get_reg_ver->xml_ver3 = (xml_ver & 0x000000ff);

	get_reg_ver->param_ver0 = 0x00;
	get_reg_ver->param_ver1 = 0x00;
	get_reg_ver->param_ver2 = 0x00;
	get_reg_ver->param_ver3 = 0x00;

	
	CAMCMP_LOG_ERR("get_reg_ver->chip_ver0 = %x\n", get_reg_ver->chip_ver0);
	CAMCMP_LOG_ERR("get_reg_ver->chip_ver1 = %x\n", get_reg_ver->chip_ver1);
	CAMCMP_LOG_ERR("get_reg_ver->xml_ver2 = %x\n", get_reg_ver->xml_ver2);
	CAMCMP_LOG_ERR("get_reg_ver->xml_ver3 = %x\n", get_reg_ver->xml_ver3);


	CAMCMP_LOG_DBG("mt9e311_get_reg_ver end \n");
	return rc;
}




















static int mt9e311_set_manual_step(struct set_manual_step_cfg *set_manual_step)
{

	int      rc = 0;
	int32_t af_lens_pos = 0;
	uint32_t af_value[2]; 
	uint32_t blank_id;

	camcmp_cmd_info_type cmd_info_af_window[]= {{0, NULL, 0, NULL, 0}};
	camcmp_cmd_info_type cmd_info_pos[]      = {{0, NULL, 0, NULL, 0}};
	camcmp_cmd_info_type cmd_info_lens_mov[] = {{0, NULL, 0, NULL, 0}};
	camcmp_blank_info_type mf_pos            = {0, NULL};
	camcmp_cmd_info_type cmd_info_af_value[] = {{0, NULL, 0, NULL, 0}}; 
	camcmp_cmd_info_type cmd_info_af_set_init[] = {{0, NULL, 0, NULL, 0}};

	if (sys_rev == MT9E311_SYS_REVISION_01){

		cmd_info_af_set_init[0].cmd_id = CAMCMP_CID_AF_SET_EVAL;

		rc = camcmp_isp_trans(cmd_info_af_set_init, CAMCMP_SIZE_CMD(cmd_info_af_set_init));
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
			return -EFAULT;
		}
	}

	
	switch (sys_rev) {
		case MT9E311_SYS_REVISION_01:
			blank_id = CAMCMP_BID_MODE_MF;
			break;
		case MT9E311_SYS_REVISION_02:
			blank_id = CAMCMP_BID_MODE_MF_R2;
			break;
		default:
			CAMCMP_LOG_ERR("invalid system revision. sys_rev = 0x%04x\n", sys_rev);
			blank_id = CAMCMP_BID_MODE_MF_R2;
			break;
	}
	rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE,
								CAMCMP_CID_AF_MODE,
								( g_mt9e311_af_mode[blank_id].mask |
								  g_mt9e311_af_ctrl[blank_id].mask ),
								( g_mt9e311_af_mode[blank_id].data |
								  g_mt9e311_af_ctrl[blank_id].data ) );
	if (rc < 0) {
		CAMCMP_LOG_ERR("i2c_read_write_modify failed. rc = %d\n", rc);
		return -EFAULT;
	}

	
	if (sys_rev == MT9E311_SYS_REVISION_02) {
		cmd_info_af_window[0].cmd_id = CAMCMP_CID_AF_WINDOW_SET;
		
		rc = camcmp_isp_trans(cmd_info_af_window, CAMCMP_SIZE_CMD(cmd_info_af_window));
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
			return -EFAULT;
		}
	}

	
	cmd_info_pos[0].cmd_id = CAMCMP_CID_GET_LENSPOS;
	cmd_info_pos[0].p_recieve_data = &af_lens_pos;
	cmd_info_pos[0].recieve_num = 1;
	rc = camcmp_isp_trans(cmd_info_pos, CAMCMP_SIZE_CMD(cmd_info_pos));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}
	af_lens_pos = (af_lens_pos & 0x00FF);
	CAMCMP_LOG_DBG("af_lens_pos=0x%04x, \n", af_lens_pos);
	CAMCMP_LOG_DBG("direction=%d, lens_step=0x%04x, \n", set_manual_step->direct, set_manual_step->step);

	
	if(set_manual_step->direct){		
		af_lens_pos -= set_manual_step->step;
	}
	else{                               
		af_lens_pos += set_manual_step->step;
	}
	CAMCMP_LOG_DBG("move_lens_pos=0x%04x, \n", af_lens_pos);
	
	
	if(af_lens_pos > 0x00FF){
		af_lens_pos = 0x00FF;
	}
	else if(af_lens_pos < 0x0000){
		af_lens_pos = 0x0000;
	}

	
	mf_pos.blank_id                   = CAMCMP_BID_MF_CUSTOM;
	mf_pos.p_blank_data               = &af_lens_pos;
	cmd_info_lens_mov[0].cmd_id       = CAMCMP_CID_MF_START;
	cmd_info_lens_mov[0].p_blank_info = &mf_pos;
	cmd_info_lens_mov[0].blank_num    = CAMCMP_SIZE_BLKINF(mf_pos);

	
	rc = camcmp_isp_trans(cmd_info_lens_mov, CAMCMP_SIZE_CMD(cmd_info_lens_mov));
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
		return -EFAULT;
	}

	
	msleep(132);  

	
	mt9e311_ctrl->af_mode = CAMERA_AF_MODE_NO_USE;

	
	cmd_info_pos[0].cmd_id = CAMCMP_CID_GET_LENSPOS;
	cmd_info_pos[0].p_recieve_data = &af_lens_pos;
	cmd_info_pos[0].recieve_num = 1;
	rc = camcmp_isp_trans(cmd_info_pos, CAMCMP_SIZE_CMD(cmd_info_pos));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}
	CAMCMP_LOG_DBG("af_lens_pos=0x%04x, \n", af_lens_pos);
	af_lens_pos = (af_lens_pos & 0x00FF);


	
	

	switch (sys_rev) {
		case MT9E311_SYS_REVISION_01:
			cmd_info_af_value[0].cmd_id = CAMCMP_CID_GET_EVALUATE;
			break;
		case MT9E311_SYS_REVISION_02:
			cmd_info_af_value[0].cmd_id = CAMCMP_CID_GET_EVALUATE_R2;
			break;
		default:
			CAMCMP_LOG_ERR("invalid system revision. sys_rev = 0x%04x\n", sys_rev);
			cmd_info_af_value[0].cmd_id = CAMCMP_CID_GET_EVALUATE_R2;
			break;
	}

	CAMCMP_LOG_ERR("invalid system revision. sys_rev = 0x%04x\n", sys_rev);
	cmd_info_af_value[0].p_recieve_data = &af_value;
	cmd_info_af_value[0].recieve_num = 2;

	rc = camcmp_isp_trans(cmd_info_af_value, CAMCMP_SIZE_CMD(cmd_info_af_value));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}
	CAMCMP_LOG_DBG("af_value[0]=0x%04x, \n", af_value[0]);
	CAMCMP_LOG_DBG("af_value[1]=0x%04x, \n", af_value[1]);

	
	set_manual_step->step_check = 1;
	set_manual_step->af_evaluate_0 = (uint8_t)(af_value[0] >> 8);
	set_manual_step->af_evaluate_1 = (uint8_t)(af_value[0] & 0x00FF);
	set_manual_step->af_evaluate_2 = (uint8_t)(af_value[1] >> 8);
	set_manual_step->af_evaluate_3 = (uint8_t)(af_value[1] & 0x00FF);
	

	
	set_manual_step->af_lens_pos_0 = 0x00;                               
	set_manual_step->af_lens_pos_1 = af_lens_pos;                        


	return rc;
}

















static int mt9e311_get_isp_id(struct get_isp_id_cfg *get_isp_id)
{
	int rc = 0;
	
	camcmp_cmd_info_type cmd_info[] = {{0, NULL, 0, NULL, 0}};
	uint32_t isp_id[4];

	
	cmd_info[0].cmd_id = CAMCMP_CID_GET_ISP_ID;
	cmd_info[0].p_recieve_data = isp_id;
	cmd_info[0].recieve_num = (sizeof(isp_id) / sizeof(uint32_t));

	
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
		return -EFAULT;
	}
	CAMCMP_LOG_DBG("device_id=0x%04X\n",          isp_id[0]);
	CAMCMP_LOG_DBG("manufacture_id=0x%04X\n",     isp_id[1]);
	CAMCMP_LOG_DBG("fw_revision_number=0x%04X\n", isp_id[2]);
	CAMCMP_LOG_DBG("revision_number=0x%04X\n",    isp_id[3]);

	
	get_isp_id->dev_id     = isp_id[0] & 0xFFFF;
	get_isp_id->manu_id    = isp_id[1] & 0xFFFF;
	get_isp_id->fw_rev_num = isp_id[2] & 0xFFFF;
	get_isp_id->rev_num    = isp_id[3] & 0xFFFF;

	return 0;
}



















static int mt9e311_set_isp_test_pat(int8_t test_pat_mode)
{
	int rc = 0;

	camcmp_cmd_info_type cmd_info[]  = {{0, NULL, 0, NULL, 0}};
	camcmp_blank_info_type set_test_pat = {0, NULL};
	camcmp_blank_isp_test_pat_enum mode = (camcmp_blank_isp_test_pat_enum)test_pat_mode;

	CAMCMP_LOG_DBG("start\n");

	CAMCMP_LOG_DBG("isp_test_pat_mode %d\n", mode);

	
	if ((mode < CAMCMP_BID_ISP_TEST_PAT_SOLID_COLOR_1) ||
			(mode > CAMCMP_BID_ISP_TEST_PAT_OFF)) {
		CAMCMP_LOG_ERR("parameter error root\n");
		return -EINVAL;
	}

	
	set_test_pat.blank_id = mode;

	cmd_info[0].cmd_id = CAMCMP_CID_ISP_TEST_PATTERN;
	cmd_info[0].p_blank_info = &set_test_pat;
	cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(set_test_pat);

	
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root\n");
		return -EFAULT;
	}

	CAMCMP_LOG_DBG("end\n");

	return rc;
}



















static int mt9e311_set_format_chg(int8_t format)
{
    int rc = 0;
	camcmp_cmd_info_type cmd_info[]  = {{0, NULL, 0, NULL, 0}};
	camcmp_blank_info_type cap_format   = {0, NULL};

	CAMCMP_LOG_DBG("mt9e311_set_format_chg. format= %d \n", format);
	switch(format){
		case 0:
			cap_format.blank_id = CAMCMP_BID_CAP_FORMAT_JPEG;
			break;
		case 1:
			cap_format.blank_id = CAMCMP_BID_CAP_FORMAT_YUV;
			break;
		case 2:
			cap_format.blank_id = CAMCMP_BID_CAP_FORMAT_RAW;
			break;
		default:
			cap_format.blank_id = CAMCMP_BID_CAP_FORMAT_JPEG;
			CAMCMP_LOG_ERR("mt9e311_set_format_chg.format error %d \n", format);
			break;
	}
	cmd_info[0].cmd_id = CAMCMP_CID_FORMAT_CHG;
	cmd_info[0].p_blank_info = &cap_format;
	cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(cap_format);

	
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
		return -EFAULT;
	}

   return rc;
}


















static int mt9e311_set_ae( struct ae_cfg ae )
{
	CAMCMP_LOG_DBG("ae_mode=%d \n", ae.ae_mode);
	CAMCMP_LOG_DBG("agc_gain=0x%04X \n", ae.user_preset.gain);
	CAMCMP_LOG_DBG("exposure_time=0x%08X \n", ae.user_preset.line);

	
	mt9e311_ctrl->ae_mode_next = ae.ae_mode;

	
	switch (ae.op_mode)
	{
		
		case SENSOR_PREVIEW_MODE:
		case SENSOR_MOVIE_MODE:
			mt9e311_ctrl->prev_exp_gain.gain = ae.user_preset.gain;
			mt9e311_ctrl->prev_exp_gain.line = ae.user_preset.line;
			break;

		
		case SENSOR_SNAPSHOT_MODE:
		case SENSOR_RAW_SNAPSHOT_MODE:
			mt9e311_ctrl->cap_exp_gain.gain = ae.user_preset.gain;
			mt9e311_ctrl->cap_exp_gain.line = ae.user_preset.line;
			break;

		default:
			CAMCMP_LOG_ERR("input failed! mode=%d\n", ae.op_mode);
			
			return -EINVAL;
	}

	return 0;
}

























static int mt9e311_set_default_focus(int prev_mode)
{
	int rc = 0;
	camcmp_cmd_info_type cmd_info;
	uint16_t scene_mode = 0;
	uint16_t lens_pos = 0;
	uint16_t req_lens_pos = 0;
	int af_mode = 0;
	uint16_t read_scene_mode = 0;

	
	if ((prev_mode != SENSOR_MOVIE_MODE) && (prev_mode != SENSOR_PREVIEW_MODE)) {
		
		return rc;
	}

	
	memset(&cmd_info, 0, sizeof(cmd_info));

	cmd_info.cmd_id = CAMCMP_CID_GET_SCENE_MODE;
	cmd_info.p_recieve_data = &scene_mode;
	cmd_info.recieve_num = 1;

	
	rc = camcmp_isp_trans(&cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
		return -EFAULT;
	}

	
	read_scene_mode = scene_mode_rd_to_set( scene_mode );

	
	if (prev_mode == SENSOR_PREVIEW_MODE) {
		switch (read_scene_mode)
		{
			case CAMERA_SCENE_MODE_OFF:     		
			case CAMERA_SCENE_MODE_PORTRAIT:		
			case CAMERA_SCENE_MODE_PARTY:			
			case CAMERA_SCENE_MODE_BACKLIGHT:		
				af_mode = CAMERA_AF_MODE_NORMAL;	
				req_lens_pos = g_mt9e311_sd_read[CAMCMP_PAN_NORMAL_REG].val;
				break;
			case CAMERA_SCENE_MODE_LANDSCAPE:		
			case CAMERA_SCENE_MODE_NIGHT:			
			case CAMERA_SCENE_MODE_SPORTS:			
				af_mode = CAMERA_AF_MODE_INFINITY;	
				req_lens_pos = 0x00;
				break;
			case CAMERA_SCENE_MODE_FOOD:			
				af_mode = CAMERA_AF_MODE_MACRO;		
				req_lens_pos = 0xFF;
				break;
			default:
				CAMCMP_LOG_ERR("invalid scene mode. scene_mode = %d\n", (read_scene_mode & 0x000F));
				af_mode = CAMERA_AF_MODE_INFINITY; 	
				req_lens_pos = 0x00;
				break;
		}
	
	} else if (prev_mode == SENSOR_MOVIE_MODE) {
		switch (read_scene_mode)
		{
			case CAMERA_SCENE_MODE_OFF:				
			case CAMERA_SCENE_MODE_PORTRAIT:		
			case CAMERA_SCENE_MODE_PARTY:			
			case CAMERA_SCENE_MODE_BACKLIGHT:		
			case CAMERA_SCENE_MODE_FOOD:			
				af_mode = CAMERA_AF_MODE_NORMAL;	
				req_lens_pos = g_mt9e311_sd_read[CAMCMP_PAN_NORMAL_REG].val;
				break;
			case CAMERA_SCENE_MODE_LANDSCAPE:		
			case CAMERA_SCENE_MODE_NIGHT:			
			case CAMERA_SCENE_MODE_SPORTS:			
				af_mode = CAMERA_AF_MODE_INFINITY;	
				req_lens_pos = 0x00;
				break;
			default:
				CAMCMP_LOG_ERR("invalid scene mode. scene_mode = %d\n", (read_scene_mode & 0x000F));
				af_mode = CAMERA_AF_MODE_INFINITY; 	
				req_lens_pos = 0x00;
				break;
		}
	} else {
		
		CAMCMP_LOG_ERR("invalid scene mode. scene_mode = %d\n", (read_scene_mode & 0x000F));
		af_mode = CAMERA_AF_MODE_INFINITY; 			
		req_lens_pos = 0x00;
	}

	
	memset(&cmd_info, 0, sizeof(cmd_info));

	cmd_info.cmd_id = CAMCMP_CID_GET_LENSPOS;
	cmd_info.p_recieve_data = &lens_pos;
	cmd_info.recieve_num = 1;
	
	rc = camcmp_isp_trans(&cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
		return -EFAULT;
	}

	if( (lens_pos & 0x00FF) == req_lens_pos ) {
		
		mt9e311_ctrl->af_mode = af_mode;
		CAMCMP_LOG_DBG("request mode is equal to existing mode. af_mode = %d", af_mode );
	} else {
		
		mt9e311_ctrl->af_mode = CAMERA_AF_MODE_NO_USE;

		
		rc = mt9e311_set_af_pos(af_mode);
		if (rc < 0) {
			CAMCMP_LOG_ERR("mt9e311_set_af_pos failed. rc = %d\n", rc);
			return -EFAULT;
		}
	}

	return rc;
}
































static int mt9e311_af_start(struct msm_sync *sync)
{
	int rc = 0;
	uint16_t scene_mode = 0;
	int af_mode = 0;
	camcmp_cmd_info_type cmd_info;
	uint16_t read_scene_mode = 0;


	CAMCMP_LOG_DBG("start\n");

	
	memset(&cmd_info, 0, sizeof(cmd_info));

	cmd_info.cmd_id = CAMCMP_CID_GET_SCENE_MODE;
	cmd_info.p_recieve_data = &scene_mode;
	cmd_info.recieve_num = 1;

	
	rc = camcmp_isp_trans(&cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
		return -EFAULT;
	}

	
	read_scene_mode = scene_mode_rd_to_set( scene_mode );

	switch (read_scene_mode)
	{
		case CAMERA_SCENE_MODE_OFF: 		
		case CAMERA_SCENE_MODE_PORTRAIT:	
		case CAMERA_SCENE_MODE_PARTY:		
		case CAMERA_SCENE_MODE_BACKLIGHT:	
			af_mode = CAMERA_AF_MODE_NORMAL;
			break;
		case CAMERA_SCENE_MODE_LANDSCAPE:	
		case CAMERA_SCENE_MODE_NIGHT:		
		case CAMERA_SCENE_MODE_SPORTS:		
			af_mode = CAMERA_AF_MODE_INFINITY;
			break;
		case CAMERA_SCENE_MODE_FOOD:		
			af_mode = CAMERA_AF_MODE_MACRO;
			break;
		default:
			CAMCMP_LOG_ERR("invalid scene mode. scene_mode = %d\n", (scene_mode & 0x000F));
			af_mode = CAMERA_AF_MODE_INFINITY;
			break;
	}

	
	rc = mt9e311_set_af_mode_off(af_mode);
	if (rc < 0) {
		return -EFAULT;
	}

	
	rc = mt9e311_set_af_mode_on(sync);
	if (rc < 0) {
		return -EFAULT;
	}


	CAMCMP_LOG_DBG("end rc = %d\n",rc);

	return rc;
}

























static int mt9e311_af_start_after(struct msm_sync *sync)
{
	int rc = 0;

	uint32_t af_lens_pos = 0;
	uint32_t af_s1_s2_state = 0;
	camcmp_cmd_info_type cmd_info[] = {{0, NULL, 0, NULL, 0}};
	camcmp_cmd_info_type cmd_info2[]  = {{0, NULL, 0, NULL, 0}};









	CAMCMP_LOG_DBG("start\n");

	
	if ((mt9e311_ctrl->af_cancel_onoff) == SENSOR_AF_ON) {
		CAMCMP_LOG_ERR("cancel. cancel = %d\n", mt9e311_ctrl->af_cancel_onoff);
		
		mt9e311_ctrl->af_cancel_onoff = SENSOR_AF_OFF;
		af_start_after_cnt = 0;
		return AF_RESULT_NG;
	}

	
	af_start_after_cnt++;
	if (af_start_after_cnt > MT9E311_CHECK_STATE_WAITMAX){
		CAMCMP_LOG_ERR("cut over. cnt = %d\n", af_start_after_cnt);
		af_start_after_cnt = 0;
		return AF_RESULT_NG;
	}


	
	cmd_info[0].cmd_id = CAMCMP_CID_GET_AF_S1_S2_STATE;
	cmd_info[0].p_recieve_data = &af_s1_s2_state;
	cmd_info[0].recieve_num = 1;
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}
	CAMCMP_LOG_DBG("af_s1_s2_state=0x%04x\n", af_s1_s2_state);
	if((af_s1_s2_state & 0x0010) != 0x0000){
		
		CAMCMP_LOG_DBG("AF_CHK_POOLING\n");
		
		timer.expires = jiffies + MT9E311_CHECK_STATE_WAIT;
		timer.data=(unsigned long)sync;
		timer.function = msm_af_start_poll;
		add_timer(&timer);
		rc = AF_RESULT_DURING;
	}else{
		
		af_start_after_cnt =0;
		msleep(132);  

		
		cmd_info2[0].cmd_id = CAMCMP_CID_GET_LENSPOS;
		cmd_info2[0].p_recieve_data = &af_lens_pos;
		cmd_info2[0].recieve_num = 1;
		rc = camcmp_isp_trans(cmd_info2, CAMCMP_SIZE_CMD(cmd_info2));
		if (rc < 0) {
			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
			return -EFAULT;
		}
		CAMCMP_LOG_DBG("af_lens_pos=0x%04x, \n", af_lens_pos);
		if (sys_rev == MT9E311_SYS_REVISION_02) {
			if ((af_lens_pos & 0xFF00) == 0x0F00) {
				
				CAMCMP_LOG_DBG("AF NG\n");
				rc = AF_RESULT_NG;
			} else {
				
				CAMCMP_LOG_DBG("AF OK\n");
				rc = AF_RESULT_OK;
			}
		} else {
			if ((af_lens_pos & 0xFF00) == 0xFF00) {
				
				CAMCMP_LOG_DBG("AF NG\n");
				rc = AF_RESULT_NG;
			} else {
				
				CAMCMP_LOG_DBG("AF OK\n");
				rc = AF_RESULT_OK;
			}
		}
	}

















































	CAMCMP_LOG_DBG("end rc = %d\n",rc);
	return rc;
}


















static int mt9e311_af_cancel(void)
{
	int rc = 0;

	CAMCMP_LOG_DBG("start\n");

	if(mt9e311_ctrl->af_exec_flag == SENSOR_ON) {
		
		mt9e311_ctrl->af_cancel_onoff = SENSOR_AF_ON;
	} else {
		
		rc = mt9e311_af_stop();
		if ( rc != 0 ) {
			CAMCMP_LOG_ERR("mt9e311_af_stop failed. rc = %d\n", rc);
			return -EFAULT;
		}
	}

	CAMCMP_LOG_DBG("end rc = %d\n",rc);

	return rc;
}


















static int mt9e311_af_stop(void)
{
	int rc = 0;
	uint32_t blank_id;
	camcmp_cmd_info_type cmd_info;


	
	memset( &cmd_info, 0, sizeof(cmd_info) );

	cmd_info.cmd_id = CAMCMP_CID_AF_S1_S2_STOP;
	rc = camcmp_isp_trans(&cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
		return -EFAULT;
	}


	
	switch (sys_rev) {
		case MT9E311_SYS_REVISION_01:
			blank_id = CAMCMP_BID_MODE_MF;
			break;
		case MT9E311_SYS_REVISION_02:
			blank_id = CAMCMP_BID_MODE_MF_R2;
			break;
		default:
			CAMCMP_LOG_ERR("invalid system revision. sys_rev = 0x%04x\n", sys_rev);
			blank_id = CAMCMP_BID_MODE_MF_R2;
			break;
	}
	rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE,
								CAMCMP_CID_AF_MODE,
								g_mt9e311_af_mode[blank_id].mask,
								g_mt9e311_af_mode[blank_id].data );
	if (rc < 0) {
		CAMCMP_LOG_ERR("i2c_read_write_modify failed. rc = %d\n", rc);
		return -EFAULT;
	}

	
	rc = mt9e311_set_default_focus(SENSOR_PREVIEW_MODE);
	if (rc < 0) {
		CAMCMP_LOG_ERR("mt9e311_set_default_focus failed. rc = %d\n", rc);
		return -EFAULT;
	}
	CAMCMP_LOG_DBG("end rc = %d\n",rc);
	return rc;
}


















static void mt9e311_af_status_change(sensor_af_status_t next_af_status)
{
  
  if(mt9e311_ctrl->afc_mode != next_af_status)
  {
    CAMCMP_LOG_DBG("mt9e311_af_status_change %d -> %d\n", mt9e311_ctrl->afc_mode, next_af_status);
    mt9e311_ctrl->afc_mode = next_af_status;
  }
}
















static int mt9e311_afc_stop(void)
{
	int rc = 0;

	CAMCMP_LOG_DBG("mt9e311_afc_stop\n");
	
	
	if(mt9e311_ctrl->afc_mode == SENSOR_AF_STATUS_MOVE)
	{
		
		rc = mt9e311_set_conti_af_mode(SENSOR_AF_CONTI_OFF);
		if (rc < 0) {
			CAMCMP_LOG_ERR("mt9e311_set_conti_af_mode failed. rc = %d\n", rc);
			return -EFAULT;
		}

		msleep(132);  

		
		rc = mt9e311_set_default_focus(SENSOR_MOVIE_MODE);
		if (rc < 0) {
			CAMCMP_LOG_ERR("mt9e311_set_default_focus failed. rc = %d\n", rc);
			return -EFAULT;
		}

		
		mt9e311_af_status_change(SENSOR_AF_STATUS_IDLE);
	}
	
	else
	{
		
	}

	return rc;
}































































static int mt9e311_afc_start(void)
{
    int rc = 0;
    camcmp_cmd_info_type cmd_info;
    camcmp_blank_info_type blank_info;
    uint32_t af_start_val = 0;
    uint32_t af_stop_val = 0;

    CAMCMP_LOG_DBG("mt9e311_afc_start\n");

    if(mt9e311_ctrl->afc_mode == SENSOR_AF_STATUS_IDLE)
    {
        
        rc = mt9e311_set_conti_af_mode(SENSOR_AF_CONTI_OFF);
        if (rc < 0) {
            CAMCMP_LOG_ERR("mt9e311_set_conti_af_mode failed. rc = %d\n", rc);
            return -EFAULT;
        }

        
        memset(&cmd_info, 0, sizeof(cmd_info));
        
        cmd_info.cmd_id = CAMCMP_CID_AF_WINDOW_MOVIE_SET;
        
        rc = camcmp_isp_trans(&cmd_info, CAMCMP_SIZE_CMD(cmd_info));
        if (rc < 0) {
            CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
            return -EFAULT;
        }

        
        memset(&cmd_info, 0, sizeof(cmd_info));
        memset(&blank_info, 0, sizeof(blank_info));

        af_start_val = g_mt9e311_sd_read[CAMCMP_AF_START].val;
        blank_info.blank_id = CAMCMP_BID_AFS_INF_VAL;
        blank_info.p_blank_data = &af_start_val;
        cmd_info.cmd_id = CAMCMP_CID_AF_SEARCH_INF;
        cmd_info.p_blank_info = &blank_info;
        cmd_info.blank_num = CAMCMP_SIZE_BLKINF(blank_info);

        
        rc = camcmp_isp_trans(&cmd_info, CAMCMP_SIZE_CMD(cmd_info));
        if (rc < 0) {
            CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
            return -EFAULT;
        }

        memset(&cmd_info, 0, sizeof(cmd_info));
        memset(&blank_info, 0, sizeof(blank_info));

        af_stop_val = g_mt9e311_sd_read[CAMCMP_AF_STOP].val;

        blank_info.blank_id = CAMCMP_BID_AFS_MAC_VAL;
        cmd_info.cmd_id = CAMCMP_CID_AF_SEARCH_MAC;
        blank_info.p_blank_data = &af_stop_val;
        cmd_info.p_blank_info = &blank_info;
        cmd_info.blank_num = CAMCMP_SIZE_BLKINF(blank_info);

        
        rc = camcmp_isp_trans(&cmd_info, CAMCMP_SIZE_CMD(cmd_info));
        if (rc < 0) {
            CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
            return -EFAULT;
        }

        
        rc = mt9e311_set_conti_af_mode(SENSOR_AF_CONTI_ON);
        if (rc < 0) {
            CAMCMP_LOG_ERR("mt9e311_set_conti_af_mode failed. rc = %d\n", rc);
            return -EFAULT;
        }
        
        
        mt9e311_ctrl->af_mode = CAMERA_AF_MODE_NO_USE;
        
        
        mt9e311_af_status_change(SENSOR_AF_STATUS_MOVE);
    } else {
        
    }

    return rc;
}





















static int mt9e311_set_brightness(int8_t brightness)
{
	int rc = 0;

	camcmp_cmd_info_type cmd_info[]  = {{0, NULL, 0, NULL, 0}};
	camcmp_blank_info_type set_brightness = {0, NULL};

	CAMCMP_LOG_DBG("brightness=%x \n", brightness);
	
	

	if ((brightness < CAMCMP_BID_BRIGHT_M6) || (brightness > CAMCMP_BID_BRIGHT_P6))



	{
		CAMCMP_LOG_ERR("param error. brightness=%d\n", brightness);
		return -EINVAL;
	}

	
	
	set_brightness.blank_id = (camcmp_blank_bright_enum)brightness;
	cmd_info[0].cmd_id = CAMCMP_CID_BRIGHTNESS;
	cmd_info[0].p_blank_info = &set_brightness;
	cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(set_brightness);

	
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}

	return rc;
}

















static int mt9e311_set_effect(int8_t effect)
{
	int rc = 0;
	camcmp_cmd_info_type cmd_info[]  = {{0, NULL, 0, NULL, 0}};
	camcmp_blank_info_type set_effect = {0, NULL};

	CAMCMP_LOG_DBG("effect=%x \n", effect);

	
	switch (effect)
	{
		case CAMERA_EFFECT_OFF:			
			set_effect.blank_id = CAMCMP_BID_EFFECT_OFF;
			break;
 
		case CAMERA_EFFECT_MONO:		
			set_effect.blank_id = CAMCMP_BID_EFFECT_GRAYSCALE;
			break;
		case CAMERA_EFFECT_NEGATIVE:		
			set_effect.blank_id = CAMCMP_BID_EFFECT_NEGATIVE;
			break;
		case CAMERA_EFFECT_SOLARIZE:		
			set_effect.blank_id = CAMCMP_BID_EFFECT_SOLARIZE;
			break;
		case CAMERA_EFFECT_SEPIA:		
			set_effect.blank_id = CAMCMP_BID_EFFECT_SEPHIA1;
			break;
		case CAMERA_EFFECT_POSTERIZE:		
			set_effect.blank_id = CAMCMP_BID_EFFECT_POSTERIZE1;
			break;
		case CAMERA_EFFECT_ALIEN:		
			set_effect.blank_id = CAMCMP_BID_EFFECT_ALIEN;
			break;
		case CAMERA_EFFECT_ANTIQUE:		
			set_effect.blank_id = CAMCMP_BID_EFFECT_ANTIQUE;
			break;
		case CAMERA_EFFECT_BLACK_AND_WHITE:	
			set_effect.blank_id = CAMCMP_BID_EFFECT_BLACK_AND_WHITE;
			break;
		case CAMERA_EFFECT_EMBOSS:		
			set_effect.blank_id = CAMCMP_BID_EFFECT_EMBOSS;
			break;
		case CAMERA_EFFECT_EMBOSS_COLORED:	
			set_effect.blank_id = CAMCMP_BID_EFFECT_EMBOSS_COLORED;
			break;
		case CAMERA_EFFECT_BLUEISH:		
			set_effect.blank_id = CAMCMP_BID_EFFECT_BLUEISH;
			break;
		case CAMERA_EFFECT_GREENISH:		
			set_effect.blank_id = CAMCMP_BID_EFFECT_GREENISH;
			break;
		case CAMERA_EFFECT_REDISH:		
			set_effect.blank_id = CAMCMP_BID_EFFECT_REDISH;
			break;
		case CAMERA_EFFECT_POSTERIZE2:		
			set_effect.blank_id = CAMCMP_BID_EFFECT_POSTERIZE2;
			break;
		case CAMERA_EFFECT_SEPHIA2:		
			set_effect.blank_id = CAMCMP_BID_EFFECT_SEPHIA2;
			break;
		case CAMERA_EFFECT_SKETCH:		
			set_effect.blank_id = CAMCMP_BID_EFFECT_SKETCH;
			break;
		case CAMERA_EFFECT_FOGGY:		
			set_effect.blank_id = CAMCMP_BID_EFFECT_FOGGY;
			break;

		default:
			CAMCMP_LOG_DBG("param error. default\n");
			return -EINVAL;
	}

	
	
	cmd_info[0].cmd_id = CAMCMP_CID_EFFECT;
	cmd_info[0].p_blank_info = &set_effect;
	cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(set_effect);

	
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}

	return rc;
}



















static int32_t mt9e311_set_zoom(struct set_zoom_cfg *set_zoom, int size)



{
	int32_t rc = 0;


	camcmp_cmd_info_type cmd_info[]  = {{0, NULL, 0, NULL, 0}};
	camcmp_blank_info_type zoom_blank = {0, NULL};


	CAMCMP_LOG_DBG("start\n");

	
	if(set_zoom->mode > ZOOM_MOVIE){
		CAMCMP_LOG_ERR("zoom mode failed.\n");
		return -EINVAL;
	}


	CAMCMP_LOG_DBG("zoom = %d, mode = %d, size = %d \n",
						set_zoom->zoom, set_zoom->mode, size);

	if(set_zoom->mode == ZOOM_PICTURE){				
		
		if( (size == SENSOR_CAMERA_8MP_SIZE) ||
			(size == SENSOR_CAMERA_6MW_SIZE)) {
			if (set_zoom->zoom != 0) {
				CAMCMP_LOG_ERR("zoom param err\n");
				return -EINVAL;
			}
		}else{
			if ((set_zoom->zoom < 0) || (set_zoom->zoom > 9)) {
				CAMCMP_LOG_ERR("zoom param err\n");
				return -EINVAL;
			}
		}
		
		switch (size)
		{
			case SENSOR_CAMERA_8MP_SIZE:			
				cmd_info[0].cmd_id = CAMCMP_CID_ZOOM_8M;
				break;
			case SENSOR_CAMERA_6MW_SIZE:			
				cmd_info[0].cmd_id = CAMCMP_CID_ZOOM_6MW;
				break;
			case SENSOR_CAMERA_5MP_SIZE:			
				cmd_info[0].cmd_id = CAMCMP_CID_ZOOM_5M;
				break;
			case SENSOR_CAMERA_3MP_SIZE:			
				cmd_info[0].cmd_id = CAMCMP_CID_ZOOM_3M;
				break;
			case SENSOR_CAMERA_2MP_SIZE:			
				cmd_info[0].cmd_id = CAMCMP_CID_ZOOM_2M;
				break;
			case SENSOR_CAMERA_1MP_SIZE:			
				cmd_info[0].cmd_id = CAMCMP_CID_ZOOM_1M;
				break;
			case SENSOR_CAMERA_WVGA_SIZE:			
				cmd_info[0].cmd_id = CAMCMP_CID_ZOOM_WVGA;
				break;
			case SENSOR_CAMERA_VGA_SIZE:			
				cmd_info[0].cmd_id = CAMCMP_CID_ZOOM_VGA;
				break;
			case SENSOR_CAMERA_QVGA_SIZE:			
				cmd_info[0].cmd_id = CAMCMP_CID_ZOOM_QVGA;
				break;
			default:
				CAMCMP_LOG_ERR("size failed.\n");
				return -EINVAL;
		}
	}else{											
		
		if((set_zoom->zoom < 0) || (set_zoom->zoom > 9)){
				CAMCMP_LOG_ERR("zoom param err\n");
				return -EINVAL;
		}

		
		switch (size)
		{
			case SENSOR_CAMERA_HD_SIZE:				
				cmd_info[0].cmd_id = CAMCMP_CID_ZOOM_HD_MOVIE;
				break;
			case SENSOR_CAMERA_VGA_SIZE:			
				cmd_info[0].cmd_id = CAMCMP_CID_ZOOM_VGA_MOVIE;
				break;
			case SENSOR_CAMERA_QVGA_SIZE:			
				cmd_info[0].cmd_id = CAMCMP_CID_ZOOM_QVGA_MOVIE;
				break;
			case SENSOR_CAMERA_QCIF_SIZE:			
				cmd_info[0].cmd_id = CAMCMP_CID_ZOOM_QCIF_MOVIE;
				break;
			default:
				CAMCMP_LOG_ERR("size failed.\n");
				return -EINVAL;
		}
	}

	
	zoom_blank.blank_id = (camcmp_blank_zoom_enum)set_zoom->zoom;

































































	cmd_info[0].p_blank_info = &zoom_blank;
	cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(zoom_blank);

	
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
		return -EFAULT;
	}


	CAMCMP_LOG_DBG("end rc = %d\n", rc);

	return rc;
}

















static int32_t mt9e311_set_picture_mode(int8_t pict_mode)
{

    int rc = 0;

    switch (pict_mode)
    {
            case CAMERA_SCENE_MODE_OFF:                    

                
                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_SCENE_MODE, CAMCMP_CID_SET_BESTSHOT, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_NORMAL].mask, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_NORMAL].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_ISO, CAMCMP_CID_ISO, 
				                            g_mt9e311_aemode_info[CAMCMP_BID_AE_MODE_OFF].mask, 
				                            g_mt9e311_aemode_info[CAMCMP_BID_AE_MODE_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE, CAMCMP_CID_SET_AF_MODE_SETTING, 
				                            g_mt9e311_afmode_info[CAMCMP_BID_AF_MODE_OFF].mask, 
				                            g_mt9e311_afmode_info[CAMCMP_BID_AF_MODE_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_PREVCTL, CAMCMP_CID_SET_PREVCTL, 
				                            g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_OFF].mask, 
				                            g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_OFF].data );

                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_WDR, CAMCMP_CID_SET_WDR, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_OFF].mask, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_OFF].data );

   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}

                break;

            case CAMERA_SCENE_MODE_PORTRAIT:                
                
                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_SCENE_MODE, CAMCMP_CID_SET_BESTSHOT, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_PORTRAIT].mask, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_PORTRAIT].data );

                if(mt9e311_ctrl->sensormode == SENSOR_MOVIE_MODE){
					
					rc = i2c_read_write_modify( CAMCMP_CID_GET_PREVCTL, CAMCMP_CID_SET_PREVCTL, 
				    	                        g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_OFF].mask, 
				        	                    g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_OFF].data );
				}
				else{
					
					rc = i2c_read_write_modify( CAMCMP_CID_GET_PREVCTL, CAMCMP_CID_SET_PREVCTL, 
				    	                        g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_ON].mask, 
				        	                    g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_ON].data );
				}

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_ISO, CAMCMP_CID_ISO, 
				                            g_mt9e311_aemode_info[CAMCMP_BID_AE_MODE_OFF].mask, 
				                            g_mt9e311_aemode_info[CAMCMP_BID_AE_MODE_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE, CAMCMP_CID_SET_AF_MODE_SETTING, 
				                            g_mt9e311_afmode_info[CAMCMP_BID_AF_MODE_ON].mask, 
				                            g_mt9e311_afmode_info[CAMCMP_BID_AF_MODE_ON].data );

                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_WDR, CAMCMP_CID_SET_WDR, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_OFF].mask, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_OFF].data );

   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}

                break;

            case CAMERA_SCENE_MODE_LANDSCAPE:               
                
                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_SCENE_MODE, CAMCMP_CID_SET_BESTSHOT, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_LANDSCAPE].mask, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_LANDSCAPE].data );

                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_WDR, CAMCMP_CID_SET_WDR, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_OFF].mask, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_PREVCTL, CAMCMP_CID_SET_PREVCTL, 
				                            g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_OFF].mask, 
				                            g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_ISO, CAMCMP_CID_ISO, 
				                            g_mt9e311_aemode_info[CAMCMP_BID_AE_MODE_OFF].mask, 
				                            g_mt9e311_aemode_info[CAMCMP_BID_AE_MODE_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE, CAMCMP_CID_SET_AF_MODE_SETTING, 
				                            g_mt9e311_afmode_info[CAMCMP_BID_AF_MODE_OFF].mask, 
				                            g_mt9e311_afmode_info[CAMCMP_BID_AF_MODE_OFF].data );

   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}

                break;

            case CAMERA_SCENE_MODE_SPORTS:                  
                
                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_SCENE_MODE, CAMCMP_CID_SET_BESTSHOT, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_SPORT].mask, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_SPORT].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_PREVCTL, CAMCMP_CID_SET_PREVCTL, 
				                            g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_OFF].mask, 
				                            g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_ISO, CAMCMP_CID_ISO, 
				                            g_mt9e311_aemode_info[CAMCMP_BID_AE_MODE_OFF].mask, 
				                            g_mt9e311_aemode_info[CAMCMP_BID_AE_MODE_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE, CAMCMP_CID_SET_AF_MODE_SETTING, 
				                            g_mt9e311_afmode_info[CAMCMP_BID_AF_MODE_OFF].mask, 
				                            g_mt9e311_afmode_info[CAMCMP_BID_AF_MODE_OFF].data );
                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_WDR, CAMCMP_CID_SET_WDR, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_OFF].mask, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_OFF].data );

   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}

                break;

            case CAMERA_SCENE_MODE_CLOSEUP:                 
				rc = i2c_read_write_modify( CAMCMP_CID_GET_SCENE_MODE, CAMCMP_CID_SET_BESTSHOT, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_CLOSEUP].mask, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_CLOSEUP].data );
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;
            case CAMERA_SCENE_MODE_NIGHT:                   
                
                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_SCENE_MODE, CAMCMP_CID_SET_BESTSHOT, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_NIGHT].mask, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_NIGHT].data );

                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_WDR, CAMCMP_CID_SET_WDR, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_OFF].mask, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_PREVCTL, CAMCMP_CID_SET_PREVCTL, 
				                            g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_OFF].mask, 
				                            g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_ISO, CAMCMP_CID_ISO, 
				                            g_mt9e311_aemode_info[CAMCMP_BID_AE_MODE_OFF].mask, 
				                            g_mt9e311_aemode_info[CAMCMP_BID_AE_MODE_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE, CAMCMP_CID_SET_AF_MODE_SETTING, 
				                            g_mt9e311_afmode_info[CAMCMP_BID_AF_MODE_OFF].mask, 
				                            g_mt9e311_afmode_info[CAMCMP_BID_AF_MODE_OFF].data );
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;

            case CAMERA_SCENE_MODE_TWILIGHT:                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_SCENE_MODE, CAMCMP_CID_SET_BESTSHOT, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_TWILIGHT].mask, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_TWILIGHT].data );
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;
            case CAMERA_SCENE_MODE_BACKLIGHT:               
                
                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_SCENE_MODE, CAMCMP_CID_SET_BESTSHOT, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_NORMAL].mask, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_NORMAL].data );

                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_WDR, CAMCMP_CID_SET_WDR, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_ON].mask, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_ON].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_PREVCTL, CAMCMP_CID_SET_PREVCTL, 
				                            g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_OFF].mask, 
				                            g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_ISO, CAMCMP_CID_ISO, 
				                            g_mt9e311_aemode_info[CAMCMP_BID_AE_MODE_OFF].mask, 
				                            g_mt9e311_aemode_info[CAMCMP_BID_AE_MODE_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE, CAMCMP_CID_SET_AF_MODE_SETTING, 
				                            g_mt9e311_afmode_info[CAMCMP_BID_AF_MODE_OFF].mask, 
				                            g_mt9e311_afmode_info[CAMCMP_BID_AF_MODE_OFF].data );
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;
            case CAMERA_SCENE_MODE_HIGH_SENSITIVE_MODE:     
				rc = i2c_read_write_modify( CAMCMP_CID_GET_SCENE_MODE, CAMCMP_CID_SET_BESTSHOT, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_HIGH_SENSITIVE_MODE].mask, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_HIGH_SENSITIVE_MODE].data );
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;
            case CAMERA_SCENE_MODE_NIGHT_PORTRAIT:          
				rc = i2c_read_write_modify( CAMCMP_CID_GET_SCENE_MODE, CAMCMP_CID_SET_BESTSHOT, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_NIGHT_PORTRAIT].mask, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_NIGHT_PORTRAIT].data );
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;
            case CAMERA_SCENE_MODE_BEACH:                   
				rc = i2c_read_write_modify( CAMCMP_CID_GET_SCENE_MODE, CAMCMP_CID_SET_BESTSHOT, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_BEACH].mask, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_BEACH].data );
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;
            case CAMERA_SCENE_MODE_FOOD:                   
                
                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_SCENE_MODE, CAMCMP_CID_SET_BESTSHOT, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_CLOSEUP].mask, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_CLOSEUP].data );

                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_WDR, CAMCMP_CID_SET_WDR, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_OFF].mask, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_PREVCTL, CAMCMP_CID_SET_PREVCTL, 
				                            g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_FACE_OFF].mask, 
				                            g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_FACE_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_ISO, CAMCMP_CID_ISO, 
				                            g_mt9e311_aemode_info[CAMCMP_BID_AE_MODE_OFF].mask, 
				                            g_mt9e311_aemode_info[CAMCMP_BID_AE_MODE_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE, CAMCMP_CID_SET_AF_MODE_SETTING, 
				                            g_mt9e311_afmode_info[CAMCMP_BID_AF_MODE_OFF].mask, 
				                            g_mt9e311_afmode_info[CAMCMP_BID_AF_MODE_OFF].data );
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;

            case CAMERA_SCENE_MODE_PARTY:                   
                
                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_SCENE_MODE, CAMCMP_CID_SET_BESTSHOT, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_PARTY].mask, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_PARTY].data );

                if(mt9e311_ctrl->sensormode == SENSOR_MOVIE_MODE){
					
					rc = i2c_read_write_modify( CAMCMP_CID_GET_PREVCTL, CAMCMP_CID_SET_PREVCTL, 
				    	                        g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_OFF].mask, 
				        	                    g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_OFF].data );
				}
				else{
					
					rc = i2c_read_write_modify( CAMCMP_CID_GET_PREVCTL, CAMCMP_CID_SET_PREVCTL, 
				    	                        g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_ON].mask, 
				        	                    g_mt9e311_prevctl_info[CAMCMP_BID_PREV_CTL_ON].data );
				}

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_ISO, CAMCMP_CID_ISO, 
				                            g_mt9e311_aemode_info[CAMCMP_BID_AE_MODE_OFF].mask, 
				                            g_mt9e311_aemode_info[CAMCMP_BID_AE_MODE_OFF].data );

				
				rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE, CAMCMP_CID_SET_AF_MODE_SETTING, 
				                            g_mt9e311_afmode_info[CAMCMP_BID_AF_MODE_ON].mask, 
				                            g_mt9e311_afmode_info[CAMCMP_BID_AF_MODE_ON].data );

                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_WDR, CAMCMP_CID_SET_WDR, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_OFF].mask, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_OFF].data );
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;

            case CAMERA_SCENE_MODE_FIREWORKS:               
				rc = i2c_read_write_modify( CAMCMP_CID_GET_SCENE_MODE, CAMCMP_CID_SET_BESTSHOT, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_FIREWORKS].mask, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_FIREWORKS].data );
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;
            case CAMERA_SCENE_MODE_SUNSET:                  
				rc = i2c_read_write_modify( CAMCMP_CID_GET_SCENE_MODE, CAMCMP_CID_SET_BESTSHOT, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_SUNSET].mask, 
				                            g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_SUNSET].data );
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;
            default:
                    CAMCMP_LOG_DBG("param error. default\n");
                    return -EINVAL;
    }

    
    rc = mt9e311_set_scene_tbl(pict_mode);

   	if (rc < 0) {
       CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
       return -EFAULT;
  	}

    
    if (sys_rev == MT9E311_SYS_REVISION_02) {
        
        rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE,
                                    CAMCMP_CID_AF_MODE,
                                    g_mt9e311_af_mode[CAMCMP_BID_MODE_MF_R2].mask,
                                    g_mt9e311_af_mode[CAMCMP_BID_MODE_MF_R2].data );
        if (rc != 0) {
            CAMCMP_LOG_ERR("i2c_read_write_modify failed. rc = %d\n", rc);
            return -EFAULT;
        }
        msleep(132);  
    }

    
    rc = mt9e311_set_default_focus(mt9e311_ctrl->sensormode);
    if (rc < 0) {
        CAMCMP_LOG_ERR("mt9e311_set_default_focus failed. rc = %d\n", rc);
        return -EFAULT;
    }

    if (sys_rev == MT9E311_SYS_REVISION_02) {
        
        if (mt9e311_ctrl->afc_mode == SENSOR_AF_STATUS_MOVE) {
            
            rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE,
                                        CAMCMP_CID_AF_MODE,
                                        g_mt9e311_af_mode[CAMCMP_BID_MODE_CAF_R2].mask,
                                        g_mt9e311_af_mode[CAMCMP_BID_MODE_CAF_R2].data );
            if (rc != 0) {
                CAMCMP_LOG_ERR("i2c_read_write_modify failed. rc = %d\n", rc);
                return -EFAULT;
            }
        }
        
        mt9e311_ctrl->af_mode = CAMERA_AF_MODE_NO_USE;
    }

	return rc;
}

















static int mt9e311_set_wb(int8_t wb)
{

        int rc = 0;
		uint32_t scene_mode_addr = 0;
		uint32_t wdr_mode_addr = 0;
		camcmp_cmd_info_type cmd_info_manual[] = {{0, NULL, 0, NULL, 0},{0, NULL, 0, NULL, 0},{0, NULL, 0, NULL, 0},{0, NULL, 0, NULL, 0},{0, NULL, 0, NULL, 0}};
		camcmp_cmd_info_type cmd_get_scene_mode[] = {{0, NULL, 0, NULL, 0}};
		camcmp_cmd_info_type cmd_get_wdr_mode[] = {{0, NULL, 0, NULL, 0}};
        camcmp_blank_info_type set_wb = {0, NULL};

        CAMCMP_LOG_DBG("wb=%x \n", wb);

        
	    cmd_get_scene_mode[0].cmd_id = CAMCMP_CID_GET_SCENE_MODE;
	    cmd_get_scene_mode[0].p_recieve_data = &scene_mode_addr;
	    cmd_get_scene_mode[0].recieve_num = 1;

        
        rc = camcmp_isp_trans(cmd_get_scene_mode, CAMCMP_SIZE_CMD(cmd_get_scene_mode));

        if (rc < 0) {
            CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
            return -EFAULT;
        }

        
	    cmd_get_wdr_mode[0].cmd_id = CAMCMP_CID_GET_WDR;
	    cmd_get_wdr_mode[0].p_recieve_data = &wdr_mode_addr;
	    cmd_get_wdr_mode[0].recieve_num = 1;

        
        rc = camcmp_isp_trans(cmd_get_wdr_mode, CAMCMP_SIZE_CMD(cmd_get_wdr_mode));

        if (rc < 0) {
            CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
            return -EFAULT;
        }

        if(
           (((scene_mode_addr & 0x8000) >> 15) == 0x0000) && 
           (((scene_mode_addr & 0x00f0) >> 4) ==  0x0000) &&
           (((wdr_mode_addr & 0x0004) >> 2) !=  0x0001)   &&
           (((scene_mode_addr & 0x000f) ==  0x0000) || ((scene_mode_addr & 0x000f) ==  0x000A))
          )
        {
        
    	    
    	    if(wb == CAMCMP_BID_WB_AUTO){
				
				rc = i2c_write_modify( scene_mode_addr, CAMCMP_CID_SET_BESTSHOT, 
				                  g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_NORMAL].mask, 
				                  g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_NORMAL].data );

				rc = i2c_read_write_modify( CAMCMP_CID_GET_WHITEBALANCE,
				                            CAMCMP_CID_SET_WHITEBLANCE,
				                            g_mt9e311_wb_info[wb].mask,
				                            g_mt9e311_wb_info[wb].data );

     	    	if (rc < 0) {
     	    		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
					return -EFAULT;
				}

			}
			else{

				rc = i2c_read_write_modify( CAMCMP_CID_GET_WHITEBALANCE,
				                            CAMCMP_CID_SET_WHITEBLANCE,
				                            g_mt9e311_wb_info[CAMCMP_BID_WB_AUTO].mask,
				                            g_mt9e311_wb_info[CAMCMP_BID_WB_AUTO].data );

     	    	if (rc < 0) {
     	    		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
					return -EFAULT;
				}

				set_wb.blank_id = wb;

	        	cmd_info_manual[0].cmd_id = CAMCMP_CID_SET_SCENE_TEMP;

	        	cmd_info_manual[1].cmd_id = CAMCMP_CID_SET_SCENE_BV;

	        	cmd_info_manual[2].cmd_id = CAMCMP_CID_SET_BEACH_MIN_TEMP;
        		cmd_info_manual[2].p_blank_info = &set_wb;
        		cmd_info_manual[2].blank_num = CAMCMP_SIZE_BLKINF(set_wb);

	        	cmd_info_manual[3].cmd_id = CAMCMP_CID_SET_BEACH_MAX_TEMP;
        		cmd_info_manual[3].p_blank_info = &set_wb;
        		cmd_info_manual[3].blank_num = CAMCMP_SIZE_BLKINF(set_wb);

   				
				rc = camcmp_isp_trans(cmd_info_manual, CAMCMP_SIZE_CMD(cmd_info_manual));
     	    	if (rc < 0) {
     	    		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
					return -EFAULT;
				}

				rc = i2c_write_modify( scene_mode_addr, CAMCMP_CID_SET_BESTSHOT, 
				                  g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_BEACH].mask, 
				                  g_mt9e311_scene_info[CAMCMP_BID_SCENE_MODE_BEACH].data );

     	    	if (rc < 0) {
     	    		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
					return -EFAULT;
				}

			}
        }
        else{
        
			i2c_read_write_modify( CAMCMP_CID_GET_WHITEBALANCE,
								   CAMCMP_CID_SET_WHITEBLANCE,
								   g_mt9e311_wb_info[wb].mask,
								   g_mt9e311_wb_info[wb].data );
		}

    return rc;

}

















static int mt9e311_set_jpeg_quality(int8_t jpeg_quality)
{

































	return 0;
}
















static int mt9e311_set_pict_size( int pict_size )
{
	int32_t rc = 0;

	CAMCMP_LOG_DBG("start\n");

	CAMCMP_LOG_DBG("end rc = %d\n",rc);

	return rc;
}




















#define PREVCTL_ON(x)	(((~0x0002) & (x)) | (0x0002))		
#define PREVCTL_OFF(x)	(((~0x0002) & (x)) & (0xFFFD))		
#define SNAPCTL_ON(x)	(((~0x0001) & (x)) | (0x0001))		
#define SNAPCTL_OFF(x)	(((~0x0001) & (x)) & (0xFFFE))		

static int mt9e311_set_antishake( int8_t antishake )
{
	int rc = 0;
	camcmp_cmd_info_type cmd_info[]  = {{0, NULL, 0, NULL, 0},{0, NULL, 0, NULL, 0}};


	camcmp_blank_info_type set_prevctl = {0, NULL};
	camcmp_blank_info_type set_snapctl = {0, NULL};





    camcmp_cmd_info_type cmd_get_antishake[]  = {{0, NULL, 0, NULL, 0}};
    uint32_t antishake_addr[2];
    uint32_t w_recv_data_p = 0;
    uint32_t w_recv_data_s = 0;


	CAMCMP_LOG_DBG("antishake=%x \n", antishake);


    
	cmd_get_antishake[0].cmd_id = CAMCMP_CID_GET_ANTISHAKE_CTRL;
	cmd_get_antishake[0].p_recieve_data = &antishake_addr;
	cmd_get_antishake[0].recieve_num = 1;

    
    rc = camcmp_isp_trans(cmd_get_antishake, CAMCMP_SIZE_CMD(cmd_get_antishake));
    if (rc < 0) {
            CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
            return -EFAULT;
    }


	
	switch (antishake)
	{
		case CAMERA_ANTISHAKE_OFF:			

			w_recv_data_p = PREVCTL_OFF(antishake_addr[0]);
			w_recv_data_s = SNAPCTL_OFF(antishake_addr[1]);



			break;
		case CAMERA_ANTISHAKE_ON:			

			w_recv_data_p = PREVCTL_ON(antishake_addr[0]);
			w_recv_data_s = SNAPCTL_ON(antishake_addr[1]);



			break;
		default:
			CAMCMP_LOG_DBG("param error. default\n");
			return -EINVAL;
	}


    
	set_prevctl.p_blank_data    = &w_recv_data_p;
	cmd_info[0].cmd_id = CAMCMP_CID_SET_PREVCTL;
	cmd_info[0].p_blank_info = &set_prevctl;
	cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(set_prevctl);

    
	set_snapctl.p_blank_data    = &w_recv_data_s;
	cmd_info[1].cmd_id = CAMCMP_CID_SET_SNAPCTL;
	cmd_info[1].p_blank_info = &set_snapctl;
	cmd_info[1].blank_num = CAMCMP_SIZE_BLKINF(set_snapctl);


	
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}

	return rc;
}



















#define ISO_AUTO(x)	(((~0x00FF) & (x)) | (0x00C3))		
static int mt9e311_set_isotype(int8_t iso_type)
{
	int rc = 0;
	camcmp_cmd_info_type cmd_info[]  = {{0, NULL, 0, NULL, 0}};
    camcmp_cmd_info_type cmd_get_iso[]  = {{0, NULL, 0, NULL, 0}};
    camcmp_blank_info_type set_iso   = {0, NULL};
    uint32_t iso_addr = 0;
    uint32_t w_recv_data = 0;

	CAMCMP_LOG_DBG("iso_type=%x \n", iso_type);

    
	cmd_get_iso[0].cmd_id = CAMCMP_CID_GET_ISO;
	cmd_get_iso[0].p_recieve_data = &iso_addr;
	cmd_get_iso[0].recieve_num = 1;

    
    rc = camcmp_isp_trans(cmd_get_iso, CAMCMP_SIZE_CMD(cmd_get_iso));
    if (rc < 0) {
            CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
            return -EFAULT;
    }

    w_recv_data = ISO_AUTO(iso_addr);

	
	cmd_info[0].cmd_id = CAMCMP_CID_ISO;
	set_iso.p_blank_data    = &w_recv_data;
    cmd_info[0].p_blank_info = &set_iso;
    cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(set_iso);

	
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc != 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans error:%d\n", __LINE__);
		return -EFAULT;
	}

	return rc;
}



















static int mt9e311_set_antibanding( int8_t antibanding )
{
	int32_t rc = 0;

	camcmp_cmd_info_type cmd_info[]  = {{0, NULL, 0, NULL, 0}};
	camcmp_blank_info_type set_antibanding = {0, NULL};
	camcmp_blank_antibanding mode = (camcmp_blank_antibanding)antibanding;

	CAMCMP_LOG_DBG("start\n");

	CAMCMP_LOG_DBG("antibanding mode %d\n", antibanding);

	
	if ((mode < CAMCMP_BID_ANTIBANDING_OFF) ||
			(mode > CAMCMP_BID_ANTIBANDING_AUTO)) {
		CAMCMP_LOG_ERR("parameter error root:%d\n", __LINE__);
		return -EINVAL;
	}

	
	set_antibanding.blank_id = mode;

	cmd_info[0].cmd_id = CAMCMP_CID_ANTIBANDING;
	cmd_info[0].p_blank_info = &set_antibanding;
	cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(set_antibanding);

	
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}

	CAMCMP_LOG_DBG("end rc = %d\n",rc);

	return rc;
}

















static int mt9e311_set_parm_pm_obs( VE008_pm_obs_data_t pm_obs )
{
	int32_t rc = 0;
	pm_obs_mode_type mode = 0;
	bool sw = false;

	CAMCMP_LOG_DBG("start\n");

	CAMCMP_LOG_DBG("pm_obs mode = %d, sw = %d\n", pm_obs.mode, pm_obs.sw);

	
	switch (pm_obs.mode)
	{
		case CAMERA_VE028_MODE:			
			mode = PM_OBS_CAMERA_MODE;
			break;
		case CAMERA_PLAIED_MODE:			
			mode = PM_OBS_PLAIED_MODE;
			break;
		case CAMERA_HD_MODE:				
			mode = PM_OBS_CAMERA_HD_MODE;
			break;
		default:
			CAMCMP_LOG_DBG("param error. default\n");
			return -EINVAL;
	}
	if (pm_obs.sw == SENSOR_ON) {
		sw = true;
	} else {
		sw = false;
	}

	
	pm_obs_a_camera(mode, sw);
	
	CAMCMP_LOG_DBG("end\n");

	return rc;
}


























static int mt9e311_get_maker_note(struct get_exif_maker_note_cfg *get_maker_note)
{

camcmp_cmd_info_type cmd_info[] = 
{
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0},
   {0, NULL, 0, NULL, 0}
};


	uint32_t xml_ver = 0;                                              
	uint32_t device_id = 0;                                            
	uint32_t manufacturer_id = 0;                                      
	uint32_t current_mode = 0;                                         
	uint32_t revision_number = 0;                                      
	uint32_t calib_rg = 0;                                             
	uint32_t calib_bg = 0;                                             
	uint32_t af_inf_position = 0;                                      
	uint32_t af_macro_position = 0;                                    
	uint32_t fd_freq = 0;                                              
	uint32_t antishake = 0;                                            
	uint32_t snr = 0;                                                  
	uint32_t scene_mode = 0;                                           
	uint32_t denoise_value = 0;                                        
	uint32_t sharpen_value = 0;                                        
	uint32_t ae_exp_time = 0;                                          
	uint32_t ae_gain = 0;                                              
	uint32_t ae_bv = 0;                                                
	uint32_t ae_tv = 0;                                                
	uint32_t ae_gv = 0;                                                
	uint32_t ae_dv = 0;                                                
	uint32_t awb_temp = 0;                                             
	uint32_t awb_ggain = 0;                                            
	uint32_t awb_temp_offset = 0;                                      
	uint32_t awb_ggain_offset = 0;                                     
	uint32_t awb_saturation = 0;                                       
	uint32_t awb_speed = 0;                                            
	uint32_t awb_min_temp;                                             
	uint32_t awb_max_temp;                                             
	uint32_t af_pos = 0;                                               
	int32_t rc = 0;
	int max = 30;
    int count = 0;

struct cmd_setting_t {uint32_t id; void* p_rcv_data;} set_info[] =
{
   {CAMCMP_CID_GET_XML_VERSION,      (void*)&xml_ver             },    
   {CAMCMP_CID_GET_DEV_ID,           (void*)&device_id           },    
   {CAMCMP_CID_GET_MAN_ID,           (void*)&manufacturer_id     },    
   {CAMCMP_CID_GET_CURRENT_ID,       (void*)&current_mode        },    
   {CAMCMP_CID_GET_CHIP_VERSION,     (void*)&revision_number     },    
   {CAMCMP_CID_GET_CALIB_RG,         (void*)&calib_rg            },    
   {CAMCMP_CID_GET_CALIB_BG,         (void*)&calib_bg            },    
   {CAMCMP_CID_GET_AFINF_POS,        (void*)&af_inf_position     },    
   {CAMCMP_CID_GET_MACRO_POS,        (void*)&af_macro_position   },    
   {CAMCMP_CID_GET_FD_FREQ,          (void*)&fd_freq             },    
   {CAMCMP_CID_GET_ANTISHAKE,        (void*)&antishake           },    
   {CAMCMP_CID_GET_SNR,              (void*)&snr                 },    
   {CAMCMP_CID_GET_SCENE_MODE,       (void*)&scene_mode          },    
   {CAMCMP_CID_GET_DENOISE_VAL,      (void*)&denoise_value       },    
   {CAMCMP_CID_GET_SHARPEN_VAL,      (void*)&sharpen_value       },    
   {CAMCMP_CID_GET_AE_EXP_TIME,      (void*)&ae_exp_time         },    
   {CAMCMP_CID_GET_AE_GAIN,          (void*)&ae_gain             },    
   {CAMCMP_CID_GET_AE_BV,            (void*)&ae_bv               },    
   {CAMCMP_CID_GET_AE_TV,            (void*)&ae_tv               },    
   {CAMCMP_CID_GET_AE_GV,            (void*)&ae_gv               },    
   {CAMCMP_CID_GET_AE_DV,            (void*)&ae_dv               },    
   {CAMCMP_CID_GET_AWB_TEMP,         (void*)&awb_temp            },    
   {CAMCMP_CID_GET_AWB_GGAIN,        (void*)&awb_ggain           },    
   {CAMCMP_CID_GET_AWB_TEMPOFF,      (void*)&awb_temp_offset     },    
   {CAMCMP_CID_GET_AWB_GGAINOFF,     (void*)&awb_ggain_offset    },    
   {CAMCMP_CID_GET_AWB_SATURATION,   (void*)&awb_saturation      },    
   {CAMCMP_CID_GET_AWB_SPEED,        (void*)&awb_speed           },    
   {CAMCMP_CID_GET_AWB_MIN_TEMP,     (void*)&awb_min_temp        },    
   {CAMCMP_CID_GET_AWB_MAX_TEMP,     (void*)&awb_max_temp        },    
   {CAMCMP_CID_GET_LENSPOS,          (void*)&af_pos              },    
};

	CAMCMP_LOG_DBG("mt9e311_get_maker_note start\n");
	
	for(count = 0; count < max; count++){

    
		cmd_info[count].cmd_id = set_info[count].id;
		cmd_info[count].p_recieve_data = set_info[count].p_rcv_data;
		cmd_info[count].recieve_num = 1;

	}

	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));

	if (rc < 0) {
		CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
		return -EFAULT;
	}
	
	
    get_maker_note->rom_ver                 = (revision_number & 0xf000) >> 24;
    get_maker_note->xml_ver                 = xml_ver;                 
    get_maker_note->param_ver               = (revision_number & 0x0f00) >> 16;
    get_maker_note->device_id               = device_id;               
    get_maker_note->manufacturer_id         = manufacturer_id;         
    get_maker_note->current_mode            = current_mode;            
    get_maker_note->revision_number         = revision_number;         
    get_maker_note->calib_rg                = calib_rg;                
    get_maker_note->calib_bg                = calib_bg;                
    get_maker_note->af_inf_position         = af_inf_position;         
    get_maker_note->af_macro_position       = af_macro_position;       
    get_maker_note->fd_freq                 = (fd_freq & 0x00ff);      
    get_maker_note->antishake               = antishake;               
    get_maker_note->snr                     = snr;                     
    get_maker_note->scene_mode              = scene_mode;              
    get_maker_note->denoise_value           = denoise_value;           
    get_maker_note->sharpen_value           = sharpen_value;           
    get_maker_note->ae_exp_time             = ae_exp_time;             
    get_maker_note->ae_gain                 = ae_gain;                 
    get_maker_note->ae_bv                   = ae_bv;                   
    get_maker_note->ae_tv                   = ae_tv;                   
    get_maker_note->ae_gv                   = ae_gv;                   
    get_maker_note->ae_dv                   = ae_dv;                   
    get_maker_note->awb_temp                = awb_temp;                
    get_maker_note->awb_ggain               = awb_ggain;               
    get_maker_note->awb_temp_offset         = awb_temp_offset;         
    get_maker_note->awb_ggain_offset        = awb_ggain_offset;        
    get_maker_note->awb_saturation          = awb_saturation;          
    get_maker_note->awb_speed               = awb_speed;               
    get_maker_note->awb_min_temp            = awb_min_temp;            
    get_maker_note->awb_max_temp            = awb_max_temp;            
    get_maker_note->af_pos                  = af_pos;                  

	CAMCMP_LOG_DBG("mt9e311_get_maker_note end\n");
	return rc;

}




static int mt9e311_get_exif_param(struct get_exif_param_cfg* exif_param)
{
	int rc                  = 0;
	uint32_t sht_time       = 0;           
    int      iso_speed      = 0;           

	camcmp_cmd_info_type cmd_info1[] = {{0, NULL, 0, NULL, 0}};

	CAMCMP_LOG_DBG("mt9e311_get_exif_param start\n");
	
	
	
	
	
	cmd_info1[0].cmd_id = CAMCMP_CID_GET_AE_EXP_TIME;
	cmd_info1[0].p_recieve_data = &sht_time;
	cmd_info1[0].recieve_num = 1;

	rc = camcmp_isp_trans(cmd_info1, CAMCMP_SIZE_CMD(cmd_info1));
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans error rc = %d\n LINE = %d\n",
							rc, __LINE__);
		return -EFAULT;
	}else
	{
		
	}

	CAMCMP_LOG_DBG("sht_time =%x \n", sht_time);

	
	exif_param->shutter_speed = sht_time;

	
	
	
	
	cmd_info1[0].cmd_id = CAMCMP_CID_GET_ISO_SPEED;
	cmd_info1[0].p_recieve_data = &iso_speed;
	cmd_info1[0].recieve_num = 1;

	rc = camcmp_isp_trans(cmd_info1, CAMCMP_SIZE_CMD(cmd_info1));
	if (rc < 0)
	{
		CAMCMP_LOG_ERR("camcmp_isp_trans error rc = %d\n LINE = %d\n",
							rc, __LINE__);
		return -EFAULT;
	}
	else
	{
		
	}

	CAMCMP_LOG_DBG("iso_speed=%x \n", iso_speed);

	
	exif_param->iso_speed = iso_speed;

	CAMCMP_LOG_DBG("mt9e311_get_exif_param end\n");
	return rc;
}
























































 



















static void mt9e311_probe_init_done(const struct msm_camera_sensor_info *data)
{
	
	gpio_direction_output(MT9E311_GPIO_CAM_CLK_EN, 0);
	gpio_free(MT9E311_GPIO_CAM_CLK_EN);

	
	msleep(MT9E311_WAIT_5);

	
	gpio_direction_output(data->sensor_reset, 0);
	gpio_free(data->sensor_reset);

	
	msleep(MT9E311_WAIT_5);

	
	
	gpio_direction_output(MT9E311_GPIO_VDD_CAM_EN2, 0);
	gpio_free(MT9E311_GPIO_VDD_CAM_EN2);

	
	msleep(MT9E311_WAIT_5);

	
	gpio_direction_output(MT9E311_GPIO_VDD_CAM_EN3, 0);
	gpio_free(MT9E311_GPIO_VDD_CAM_EN3);



 
	
	msleep(MT9E311_WAIT_5);

	
	gpio_direction_output(MT9E311_GPIO_VDD_CAM_EN4, 0);
	gpio_free(MT9E311_GPIO_VDD_CAM_EN4);



	
	msleep(MT9E311_WAIT_5);

	
	gpio_direction_output(data->sensor_pwd, 0);
	gpio_free(data->sensor_pwd);

	
	msleep(MT9E311_WAIT_5);








 


}
















static int32_t mt9e311_setting(void)
{
	int32_t rc;
	camcmp_cmd_info_type cmd_info[] = {{0, NULL, 0, NULL, 0}};
	camcmp_cmd_info_type blk_cmd_info[] = {{0, NULL, 0, NULL, 0}};
	camcmp_cmd_info_type rev_cmd_info[] = {{0, NULL, 0, NULL, 0}};
	camcmp_blank_info_type blk_info  = {0, NULL};
	uint16_t lens_pos = 0;

	uint32_t w_addr = 0;      

	
	
	
	
	
	
	
	uint32_t scratchpad_lp = 0;
	uint32_t scratchpad_calib = 0xA4AC-0xA000;
	
	
	

	CAMCMP_LOG_DBG("start\n");
	CAMCMP_LOG_DBG("check device id\n");

	
	
	
	cmd_info[0].cmd_id = CAMCMP_CID_CHK_DEV_ID;
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc != 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans failed (%d)\n", rc);
		return -EFAULT;
	}

	CAMCMP_LOG_DBG("Setting high priority basic registers\n");

	
	
	
	if ( sys_rev == MT9E311_SYS_REVISION_02 ) {
		cmd_info[0].cmd_id = CAMCMP_CID_BASIC_INIT_HP_R2;
	}
	else {	
		cmd_info[0].cmd_id = CAMCMP_CID_BASIC_INIT_HP;
	}
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc != 0) {
		CAMCMP_LOG_ERR("Setting high priority basic registers failed (%d)\n", rc);
		return -EFAULT;
	}

	CAMCMP_LOG_DBG("Loading high priority patch\n");

	
	
	
	
	rev_cmd_info[0].cmd_id = CAMCMP_CID_GET_LOAD_P_ADDR;
	rev_cmd_info[0].p_recieve_data = &w_addr;
	rev_cmd_info[0].recieve_num    = 1;
	rc = camcmp_isp_trans(rev_cmd_info, CAMCMP_SIZE_CMD(rev_cmd_info));
	if (rc != 0) {
		CAMCMP_LOG_ERR("Loading high priority patch read address failed (%d)\n", rc);
		return -EFAULT;
	}

	
	blk_info.blank_id = CAMCMP_BID_PATCH_HP;
	blk_info.p_blank_data = &w_addr;
	if ( sys_rev == MT9E311_SYS_REVISION_02 ) {
		blk_cmd_info[0].cmd_id = CAMCMP_CID_PATCH_HP_R2;
	}
	else {	
		blk_cmd_info[0].cmd_id = CAMCMP_CID_PATCH_HP;
	}
	blk_cmd_info[0].p_blank_info = &blk_info;
	blk_cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(blk_info);
	rc = camcmp_isp_trans(blk_cmd_info, CAMCMP_SIZE_CMD(blk_cmd_info));
	if (rc != 0) {
		CAMCMP_LOG_ERR("Loading high priority patch failed (%d)\n", rc);
		return -EFAULT;
	}

	CAMCMP_LOG_DBG("Loading high priority scratchpad\n");

	
	
	
	
	rev_cmd_info[0].cmd_id = CAMCMP_CID_GET_LOAD_P_ADDR;
	rev_cmd_info[0].p_recieve_data = &w_addr;
	rev_cmd_info[0].recieve_num    = 1;
	rc = camcmp_isp_trans(rev_cmd_info, CAMCMP_SIZE_CMD(rev_cmd_info));
	if (rc != 0) {
		CAMCMP_LOG_ERR("Loading high priority scratchpad read address failed (%d)\n", rc);
		return -EFAULT;
	}

	
	blk_info.blank_id = CAMCMP_BID_SCRATCHPAD_HP;
	blk_info.p_blank_data = &w_addr;
	if ( sys_rev == MT9E311_SYS_REVISION_02 ) {
		blk_cmd_info[0].cmd_id = CAMCMP_CID_SCRATCHPAD_HP_R2;
	}
	else {	
		blk_cmd_info[0].cmd_id = CAMCMP_CID_SCRATCHPAD_HP;
	}
	blk_cmd_info[0].p_blank_info = &blk_info;
	blk_cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(blk_info);
	rc = camcmp_isp_trans(blk_cmd_info, CAMCMP_SIZE_CMD(blk_cmd_info));
	if (rc != 0) {
		CAMCMP_LOG_ERR("Loading high priority scratchpad failed (%d)\n", rc);
		return -EFAULT;
	}

	CAMCMP_LOG_DBG("start frame processing\n");

	
	
	
	
	if ( sys_rev == MT9E311_SYS_REVISION_02 ) {
		cmd_info[0].cmd_id = CAMCMP_CID_SYS_START_GO_R2;
	}
	else {	
		cmd_info[0].cmd_id = CAMCMP_CID_SYS_START_GO;
	}
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc != 0) {
		CAMCMP_LOG_ERR("start frame processing failed (%d)\n", rc);
		return -EFAULT;
	}

	CAMCMP_LOG_DBG("Loading low priority patch\n");

	
	
	
	
	rev_cmd_info[0].cmd_id = CAMCMP_CID_GET_PATCH_LP_ADDR;
	rev_cmd_info[0].p_recieve_data = &w_addr;
	rev_cmd_info[0].recieve_num    = 1;
	rc = camcmp_isp_trans(rev_cmd_info, CAMCMP_SIZE_CMD(rev_cmd_info));
	if (rc != 0) {
		CAMCMP_LOG_ERR("Loading low priority patch read address failed (%d)\n", rc);
		return -EFAULT;
	}

	
	blk_info.blank_id = CAMCMP_BID_PATCH_LP;
	blk_info.p_blank_data = &w_addr;
	if ( sys_rev == MT9E311_SYS_REVISION_02 ) {
		blk_cmd_info[0].cmd_id = CAMCMP_CID_PATCH_LP_R2;
	}
	else {	
		blk_cmd_info[0].cmd_id = CAMCMP_CID_PATCH_LP;
	}
	blk_cmd_info[0].p_blank_info = &blk_info;
	blk_cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(blk_info);
	rc = camcmp_isp_trans(blk_cmd_info, CAMCMP_SIZE_CMD(blk_cmd_info));
	if (rc != 0) {
		CAMCMP_LOG_ERR("Loading low priority patch failed (%d)\n", rc);
		return -EFAULT;
	}


	
	CAMCMP_LOG_DBG("Loading calibration scratchpad\n");

	if (sys_rev == MT9E311_SYS_REVISION_01) {
		
		
		
		
		rev_cmd_info[0].cmd_id = CAMCMP_CID_GET_LOAD_P_ADDR;
		rev_cmd_info[0].p_recieve_data = &w_addr;
		rev_cmd_info[0].recieve_num    = 1;
		rc = camcmp_isp_trans(rev_cmd_info, CAMCMP_SIZE_CMD(rev_cmd_info));
		if (rc != 0) {
			CAMCMP_LOG_ERR("Loading calibration scratchpad read address failed (%d)\n", rc);
			return -EFAULT;
		}

		







		w_addr += scratchpad_calib;

		
		blk_info.blank_id = CAMCMP_BID_SCRATCHPAD_CALIB;
		blk_info.p_blank_data = &w_addr;
		blk_cmd_info[0].cmd_id = CAMCMP_CID_SCRATCHPAD_CALIB;
		blk_cmd_info[0].p_blank_info = &blk_info;
		blk_cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(blk_info);
		rc = camcmp_isp_trans(blk_cmd_info, CAMCMP_SIZE_CMD(blk_cmd_info));
		if (rc != 0) {
			CAMCMP_LOG_ERR("Loading calibration scratchpad failed (%d)\n", rc);
			return -EFAULT;
		}

		CAMCMP_LOG_DBG("Loading low priority scratchpad\n");
	}

	
	
	
	
	rev_cmd_info[0].cmd_id = CAMCMP_CID_GET_LOAD_P_ADDR;
	rev_cmd_info[0].p_recieve_data = &w_addr;
	rev_cmd_info[0].recieve_num    = 1;
	rc = camcmp_isp_trans(rev_cmd_info, CAMCMP_SIZE_CMD(rev_cmd_info));
	if (rc != 0) {
		CAMCMP_LOG_ERR("Loading low priority scratchpad read address failed (%d)\n", rc);
		return -EFAULT;
	}

	
	
	
	
	
	
	
	if ( sys_rev == MT9E311_SYS_REVISION_02 ) {
		scratchpad_lp = 0xa9d0-0xa000;      
	}
	else {	
		scratchpad_lp = 0xA944-0xA000;
	}

	







	w_addr += scratchpad_lp;


	
	blk_info.blank_id = CAMCMP_BID_SCRATCHPAD_LP;
	blk_info.p_blank_data = &w_addr;
	if ( sys_rev == MT9E311_SYS_REVISION_02 ) {
		blk_cmd_info[0].cmd_id = CAMCMP_CID_SCRATCHPAD_LP_R2;
	}
	else {	
		blk_cmd_info[0].cmd_id = CAMCMP_CID_SCRATCHPAD_LP;
	}
	blk_cmd_info[0].p_blank_info = &blk_info;
	blk_cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(blk_info);
	rc = camcmp_isp_trans(blk_cmd_info, CAMCMP_SIZE_CMD(blk_cmd_info));
	if (rc != 0) {
		CAMCMP_LOG_ERR("Loading low priority scratchpad failed (%d)\n", rc);
		return -EFAULT;
	}

	CAMCMP_LOG_DBG("Setting calibration basic registers\n");

	if (sys_rev == MT9E311_SYS_REVISION_01) {
		
		
		
		cmd_info[0].cmd_id = CAMCMP_CID_BASIC_INIT_CALIB;
		rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
		if (rc != 0) {
			CAMCMP_LOG_ERR("Setting low priority basic registers failed (%d)\n", rc);
			return -EFAULT;
		}

		CAMCMP_LOG_DBG("Setting low priority basic registers\n");
	}

	
	
	
	if ( sys_rev == MT9E311_SYS_REVISION_02 ) {
		cmd_info[0].cmd_id = CAMCMP_CID_BASIC_INIT_LP_R2;
	}
	else {	
		cmd_info[0].cmd_id = CAMCMP_CID_BASIC_INIT_LP;
	}
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc != 0) {
		CAMCMP_LOG_ERR("Setting low priority basic registers failed (%d)\n", rc);
		return -EFAULT;
	}


	
	
	
	cmd_info[0].cmd_id = CAMCMP_CID_AF_IQ_S2_CTRL_LEN;
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc != 0) {
		CAMCMP_LOG_ERR("iq_s2_ctrl_len setting failed. rc = %d\n", rc);
		return -EFAULT;
	}

	
	
	
	if (sys_rev == MT9E311_SYS_REVISION_01) {
		
		rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE,
									CAMCMP_CID_AF_MODE,
									g_mt9e311_af_ctrl[CAMCMP_BID_MODE_MF].mask,
									g_mt9e311_af_ctrl[CAMCMP_BID_MODE_MF].data );
		if (rc != 0) {
			CAMCMP_LOG_ERR("af_mode setting failed. rc = %d\n", rc);
			return -EFAULT;
		}

		
		memset(cmd_info, 0, sizeof(cmd_info));

		cmd_info[0].cmd_id = CAMCMP_CID_GET_LENSPOS;
		cmd_info[0].p_recieve_data = &lens_pos;
		cmd_info[0].recieve_num = 1;
		
		rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
			return -EFAULT;
		}

		if( (lens_pos & 0x00FF) == g_mt9e311_sd_read[CAMCMP_PAN_NORMAL_REG].val ) {
			
			mt9e311_ctrl->af_mode = CAMERA_AF_MODE_NORMAL;
			CAMCMP_LOG_DBG("request mode is equal to existing mode. af_mode = %d", mt9e311_ctrl->af_mode );
		} else {
			
			mt9e311_ctrl->af_mode = CAMERA_AF_MODE_NO_USE;

			
			rc = mt9e311_set_af_pos(CAMERA_AF_MODE_NORMAL);
			if (rc < 0) {
				CAMCMP_LOG_ERR("mt9e311_set_af_pos failed. rc = %d\n", rc);
				return -EFAULT;
			}
		}
	} else if (sys_rev == MT9E311_SYS_REVISION_02) {
		
		memset(cmd_info, 0, sizeof(cmd_info));

		cmd_info[0].cmd_id = CAMCMP_CID_POLL_LOAD_OTP;
		
		rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
		if (rc < 0) {
			CAMCMP_LOG_ERR("load_otp pilling failed. rc = %d\n", rc);
			return -EFAULT;
		}

		
		rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE,
									CAMCMP_CID_AF_MODE,
									( g_mt9e311_af_mode[CAMCMP_BID_MODE_MF_R2].mask |
									  g_mt9e311_af_ctrl[CAMCMP_BID_MODE_MF_R2].mask ),
									( g_mt9e311_af_mode[CAMCMP_BID_MODE_MF_R2].data |
									  g_mt9e311_af_ctrl[CAMCMP_BID_MODE_MF_R2].data ) );
		if (rc != 0) {
			CAMCMP_LOG_ERR("af_mode setting failed. rc = %d\n", rc);
			return -EFAULT;
		}

		
		memset(cmd_info, 0, sizeof(cmd_info));
		memset(&blk_info, 0, sizeof(blk_info));
			
		cmd_info[0].cmd_id = CAMCMP_CID_AF_WINDOW_SET;
		
		rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
		if (rc < 0) {
			CAMCMP_LOG_ERR("af_window setting failed. rc = %d\n", rc);
			return -EFAULT;
		}

		
		memset(cmd_info, 0, sizeof(cmd_info));

		cmd_info[0].cmd_id = CAMCMP_CID_GET_LENSPOS;
		cmd_info[0].p_recieve_data = &lens_pos;
		cmd_info[0].recieve_num = 1;
		
		rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
		if (rc < 0) {
			CAMCMP_LOG_ERR("camcmp_isp_trans failed. rc = %d\n", rc);
			return -EFAULT;
		}

		if( (lens_pos & 0x00FF) == g_mt9e311_sd_read[CAMCMP_PAN_NORMAL_REG].val ) {
			
			mt9e311_ctrl->af_mode = CAMERA_AF_MODE_NORMAL;
			CAMCMP_LOG_DBG("request mode is equal to existing mode. af_mode = %d", mt9e311_ctrl->af_mode );
		} else {
			
			mt9e311_ctrl->af_mode = CAMERA_AF_MODE_NO_USE;

			
			rc = mt9e311_set_af_pos(CAMERA_AF_MODE_NORMAL);
			if (rc < 0) {
				CAMCMP_LOG_ERR("mt9e311_set_af_pos failed. rc = %d\n", rc);
				return -EFAULT;
			}
		}
	} else {
		CAMCMP_LOG_ERR("invalid system revision. sys_rev = 0x%04x\n", sys_rev);
		return -EFAULT;
	}

	CAMCMP_LOG_DBG("end\n");

	return 0;
}


















static int mt9e311_probe_init_sensor(const struct msm_camera_sensor_info *data)
{
	int32_t rc = 0;



















 


	
	rc = gpio_request(data->sensor_pwd, "mt9e311");
	if (!rc) {
		gpio_direction_output(data->sensor_pwd, 1);
	} else {
		
		mt9e311_probe_init_done(data);
		CAMCMP_LOG_ERR("sensor_pwd(%d) Error, rc = %d\n", data->sensor_pwd, rc);
		return -EFAULT;
	}



 
	
	msleep(MT9E311_WAIT_10);

	
	rc = gpio_request(MT9E311_GPIO_VDD_CAM_EN4, "mt9e311");
	if (!rc) {
		gpio_direction_output(MT9E311_GPIO_VDD_CAM_EN4, 1);
	} else {
		
		mt9e311_probe_init_done(data);
		CAMCMP_LOG_ERR("VDD_CAM_EN4 Error, rc = %d\n", rc);
		return -EFAULT;
	}



	
	msleep(MT9E311_WAIT_10 + MT9E311_WAIT_1);

	
	rc = gpio_request(MT9E311_GPIO_VDD_CAM_EN3, "mt9e311");
	if (!rc) {
		gpio_direction_output(MT9E311_GPIO_VDD_CAM_EN3, 1);
	} else {
		
		mt9e311_probe_init_done(data);
		CAMCMP_LOG_ERR("VDD_CAM_EN3 Error, rc = %d\n", rc);
		return -EFAULT;
	}

	
	msleep(MT9E311_WAIT_5 + MT9E311_WAIT_1);

	
	rc = gpio_request(MT9E311_GPIO_VDD_CAM_EN2, "mt9e311");
	if (!rc) {
		gpio_direction_output(MT9E311_GPIO_VDD_CAM_EN2, 1);
	} else {
		
		mt9e311_probe_init_done(data);
		CAMCMP_LOG_ERR("VDD_CAM_EN2 Error, rc = %d\n", rc);
		return -EFAULT;
	}

	
	msleep(MT9E311_WAIT_5 + MT9E311_WAIT_1);

	
	rc = gpio_request(MT9E311_GPIO_CAM_CLK_EN, "mt9e311");
	if (!rc) {
		gpio_direction_output(MT9E311_GPIO_CAM_CLK_EN, 1);
	} else {
		
		mt9e311_probe_init_done(data);
		CAMCMP_LOG_ERR("CAM_CLK_EN Error, rc = %d\n", rc);
		return -EFAULT;
	}

	
	msleep(MT9E311_WAIT_1+MT9E311_WAIT_1);

	
	rc = gpio_request(data->sensor_reset, "mt9e311");
	if (!rc) {
		gpio_direction_output(data->sensor_reset, 1);
	} else {
		
		mt9e311_probe_init_done(data);
		CAMCMP_LOG_ERR("sensor_reset(%d) Error, rc = %d\n", data->sensor_reset, rc);
		return -EFAULT;
	}

	 
	msleep(MT9E311_WAIT_1+MT9E311_WAIT_1);

	return 0;
}












































































































































int mt9e311_sensor_release(void)
{
	int rc = 0;

	mutex_lock(&mt9e311_mut);


    rc = mt9e311_poweroff();



































	CAMCMP_LOG_DBG("mt9e311_release completed\n");

	mutex_unlock(&mt9e311_mut);

	return rc;
}





















int mt9e311_sensor_open_init(const struct msm_camera_sensor_info *data)
{
	int32_t rc;
	camcmp_cmd_info_type cmd_info[] = {{0, NULL, 0, NULL, 0}};

	
	mt9e311_ctrl = (struct mt9e311_ctrl *)kzalloc(sizeof(struct mt9e311_ctrl), GFP_KERNEL);
	if (!mt9e311_ctrl) {
		CAMCMP_LOG_ERR("mt9e311_init failed!\n");
		kfree(mt9e311_ctrl);
		return -ENOMEM;
	}
	
	if (data) {
		mt9e311_ctrl->sensordata = data;
	}

	
	mt9e311_ctrl->sensormode = SENSOR_UNKNOWN_MODE;

	
	mt9e311_ctrl->prev_res = SENSOR_CAMERA_VGA_SIZE;

	
	mt9e311_ctrl->init_curr_lens_pos = 0;

	
	mt9e311_ctrl->af_cancel_onoff = SENSOR_AF_OFF;

	
	mt9e311_ctrl->af_exec_flag = SENSOR_OFF;

	
	mt9e311_ctrl->af_mode = CAMERA_AF_MODE_NO_USE;

	
	mt9e311_ctrl->ae_mode = CAMERA_AE_MODE_AUTO;
	mt9e311_ctrl->ae_mode_next = CAMERA_AE_MODE_AUTO;


	
	af_start_after_cnt =0;

	
	rc = camcmp_makecmd_init();
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_makecmd_init failed\n");
		if (mt9e311_ctrl) {
			kfree(mt9e311_ctrl);
			mt9e311_ctrl = NULL;
		}
		return -EFAULT;
	}

	
	CAMCMP_LOG_DBG("mt9e311_sensor_open_init: camera power on \n");
	rc = mt9e311_probe_init_sensor(data);
	if (rc < 0) {
		if (mt9e311_ctrl) {
			kfree(mt9e311_ctrl);
			mt9e311_ctrl = NULL;
		}
		return rc;
	}

	
	
	
	memset(cmd_info, 0, sizeof(cmd_info));

	cmd_info[0].cmd_id = CAMCMP_CID_GET_SYS_REVISION;
	cmd_info[0].p_recieve_data = &sys_rev;
	cmd_info[0].recieve_num = 1;

	
	rc = camcmp_isp_trans(cmd_info, CAMCMP_SIZE_CMD(cmd_info));
	if (rc < 0) {
		CAMCMP_LOG_ERR("Get System Revision failed. rc = %d\n", rc);
		if (mt9e311_ctrl) {
			kfree(mt9e311_ctrl);
			mt9e311_ctrl = NULL;
		}
		return -EFAULT;
	}

	
	
	
	if (sys_rev == MT9E311_SYS_REVISION_01) {
		g_mt9e311_sd_read[CAMCMP_PAN_NORMAL_REG].val = 0x32;
	}

	















	
	CAMCMP_LOG_DBG("mt9e311_sensor_open_init: camera initialize \n");
	rc = mt9e311_setting();
	if (rc < 0) {
		CAMCMP_LOG_ERR("mt9e311_setting failed. rc = %d\n", rc);
		mt9e311_sensor_release();
		return rc;
	}

	return rc;
}




















int mt9e311_sensor_config(void *__sync, void __user *argp)
{
	struct sensor_cfg_data cdata;
	struct msm_sync *sync = (struct msm_sync *) __sync;
	int rc = 0;
	int ret;

	CAMCMP_LOG_DBG("start here !!!\n");
	if (copy_from_user(&cdata,
			   (void *)argp, sizeof(struct sensor_cfg_data))) {
		return -EFAULT;
	}


    
	if(!mt9e311_ctrl)
	{
        CAMCMP_LOG_DBG("mt9e311 Control Buffer Noting!!\n");
        return 0;
    }


	mutex_lock(&mt9e311_mut);

	CAMCMP_LOG_DBG("cfgtype = %d\n", cdata.cfgtype);
	switch (cdata.cfgtype) {
	case CFG_SET_MODE:
	
		CAMCMP_LOG_DBG("CFG_SET_MODE\n");
		rc = mt9e311_set_mode(cdata.mode, cdata.rs);
		break;

	case  CFG_SET_AF_MODE:
		CAMCMP_LOG_DBG("CFG_SET_AF_MODE\n");
		rc = mt9e311_set_af_mode(sync, &(cdata.cfg.af_mode)); 
		if (copy_to_user((void *)argp, &cdata,
				 sizeof(struct sensor_cfg_data))) {
			rc = -EFAULT;
		}
		break;

	case CFG_GET_EEPROM_READ:
		rc = mt9e311_get_otp(&(cdata.cfg.get_otp));
		if (copy_to_user((void *)argp, &cdata,
				 sizeof(struct sensor_cfg_data))) {
			rc = -EFAULT;
		}
		break;
	case CFG_SENSOR_PAT_CHG:

		CAMCMP_LOG_DBG("CFG_SENSOR_PAT_CHG\n");
		rc = mt9e311_set_sensor_pat_chg(cdata.cfg.test_pat);
		break;


    case CFG_GET_AF_POS:
		CAMCMP_LOG_DBG("CFG_GET_AF_POS\n");
		rc = mt9e311_get_af_pos(&(cdata.cfg.get_af_pos));
		if (rc >= 0) {
			if (copy_to_user((void *)argp, &cdata,
					 sizeof(struct sensor_cfg_data))) {
				rc = -EFAULT;
			}
		}
		break;
    case CFG_GET_MONI_GAIN:
		CAMCMP_LOG_DBG("CFG_GET_MONI_GAIN\n");
		rc = mt9e311_get_moni_gain(&(cdata.cfg.get_monigain));
		if (rc >= 0) {
			if (copy_to_user((void *)argp, &cdata,
					 sizeof(struct sensor_cfg_data))) {
				rc = -EFAULT;
			}
		}
		break;
    case CFG_GET_REGVER:
		CAMCMP_LOG_DBG("CFG_GET_REGVER\n");
		rc = mt9e311_get_reg_ver(&(cdata.cfg.get_reg_ver));
		
		CAMCMP_LOG_ERR("cdata.cfg.get_reg_ver.chip_ver0 = %x\n", cdata.cfg.get_reg_ver.chip_ver0);
		CAMCMP_LOG_ERR("cdata.cfg.get_reg_ver.chip_ver1 = %x\n", cdata.cfg.get_reg_ver.chip_ver1);
		CAMCMP_LOG_ERR("cdata.cfg.get_reg_ver.xml_ver2 = %x\n", cdata.cfg.get_reg_ver.xml_ver2);
		CAMCMP_LOG_ERR("cdata.cfg.get_reg_ver.xml_ver3 = %x\n", cdata.cfg.get_reg_ver.xml_ver3);
		if (rc >= 0) {
			if (copy_to_user((void *)argp, &cdata,
					 sizeof(struct sensor_cfg_data))) {
				rc = -EFAULT;
			}
		}
		break;

	case CFG_SET_DEFAULT_FOCUS:
		CAMCMP_LOG_DBG("CFG_SET_DEFAULT_FOCUS\n");

		rc = mt9e311_af_stop();



		break;

	case CFG_AUTO_FOCUS:
		CAMCMP_LOG_DBG("CFG_AUTO_FOCUS\n");
		rc = mt9e311_af_start(sync); 
		break;

	case CFG_AUTO_FOCUS_AFTER:
		CAMCMP_LOG_DBG("CFG_AUTO_FOCUS_AFTER\n");
		rc = mt9e311_af_start_after(sync); 
		if( rc != AF_RESULT_DURING ){
			
			mt9e311_ctrl->af_exec_flag = SENSOR_OFF;
			if (rc != AF_RESULT_OK) {
				
				ret = mt9e311_af_stop();
				if (ret != 0) {
					CAMCMP_LOG_ERR("mt9e311_af_stop failed. ret = %d\n", ret);
					rc = -EFAULT;
				}
			}
		}
		break;

	case CFG_AUTO_FOCUS_CANCEL:
		CAMCMP_LOG_DBG("CFG_AUTO_FOCUS_CANCEL\n");
		rc = mt9e311_af_cancel();
		break;

	case CFG_SET_BRIGHTNESS:
		CAMCMP_LOG_DBG("CFG_SET_BRIGHTNESS\n");
		rc = mt9e311_set_brightness(cdata.cfg.brightness);
		break;

	case CFG_SET_EFFECT:
		CAMCMP_LOG_DBG("CFG_SET_EFFECT\n");
		rc = mt9e311_set_effect(cdata.cfg.effect);
		break;

    case CFG_SET_ZOOM:
		CAMCMP_LOG_DBG("CFG_SET_ZOOM\n");

		rc = mt9e311_set_zoom( &(cdata.cfg.set_zoom) , cdata.rs);



		break;

    case CFG_SET_PICTURE_MODE:
		CAMCMP_LOG_DBG("CFG_SET_PICTURE_MODE\n");
		rc = mt9e311_set_picture_mode( cdata.cfg.picture_mode );
		break;

	case CFG_SET_WB:
		CAMCMP_LOG_DBG("mt9e311_sensor_config CFG_SET_WB\n");
		rc = mt9e311_set_wb(cdata.cfg.wb);
		break;

	case CFG_SET_JPEG_QUALITY:
		CAMCMP_LOG_DBG("mt9e311_sensor_config CFG_SET_JPEG_QUALITY\n");
		rc = mt9e311_set_jpeg_quality(cdata.cfg.jpeg_quality);
		break;

    case CFG_SET_MANUAL_STEP:
		CAMCMP_LOG_DBG("CFG_SET_MANUAL_STEP\n");
		rc = mt9e311_set_manual_step(&(cdata.cfg.set_manual_step));
		if (rc >= 0) {
			if (copy_to_user((void *)argp, &cdata,
					 sizeof(struct sensor_cfg_data))) {
				rc = -EFAULT;
			}
		}
		break;

	case CFG_SET_PICTURE_SIZE:
		CAMCMP_LOG_DBG("CFG_SET_PICTURE_SIZE\n");
		rc = mt9e311_set_pict_size( cdata.rs );
		break;


	case CFG_SET_ANTISHAKE:
		CAMCMP_LOG_DBG("CFG_SET_ANTISHAKE\n");
		rc = mt9e311_set_antishake( cdata.cfg.antishake );
		break;



	case CFG_GET_ISP_ID:
		CAMCMP_LOG_DBG("CFG_GET_ISP_ID\n");
		rc = mt9e311_get_isp_id( &(cdata.cfg.get_isp_id) );
		if (copy_to_user((void *)argp, &cdata,
				 sizeof(struct sensor_cfg_data))) {
			rc = -EFAULT;
		}
		break;



	case CFG_ISP_PAT_CHG:
		CAMCMP_LOG_DBG("CFG_ISP_PAT_CHG\n");
		rc = mt9e311_set_isp_test_pat(cdata.cfg.test_pat);
		break;



	case CFG_SET_FORMAT_CHG:
		CAMCMP_LOG_DBG("CFG_SET_FORMAT_CHG\n");
		rc = mt9e311_set_format_chg(cdata.cfg.set_format_chg.format);
		break;



	case CFG_SET_AE:
		CAMCMP_LOG_DBG("mt9e311_sensor_config CFG_SET_AE\n");
		rc = mt9e311_set_ae(cdata.cfg.ae);
		break;



  case CFG_SET_ANTIBANDING:
		CAMCMP_LOG_DBG("CFG_SET_ANTIBANDING\n");
		rc = mt9e311_set_antibanding( cdata.cfg.antibanding );
		break;




	case CFG_GET_MAKER_NOTE:
		CAMCMP_LOG_DBG("CFG_GET_MAKER_NOTE\n");
		rc = mt9e311_get_maker_note( &(cdata.cfg.get_exif_maker_note) );
		if (copy_to_user((void *)argp, &cdata,
				 sizeof(struct sensor_cfg_data))) {
			rc = -EFAULT;
		}
		break;



	case CFG_SET_ISO:
		CAMCMP_LOG_DBG("CFG_SET_ISO\n");
		rc = mt9e311_set_isotype(cdata.cfg.iso_type);
		break; 



	
	case CFG_AFC_STOP:
		CAMCMP_LOG_DBG("CFG_AFC_STOP\n");
		rc = mt9e311_afc_stop();
		break;
	
	case CFG_AFC_START:
		CAMCMP_LOG_DBG("CFG_AFC_START\n");
		rc = mt9e311_afc_start();
		break;
	
    














  case CFG_SET_FACE_MODE:
		CAMCMP_LOG_DBG("CFG_SET_FACE_MODE\n");
		rc = mt9e311_set_face_mode( cdata.cfg.face );
		break;
  case CFG_GET_FACE_POS:
		CAMCMP_LOG_DBG("CFG_GET_FACE_POS\n");
		rc = mt9e311_get_face_pos( cdata.cfg.face_pos );
		if (copy_to_user((void *)argp, &cdata, sizeof(msm_camera_face_info_t))) {
			rc = -EFAULT;
		}
		break;



  case CFG_SET_PM_OBS:
		CAMCMP_LOG_DBG("CFG_SET_PM_OBS\n");
		rc = mt9e311_set_parm_pm_obs( cdata.cfg.pm_obs );
		break;

  case CFG_SET_ISP_SENSOR_WRITE:
		CAMCMP_LOG_DBG("CFG_SET_ISP_SENSOR_WRITE\n");
		rc = mt9e311_set_sensor_reg_write( cdata.cfg.isp_sensor_write_param );
		break;

  case CFG_SET_ISP_SENSOR_READ:
		CAMCMP_LOG_DBG("CFG_SET_ISP_SENSOR_READ\n");
		rc = mt9e311_set_sensor_reg_read( &(cdata.cfg.isp_sensor_read_param)  );
		if (copy_to_user((void *)argp, &cdata,
				 sizeof(struct sensor_cfg_data))) {
			rc = -EFAULT;
		}
		break;

    case    CFG_POWER_OFF   :
		CAMCMP_LOG_DBG("CFG_POWER_OFF\n");
		rc = mt9e311_poweroff();
        break;



	case CFG_GET_PARAM_EXIF:
		CAMCMP_LOG_DBG("CFG_GET_PARAM_EXIF\n");
		rc = mt9e311_get_exif_param(&(cdata.cfg.get_exif_param));
		if (rc >= 0) {
			if (copy_to_user((void *)argp, &cdata,
					 sizeof(struct sensor_cfg_data))) {
				rc = -EFAULT;
			}
		}
		break;


	default:
		rc = -EINVAL;
		break;
	}

	mutex_unlock(&mt9e311_mut);
	return rc;
}















static void mt9e311_init_client(struct i2c_client *client)
{
	
	init_waitqueue_head(&mt9e311_wait_queue);
}
















static int mt9e311_i2c_probe(struct i2c_client *client,
			     const struct i2c_device_id *id)
{
	int rc = 0;
	CAMCMP_LOG_DBG("mt9e311_probe called!\n");

	if (!i2c_check_functionality(client->adapter, I2C_FUNC_I2C)) {
		CAMCMP_LOG_ERR("i2c_check_functionality failed\n");
		rc = -EFAULT;
		goto probe_failure;
	}

	mt9e311_sensorw = (struct mt9e311_work *)kzalloc(sizeof(struct mt9e311_work), GFP_KERNEL);
	if (!mt9e311_sensorw) {
		CAMCMP_LOG_ERR("kzalloc failed.\n");
		rc = -ENOMEM;
		goto probe_failure;
	}

	i2c_set_clientdata(client, mt9e311_sensorw);
	mt9e311_init_client(client);
	mt9e311_client = client;
	
	
	rc = camcmp_i2c_init((void*)client);
	if (rc < 0) {
		CAMCMP_LOG_ERR("camcmp_i2c_init failed\n");
		rc = -EFAULT;
		goto probe_failure;
	}

	msleep(MT9E311_WAIT_50);

	CAMCMP_LOG_DBG("mt9e311_probe successed! rc = %d\n", rc);
	return rc;

probe_failure:
	CAMCMP_LOG_ERR("mt9e311_probe failed! rc = %d\n", rc);
	return rc;
}





static const struct i2c_device_id mt9e311_i2c_id[] = {
	{"mt9e311", 0},
	{}
};




static struct i2c_driver mt9e311_i2c_driver = {
	.id_table = mt9e311_i2c_id,
	.probe = mt9e311_i2c_probe,
	.remove = __exit_p(mt9e311_i2c_remove),
	.driver = {
		   .name = "mt9e311",
		   },
};
















static int mt9e311_sensor_probe(const struct msm_camera_sensor_info *info,
				struct msm_sensor_ctrl *ctrl)
{
	int rc = i2c_add_driver(&mt9e311_i2c_driver);
	if (rc < 0 || mt9e311_client == NULL) {
		rc = -ENOTSUPP;
		goto probe_done;
	}








	ctrl->s_init = mt9e311_sensor_open_init;	
	ctrl->s_release = mt9e311_sensor_release;	
	ctrl->s_config = NULL;						
	ctrl->s_config2 = mt9e311_sensor_config;	 

probe_done:
	CAMCMP_LOG_ERR("%s\n", __FILE__);
	return rc;
}














static int __mt9e311_probe(struct platform_device *pdev)
{
	return msm_camera_drv_start(pdev, mt9e311_sensor_probe);
}




static struct platform_driver msm_camera_driver = {
	.probe = __mt9e311_probe,
	.driver = {
		   .name = "msm_camera_mt9e311",
		   .owner = THIS_MODULE,
		   },
};













static int __init mt9e311_init(void)
{
	return platform_driver_register(&msm_camera_driver);
}



module_init(mt9e311_init);





















#define MODIFY_WRITE(x,y,z)	(((~(y)) & (x)) | (z))
static int i2c_read_write_modify( int r_cid, int w_cid, uint32_t mask, uint32_t data)
{
	int rc = 0;
	camcmp_cmd_info_type read_cmd_info[]  = {{0, NULL, 0, NULL, 0}};
    camcmp_cmd_info_type write_cmd_info[]  = {{0, NULL, 0, NULL, 0}};
    camcmp_blank_info_type set_blank_info   = {0, NULL};
    uint32_t read_value = 0;
    uint32_t write_value = 0;

    
	read_cmd_info[0].cmd_id = r_cid;
	read_cmd_info[0].p_recieve_data = &read_value;
	read_cmd_info[0].recieve_num = 1;

    rc = camcmp_isp_trans(read_cmd_info, CAMCMP_SIZE_CMD(read_cmd_info));

    if (rc < 0) {
            CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
            return -EFAULT;
    }

    write_value = MODIFY_WRITE(read_value, mask, data);

	
	write_cmd_info[0].cmd_id = w_cid;
	set_blank_info.p_blank_data    = &write_value;
    write_cmd_info[0].p_blank_info = &set_blank_info;
    write_cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(set_blank_info);

	rc = camcmp_isp_trans(write_cmd_info, CAMCMP_SIZE_CMD(write_cmd_info));
	if (rc != 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans error:%d\n", __LINE__);
		return -EFAULT;
	}

	return rc;
}




















static int i2c_write_modify( uint32_t read_data, int w_cid, uint32_t mask, uint32_t data )
{
	int rc = 0;
    camcmp_cmd_info_type write_cmd_info[]  = {{0, NULL, 0, NULL, 0}};
    camcmp_blank_info_type set_blank_info   = {0, NULL};
    uint32_t write_value = 0;

    write_value = MODIFY_WRITE(read_data, mask, data);

	
	write_cmd_info[0].cmd_id = w_cid;
	set_blank_info.p_blank_data    = &write_value;
    write_cmd_info[0].p_blank_info = &set_blank_info;
    write_cmd_info[0].blank_num = CAMCMP_SIZE_BLKINF(set_blank_info);

	rc = camcmp_isp_trans(write_cmd_info, CAMCMP_SIZE_CMD(write_cmd_info));
	if (rc != 0) {
		CAMCMP_LOG_ERR("camcmp_isp_trans error:%d\n", __LINE__);
		return -EFAULT;
	}

	return rc;
}
























































































































static uint16_t scene_mode_rd_to_set( uint16_t scene_mode )
{
	uint16_t ret_scene_mode=0;
	
	ret_scene_mode = scene_mode;
	switch( ret_scene_mode & 0x000F ) {
		case CAMERA_SCENE_MODE_CLOSEUP:
			ret_scene_mode = CAMERA_SCENE_MODE_FOOD;
			break;
		case CAMERA_SCENE_MODE_OFF:
			ret_scene_mode = CAMERA_SCENE_MODE_BACKLIGHT;
			break;
		default:
			
			ret_scene_mode = ( scene_mode & 0x000F );
			break;
	}
	if( ret_scene_mode != ( scene_mode & 0x000F ) ){
		CAMCMP_LOG_DBG("change read register value [%d] to [%d] \n", scene_mode, ret_scene_mode );
	}
	return ret_scene_mode;
}


















#define CAMCMP_COUNT_TBL_CMD(a)     (sizeof(a)/sizeof(write_cmd_setting_t))
static int32_t mt9e311_set_scene_tbl(int8_t scene)
{
    int rc = 0;
	int count_cmn = 0;
	int cmn_max = 8;
	int max_count = 0;
    int count = 0;

    camcmp_cmd_info_type cmd_info_cmn[]  = {{0, NULL, 0, NULL, 0},{0, NULL, 0, NULL, 0},
                                            {0, NULL, 0, NULL, 0},{0, NULL, 0, NULL, 0},
                                            {0, NULL, 0, NULL, 0},{0, NULL, 0, NULL, 0},
                                            {0, NULL, 0, NULL, 0},{0, NULL, 0, NULL, 0}};

    camcmp_cmd_info_type cmd_info[]  =      {{0, NULL, 0, NULL, 0},{0, NULL, 0, NULL, 0},
                                            {0, NULL, 0, NULL, 0},{0, NULL, 0, NULL, 0},
                                            {0, NULL, 0, NULL, 0},{0, NULL, 0, NULL, 0},
                                            {0, NULL, 0, NULL, 0},{0, NULL, 0, NULL, 0},
                                            {0, NULL, 0, NULL, 0},{0, NULL, 0, NULL, 0},
                                            {0, NULL, 0, NULL, 0}};


    camcmp_blank_info_type set_ae_upper_et[] = {{0, NULL}};
    camcmp_blank_info_type set_ae_max_et[] = {{0, NULL}};
    camcmp_blank_info_type set_ae_usg[] = {{0, NULL}};
    camcmp_blank_info_type set_ae_bv_min[] = {{0, NULL}};
    camcmp_blank_info_type set_atm_gain[] = {{0, NULL}};
    camcmp_blank_info_type set_atm_ppm[] = {{0, NULL}};
    camcmp_blank_info_type set_ae_bklight[] = {{0, NULL}};
    camcmp_blank_info_type set_sc_gshift[] = {{0, NULL}};

    write_cmd_setting_t set_info_common[] =
    {
       {CAMCMP_CID_SET_AE_UPPER_ET,       (void*)&set_ae_upper_et},  
       {CAMCMP_CID_SET_AE_MAX_ET,         (void*)&set_ae_max_et},    
       {CAMCMP_CID_SET_SNAPSHOT_AE_USG,   (void*)&set_ae_usg},       
       {CAMCMP_CID_SET_AE_BV_MIN,         (void*)&set_ae_bv_min},    
       {CAMCMP_CID_SET_ATM_GAIN,          (void*)&set_atm_gain},     
       {CAMCMP_CID_SET_ATM_PPM,           (void*)&set_atm_ppm},      
       {CAMCMP_CID_SET_AE_BACKLIGHT,      (void*)&set_ae_bklight},   
       {CAMCMP_CID_SET_ATM_SC_GSHIFT,     (void*)&set_sc_gshift},    
    };

    write_cmd_setting_t set_info_portrait[] =
    {
       {CAMCMP_CID_SET_SCENE_PORTRAIT_BV,   0        },    
       {CAMCMP_CID_SET_SCENE_SHARPEN_1,     0        },    
       {CAMCMP_CID_SET_SCENE_DENOISE_1,     0        },    
       {CAMCMP_CID_SET_SCENE_SATURATION_1,  0        },    
       {CAMCMP_CID_SET_SCENE_CONTRAST_1,   	0        },    
       {CAMCMP_CID_SET_SCENE_TEMP_OFF_1,    0        },    
       {CAMCMP_CID_SET_SCENE_BV_OFFSET_1,   0        },    
       {CAMCMP_CID_SET_SCENE_ISO,   		0        },    
    };

    write_cmd_setting_t set_info_landscape[] =
    {
       {CAMCMP_CID_SET_SCENE_PORTRAIT_BV, 				0        },    
       {CAMCMP_CID_SET_SCENE_LANDSCAPE_MIN_TEMP,    	0        },    
       {CAMCMP_CID_SET_SCENE_LANDSCAPE_MAX_TEMP,    	0        },    
       {CAMCMP_CID_SET_SCENE_LANDSCAPE_AF_POS_BOUND,    0        },    
       {CAMCMP_CID_SET_SCENE_SHAPEN_2,   				0        },    
       {CAMCMP_CID_SET_SCENE_DENOISE_2,     			0        },    
       {CAMCMP_CID_SET_SCENE_SATURATION_2,   			0        },    
       {CAMCMP_CID_SET_SCENE_CONTRAST_2,   				0        },    
       {CAMCMP_CID_SET_SCENE_TEMP_OFF_2,   				0        },    
       {CAMCMP_CID_SET_SCENE_BV_OFFSET_2,      			0        },    
       {CAMCMP_CID_SET_SCENE_ISO_2,   					0        },    
    };

    write_cmd_setting_t set_info_food[] =
    {
       {CAMCMP_CID_SET_SCENE_MACRO_AF_POS,          	0        },    
       {CAMCMP_CID_SET_SCENE_SHARPEN_4,             	0        },    
       {CAMCMP_CID_SET_SCENE_DENOISE_4,           		0        },    
       {CAMCMP_CID_SET_SCENE_SATURATION_4,              0        },    
       {CAMCMP_CID_SET_SCENE_CONTRAST_4,   				0        },    
       {CAMCMP_CID_SET_SCENE_TEMP_OFF_4,      			0        },    
       {CAMCMP_CID_SET_SCENE_BV_OFF_4,   				0        },    
       {CAMCMP_CID_SET_SCENE_ISO_4,   					0        },    
    };

    write_cmd_setting_t set_info_night[] =
    {
       {CAMCMP_CID_SET_SCENE_NIGHT_MAX_EXP_TIME,        0        },    
       {CAMCMP_CID_SET_SCENE_NIGHT_MIN_TEMP,            0        },    
       {CAMCMP_CID_SET_SCENE_NIGHT_MAX_TEMP,            0        },    
       {CAMCMP_CID_SET_SCENE_NIGHT_AF_POS_BOUND,        0        },    
       {CAMCMP_CID_SET_SCENE_SHAPEN_5,   				0        },    
       {CAMCMP_CID_SET_SCENE_DENOISE_5,      			0        },    
       {CAMCMP_CID_SET_SCENE_SATURATION_5,   			0        },    
       {CAMCMP_CID_SET_SCENE_CONTRAST_5,   				0        },    
       {CAMCMP_CID_SET_SCENE_TEMP_OFF_5,      			0        },    
       {CAMCMP_CID_SET_SCENE_BV_OFFSET_5,   			0        },    
       {CAMCMP_CID_SET_SCENE_ISO_5,   					0        },    
    };

    write_cmd_setting_t set_info_backlight[] =
    {
       {CAMCMP_CID_SET_SCENE_DENOISE_7,        			0        },    
    };

    write_cmd_setting_t set_info_party[] =
    {
       {CAMCMP_CID_SET_SCENE_SHARPEN_12,           		0        		   	},    
       {CAMCMP_CID_SET_SCENE_DENOISE_12,                0        			},    
       {CAMCMP_CID_SET_SCENE_SATURATION_12,   			0        			},    
       {CAMCMP_CID_SET_SCENE_CONTRAST_12,      			0        			},    
       {CAMCMP_CID_SET_SCENE_TEMP_OFF_12,   			0        			},    
       {CAMCMP_CID_SET_SCENE_BV_OFF_12,   				0        			},    
       {CAMCMP_CID_SET_SCENE_ISO_12,      				0        			},    
    };

    write_cmd_setting_t set_info_sport[] =
    {
       {CAMCMP_CID_SET_SCENE_SPORT_AF_POS_BOUND,        0        			},    
       {CAMCMP_CID_SET_SCENE_SHARPEN_3,           		0        			},    
       {CAMCMP_CID_SET_SCENE_DENOISE_3,                 0        			},    
       {CAMCMP_CID_SET_SCENE_SATURATION_3,   			0        			},    
       {CAMCMP_CID_SET_SCENE_CONTRAST_3,      			0        			},    
       {CAMCMP_CID_SET_SCENE_TEMP_OFF_3,   				0        			},    
       {CAMCMP_CID_SET_SCENE_BV_OFF_3,   				0        			},    
       {CAMCMP_CID_SET_SCENE_ISO_3,      				0        			},    
    };

	CAMCMP_LOG_DBG("mt9e311_set_scene_tbl\n");

    switch (scene)
    {
            case CAMERA_SCENE_MODE_OFF:                    

				max_count = CAMCMP_COUNT_TBL_CMD(set_info_common);
				set_ae_upper_et[0].blank_id = CAMCMP_BID_AE_UPPER_ET_OFF;
				set_ae_max_et[0].blank_id = CAMCMP_BID_AE_MAX_ET_OFF;
				set_ae_usg[0].blank_id = CAMCMP_BID_AE_USG_OFF;
				set_ae_bv_min[0].blank_id = CAMCMP_BID_AE_BV_OFF;
				set_atm_gain[0].blank_id = CAMCMP_BID_ATM_GAIN_OFF;
				set_atm_ppm[0].blank_id = CAMCMP_BID_ATM_PPM_OFF;
				set_ae_bklight[0].blank_id = CAMCMP_BID_AE_BKLIGHT_OFF;
				set_sc_gshift[0].blank_id = CAMCMP_BID_SC_GSHIFT_OFF;

    			
				for(count_cmn = 0; count_cmn < cmn_max; count_cmn++){
					cmd_info_cmn[count_cmn].cmd_id = set_info_common[count_cmn].id;
					cmd_info_cmn[count_cmn].p_blank_info = set_info_common[count_cmn].p_blank_data;
					cmd_info_cmn[count_cmn].blank_num = CAMCMP_SIZE_BLKINF(set_info_common[count_cmn]);
				}

    			
   			 	rc = camcmp_isp_trans(cmd_info_cmn, CAMCMP_SIZE_CMD(cmd_info_cmn));
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;

            case CAMERA_SCENE_MODE_PORTRAIT:                

				max_count = CAMCMP_COUNT_TBL_CMD(set_info_portrait);
				set_ae_upper_et[0].blank_id = CAMCMP_BID_AE_UPPER_ET_OFF;
				set_ae_max_et[0].blank_id = CAMCMP_BID_AE_MAX_ET_OFF;
				set_ae_usg[0].blank_id = CAMCMP_BID_AE_USG_OFF;
				set_ae_bv_min[0].blank_id = CAMCMP_BID_AE_BV_OFF;
				set_atm_gain[0].blank_id = CAMCMP_BID_ATM_GAIN_OFF;
				set_atm_ppm[0].blank_id = CAMCMP_BID_ATM_PPM_OFF;
				set_ae_bklight[0].blank_id = CAMCMP_BID_AE_BKLIGHT_OFF;
				set_sc_gshift[0].blank_id = CAMCMP_BID_SC_GSHIFT_OFF;

    			
				for(count_cmn = 0; count_cmn < cmn_max; count_cmn++){
					cmd_info_cmn[count_cmn].cmd_id = set_info_common[count_cmn].id;
					cmd_info_cmn[count_cmn].p_blank_info = set_info_common[count_cmn].p_blank_data;
					cmd_info_cmn[count_cmn].blank_num = CAMCMP_SIZE_BLKINF(set_info_common[count_cmn]);
				}

    			
   			 	rc = camcmp_isp_trans(cmd_info_cmn, CAMCMP_SIZE_CMD(cmd_info_cmn));
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}

    			
				for(count = 0; count < max_count; count++){
					cmd_info[count].cmd_id = set_info_portrait[count].id;
					if(set_info_portrait[count].p_blank_data != 0){
						cmd_info[count].p_blank_info = set_info_portrait[count].p_blank_data;
						cmd_info[count].blank_num = CAMCMP_SIZE_BLKINF(set_info_portrait[count]);
					}
				}

    			
   			 	rc = camcmp_isp_trans(cmd_info, max_count);
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;

            case CAMERA_SCENE_MODE_LANDSCAPE:               

				max_count = CAMCMP_COUNT_TBL_CMD(set_info_landscape);
				set_ae_upper_et[0].blank_id = CAMCMP_BID_AE_UPPER_ET_OFF;
				set_ae_max_et[0].blank_id = CAMCMP_BID_AE_MAX_ET_OFF;
				set_ae_usg[0].blank_id = CAMCMP_BID_AE_USG_OFF;
				set_ae_bv_min[0].blank_id = CAMCMP_BID_AE_BV_OFF;
				set_atm_gain[0].blank_id = CAMCMP_BID_ATM_GAIN_OFF;
				set_atm_ppm[0].blank_id = CAMCMP_BID_ATM_PPM_OFF;
				set_ae_bklight[0].blank_id = CAMCMP_BID_AE_BKLIGHT_OFF;
				set_sc_gshift[0].blank_id = CAMCMP_BID_SC_GSHIFT_OFF;

    			
				for(count_cmn = 0; count_cmn < cmn_max; count_cmn++){
					cmd_info_cmn[count_cmn].cmd_id = set_info_common[count_cmn].id;
					cmd_info_cmn[count_cmn].p_blank_info = set_info_common[count_cmn].p_blank_data;
					cmd_info_cmn[count_cmn].blank_num = CAMCMP_SIZE_BLKINF(set_info_common[count_cmn]);
				}

    			
   			 	rc = camcmp_isp_trans(cmd_info_cmn, CAMCMP_SIZE_CMD(cmd_info_cmn));
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}


    			
				for(count = 0; count < max_count; count++){
					cmd_info[count].cmd_id = set_info_landscape[count].id;
					if(set_info_landscape[count].p_blank_data != 0){
						cmd_info[count].p_blank_info = set_info_landscape[count].p_blank_data;
						cmd_info[count].blank_num = CAMCMP_SIZE_BLKINF(set_info_landscape[count]);
					}
				}

    			
   			 	rc = camcmp_isp_trans(cmd_info, max_count);
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;

            case CAMERA_SCENE_MODE_SPORTS:                  

				max_count = CAMCMP_COUNT_TBL_CMD(set_info_sport);
				set_ae_upper_et[0].blank_id = CAMCMP_BID_AE_UPPER_ET_SPORT;
				set_ae_max_et[0].blank_id = CAMCMP_BID_AE_MAX_ET_SPORT;
				set_ae_usg[0].blank_id = CAMCMP_BID_AE_USG_SPORT;
				set_ae_bv_min[0].blank_id = CAMCMP_BID_AE_BV_OFF;
				set_atm_gain[0].blank_id = CAMCMP_BID_ATM_GAIN_OFF;
				set_atm_ppm[0].blank_id = CAMCMP_BID_ATM_PPM_OFF;
				set_ae_bklight[0].blank_id = CAMCMP_BID_AE_BKLIGHT_OFF;
				set_sc_gshift[0].blank_id = CAMCMP_BID_SC_GSHIFT_OFF;

    			
				for(count_cmn = 0; count_cmn < cmn_max; count_cmn++){
					cmd_info_cmn[count_cmn].cmd_id = set_info_common[count_cmn].id;
					cmd_info_cmn[count_cmn].p_blank_info = set_info_common[count_cmn].p_blank_data;
					cmd_info_cmn[count_cmn].blank_num = CAMCMP_SIZE_BLKINF(set_info_common[count_cmn]);
				}

    			
   			 	rc = camcmp_isp_trans(cmd_info_cmn, CAMCMP_SIZE_CMD(cmd_info_cmn));
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}

    			
				for(count = 0; count < max_count; count++){
					cmd_info[count].cmd_id = set_info_sport[count].id;
					if(set_info_sport[count].p_blank_data != 0){
						cmd_info[count].p_blank_info = set_info_sport[count].p_blank_data;
						cmd_info[count].blank_num = CAMCMP_SIZE_BLKINF(set_info_sport[count]);
					}
				}

    			
   			 	rc = camcmp_isp_trans(cmd_info, max_count);
   			 	
                
				rc = i2c_read_write_modify( CAMCMP_CID_GET_WDR, CAMCMP_CID_SET_WDR, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_OFF].mask, 
				                            g_mt9e311_wdr_info[CAMCMP_BID_WDR_OFF].data );

   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;

            case CAMERA_SCENE_MODE_CLOSEUP:                 

                break;
            case CAMERA_SCENE_MODE_NIGHT:                   

				max_count = CAMCMP_COUNT_TBL_CMD(set_info_night);
				set_ae_upper_et[0].blank_id = CAMCMP_BID_AE_UPPER_ET_OFF;
				set_ae_max_et[0].blank_id = CAMCMP_BID_AE_MAX_ET_OFF;
				set_ae_usg[0].blank_id = CAMCMP_BID_AE_USG_OFF;
				set_ae_bv_min[0].blank_id = CAMCMP_BID_AE_BV_NIGHT;
				set_atm_gain[0].blank_id = CAMCMP_BID_ATM_GAIN_OFF;
				set_atm_ppm[0].blank_id = CAMCMP_BID_ATM_PPM_OFF;
				set_ae_bklight[0].blank_id = CAMCMP_BID_AE_BKLIGHT_OFF;
				set_sc_gshift[0].blank_id = CAMCMP_BID_SC_GSHIFT_OFF;

    			
				for(count_cmn = 0; count_cmn < cmn_max; count_cmn++){
					cmd_info_cmn[count_cmn].cmd_id = set_info_common[count_cmn].id;
					cmd_info_cmn[count_cmn].p_blank_info = set_info_common[count_cmn].p_blank_data;
					cmd_info_cmn[count_cmn].blank_num = CAMCMP_SIZE_BLKINF(set_info_common[count_cmn]);
				}

    			
   			 	rc = camcmp_isp_trans(cmd_info_cmn, CAMCMP_SIZE_CMD(cmd_info_cmn));
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}

    			
				for(count = 0; count < max_count; count++){
					cmd_info[count].cmd_id = set_info_night[count].id;
					if(set_info_night[count].p_blank_data != 0){
						cmd_info[count].p_blank_info = set_info_night[count].p_blank_data;
						cmd_info[count].blank_num = CAMCMP_SIZE_BLKINF(set_info_night[count]);
					}
				}

    			
   			 	rc = camcmp_isp_trans(cmd_info, max_count);
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;

            case CAMERA_SCENE_MODE_TWILIGHT:                

                break;
            case CAMERA_SCENE_MODE_BACKLIGHT:               
				max_count = CAMCMP_COUNT_TBL_CMD(set_info_backlight);
				set_ae_upper_et[0].blank_id = CAMCMP_BID_AE_UPPER_ET_OFF;
				set_ae_max_et[0].blank_id = CAMCMP_BID_AE_MAX_ET_OFF;
				set_ae_usg[0].blank_id = CAMCMP_BID_AE_USG_OFF;
				set_ae_bv_min[0].blank_id = CAMCMP_BID_AE_BV_OFF;
				set_atm_gain[0].blank_id = CAMCMP_BID_ATM_GAIN_BACKLIGHT;
				set_atm_ppm[0].blank_id = CAMCMP_BID_ATM_PPM_BACKLIGHT;
				set_ae_bklight[0].blank_id = CAMCMP_BID_AE_BKLIGHT_BACKLIGHT;
				set_sc_gshift[0].blank_id = CAMCMP_BID_SC_GSHIFT_BACKLIGHT;

    			
				for(count_cmn = 0; count_cmn < cmn_max; count_cmn++){
					cmd_info_cmn[count_cmn].cmd_id = set_info_common[count_cmn].id;
					cmd_info_cmn[count_cmn].p_blank_info = set_info_common[count_cmn].p_blank_data;
					cmd_info_cmn[count_cmn].blank_num = CAMCMP_SIZE_BLKINF(set_info_common[count_cmn]);
				}

    			
   			 	rc = camcmp_isp_trans(cmd_info_cmn, CAMCMP_SIZE_CMD(cmd_info_cmn));
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}

    			
				for(count = 0; count < max_count; count++){
					cmd_info[count].cmd_id = set_info_backlight[count].id;
					if(set_info_backlight[count].p_blank_data != 0){
						cmd_info[count].p_blank_info = set_info_backlight[count].p_blank_data;
						cmd_info[count].blank_num = CAMCMP_SIZE_BLKINF(set_info_backlight[count]);
					}
				}

    			
   			 	rc = camcmp_isp_trans(cmd_info, max_count);
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;
            case CAMERA_SCENE_MODE_HIGH_SENSITIVE_MODE:     

                break;
            case CAMERA_SCENE_MODE_NIGHT_PORTRAIT:          

                break;
            case CAMERA_SCENE_MODE_BEACH:                   

                break;
            case CAMERA_SCENE_MODE_FOOD:                   

				max_count = CAMCMP_COUNT_TBL_CMD(set_info_food);
				set_ae_upper_et[0].blank_id = CAMCMP_BID_AE_UPPER_ET_OFF;
				set_ae_max_et[0].blank_id = CAMCMP_BID_AE_MAX_ET_OFF;
				set_ae_usg[0].blank_id = CAMCMP_BID_AE_USG_OFF;
				set_ae_bv_min[0].blank_id = CAMCMP_BID_AE_BV_OFF;
				set_atm_gain[0].blank_id = CAMCMP_BID_ATM_GAIN_OFF;
				set_atm_ppm[0].blank_id = CAMCMP_BID_ATM_PPM_OFF;
				set_ae_bklight[0].blank_id = CAMCMP_BID_AE_BKLIGHT_OFF;
				set_sc_gshift[0].blank_id = CAMCMP_BID_SC_GSHIFT_OFF;

    			
				for(count_cmn = 0; count_cmn < cmn_max; count_cmn++){
					cmd_info_cmn[count_cmn].cmd_id = set_info_common[count_cmn].id;
					cmd_info_cmn[count_cmn].p_blank_info = set_info_common[count_cmn].p_blank_data;
					cmd_info_cmn[count_cmn].blank_num = CAMCMP_SIZE_BLKINF(set_info_common[count_cmn]);
				}

    			
   			 	rc = camcmp_isp_trans(cmd_info_cmn, CAMCMP_SIZE_CMD(cmd_info_cmn));
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}

    			
				for(count = 0; count < max_count; count++){
					cmd_info[count].cmd_id = set_info_food[count].id;
					if(set_info_food[count].p_blank_data != 0){
						cmd_info[count].p_blank_info = set_info_food[count].p_blank_data;
						cmd_info[count].blank_num = CAMCMP_SIZE_BLKINF(set_info_food[count]);
					}
				}

    			
   			 	rc = camcmp_isp_trans(cmd_info, max_count);
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;

            case CAMERA_SCENE_MODE_PARTY:                   

				max_count = CAMCMP_COUNT_TBL_CMD(set_info_party);
				set_ae_upper_et[0].blank_id = CAMCMP_BID_AE_UPPER_ET_PARTY;
				set_ae_max_et[0].blank_id = CAMCMP_BID_AE_MAX_ET_PARTY;
				set_ae_usg[0].blank_id = CAMCMP_BID_AE_USG_OFF;
				set_ae_bv_min[0].blank_id = CAMCMP_BID_AE_BV_PARTY;
				set_atm_gain[0].blank_id = CAMCMP_BID_ATM_GAIN_OFF;
				set_atm_ppm[0].blank_id = CAMCMP_BID_ATM_PPM_OFF;
				set_ae_bklight[0].blank_id = CAMCMP_BID_AE_BKLIGHT_OFF;
				set_sc_gshift[0].blank_id = CAMCMP_BID_SC_GSHIFT_OFF;

    			
				for(count_cmn = 0; count_cmn < cmn_max; count_cmn++){
					cmd_info_cmn[count_cmn].cmd_id = set_info_common[count_cmn].id;
					cmd_info_cmn[count_cmn].p_blank_info = set_info_common[count_cmn].p_blank_data;
					cmd_info_cmn[count_cmn].blank_num = CAMCMP_SIZE_BLKINF(set_info_common[count_cmn]);
				}

    			
   			 	rc = camcmp_isp_trans(cmd_info_cmn, CAMCMP_SIZE_CMD(cmd_info_cmn));
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}

    			
				for(count = 0; count < max_count; count++){
					cmd_info[count].cmd_id = set_info_party[count].id;
					if(set_info_party[count].p_blank_data != 0){
						cmd_info[count].p_blank_info = set_info_party[count].p_blank_data;
						cmd_info[count].blank_num = CAMCMP_SIZE_BLKINF(set_info_party[count]);
					}
				}

    			
   			 	rc = camcmp_isp_trans(cmd_info, max_count);
   			 	if (rc < 0) {
         			CAMCMP_LOG_ERR("error root:%d\n", __LINE__);
         	   		return -EFAULT;
  			  	}
                break;

            case CAMERA_SCENE_MODE_FIREWORKS:               

                break;
            case CAMERA_SCENE_MODE_SUNSET:                  

                break;
            default:
                    CAMCMP_LOG_DBG("param error. default\n");
                    return -EINVAL;
	}

    return rc;

}


static int mt9e311_poweroff(void)
{
	int32_t rc = 0;

    CAMCMP_LOG_DBG("%s start\n", __func__);
    
    
    if(mt9e311_ctrl)
    {
		if (sys_rev == MT9E311_SYS_REVISION_01) {
			
			rc = mt9e311_set_af_pos(CAMERA_AF_MODE_INFINITY);
			if (rc != 0) {
				CAMCMP_LOG_ERR("mt9e311_set_af_pos failed. rc = %d\n", rc);
			}
		} else if (sys_rev == MT9E311_SYS_REVISION_02) {
			
			rc = i2c_read_write_modify( CAMCMP_CID_GET_AF_STATE,
										CAMCMP_CID_AF_MODE,
										g_mt9e311_af_mode[CAMCMP_BID_MODE_MF_R2].mask,
										g_mt9e311_af_mode[CAMCMP_BID_MODE_MF_R2].data );
			if (rc != 0) {
				CAMCMP_LOG_ERR("af_mode setting failed. rc = %d\n", rc);
			}

			
			rc = mt9e311_set_af_pos(CAMERA_AF_MODE_INFINITY);
			if (rc != 0) {
				CAMCMP_LOG_ERR("mt9e311_set_af_pos failed. rc = %d\n", rc);
			}
		} else {
			CAMCMP_LOG_ERR("invalid system revision. sys_rev = 0x%04x\n", sys_rev);
		}

		mt9e311_probe_init_done(mt9e311_ctrl->sensordata);

		if (mt9e311_ctrl) {
			kfree(mt9e311_ctrl);
			mt9e311_ctrl = NULL;
		}
    }

    CAMCMP_LOG_DBG("%s completed\n", __func__);
    
    return 0;
}



